/*
 * ScenarioEditorPanel.java
 *
 * Created on 5  2004 ., 12:20
 */

package fi.helsinki.dacopan.animseq;

import java.awt.event.MouseEvent;

import fi.helsinki.dacopan.Localization;

/**
 * A panel representing the actual contents of the 'Scenario play list' dialog.
 */
public class ScenarioEditorPanel extends javax.swing.JPanel {
    
    private AnimationSequence animSeq;
    
    /** State flag to avoid circular event handling, used by refreshWidgets(). */
    private boolean isRefreshing = false;
    
    /** State flag to distinguish between meaningful valueChanged and mousePressed
     * events.
     */
    private int previousIndex = -1;
    
    /** Creates new form ScenarioEditorPanel */
    public ScenarioEditorPanel(AnimationSequence animSeq) {
        this.animSeq = animSeq;
        initComponents();
        setRecordingButtonsVisible(false);
        refreshWidgets();
    }
    
    /** This method is called from within the constructor to
     * initialize the form.
     * WARNING: Do NOT modify this code. The content of this method is
     * always regenerated by the Form Editor.
     */
    private void initComponents() {//GEN-BEGIN:initComponents
        java.awt.GridBagConstraints gridBagConstraints;

        jPanelPlayList = new javax.swing.JPanel();
        jScrollPane1 = new javax.swing.JScrollPane();
        jList1 = new javax.swing.JList();
        jToggleButtonRecordingButton = new javax.swing.JToggleButton();
        jButtonNextItem = new javax.swing.JButton();
        jButtonRecordStart = new javax.swing.JButton();
        jButtonRecordEnd = new javax.swing.JButton();
        jButtonRecordEnc = new javax.swing.JButton();
        jButtonDeleteItem = new javax.swing.JButton();
        jButtonEditItem = new javax.swing.JButton();
        jButtonRecordPause = new javax.swing.JButton();

        setLayout(new java.awt.GridBagLayout());

        jPanelPlayList.setLayout(new java.awt.BorderLayout());

        jPanelPlayList.setBorder(new javax.swing.border.LineBorder(new java.awt.Color(0, 0, 0)));
        jScrollPane1.setHorizontalScrollBarPolicy(javax.swing.JScrollPane.HORIZONTAL_SCROLLBAR_NEVER);
        jScrollPane1.setPreferredSize(new java.awt.Dimension(300, 180));
        jScrollPane1.setAutoscrolls(true);
        jList1.setSelectionMode(javax.swing.ListSelectionModel.SINGLE_SELECTION);
        jList1.setMaximumSize(null);
        jList1.setMinimumSize(null);
        jList1.setPreferredSize(null);
        jList1.addListSelectionListener(new javax.swing.event.ListSelectionListener() {
            public void valueChanged(javax.swing.event.ListSelectionEvent evt) {
                jList1ValueChanged(evt);
            }
        });
        jList1.addMouseListener(new java.awt.event.MouseAdapter() {
            public void mousePressed(java.awt.event.MouseEvent evt) {
                jList1MousePressed(evt);
            }
        });

        jScrollPane1.setViewportView(jList1);

        jPanelPlayList.add(jScrollPane1, java.awt.BorderLayout.CENTER);

        gridBagConstraints = new java.awt.GridBagConstraints();
        gridBagConstraints.gridwidth = 2;
        gridBagConstraints.fill = java.awt.GridBagConstraints.BOTH;
        gridBagConstraints.weightx = 1.0;
        gridBagConstraints.weighty = 1.0;
        add(jPanelPlayList, gridBagConstraints);

        jToggleButtonRecordingButton.setText(Localization.getString("animseq.recording_mode"));
        jToggleButtonRecordingButton.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                jToggleButtonRecordingButtonActionPerformed(evt);
            }
        });

        gridBagConstraints = new java.awt.GridBagConstraints();
        gridBagConstraints.gridx = 0;
        gridBagConstraints.gridy = 1;
        gridBagConstraints.insets = new java.awt.Insets(1, 1, 1, 1);
        add(jToggleButtonRecordingButton, gridBagConstraints);

        jButtonNextItem.setText(Localization.getString("animseq.next_item"));
        jButtonNextItem.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                jButtonNextItemActionPerformed(evt);
            }
        });

        gridBagConstraints = new java.awt.GridBagConstraints();
        gridBagConstraints.gridx = 1;
        gridBagConstraints.gridy = 1;
        add(jButtonNextItem, gridBagConstraints);

        jButtonRecordStart.setText(Localization.getString("animseq.record_start"));
        jButtonRecordStart.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                jButtonRecordStartActionPerformed(evt);
            }
        });

        gridBagConstraints = new java.awt.GridBagConstraints();
        gridBagConstraints.gridx = 0;
        gridBagConstraints.gridy = 3;
        gridBagConstraints.insets = new java.awt.Insets(1, 1, 1, 1);
        add(jButtonRecordStart, gridBagConstraints);

        jButtonRecordEnd.setText(Localization.getString("animseq.record_end"));
        jButtonRecordEnd.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                jButtonRecordEndActionPerformed(evt);
            }
        });

        gridBagConstraints = new java.awt.GridBagConstraints();
        gridBagConstraints.gridx = 0;
        gridBagConstraints.gridy = 4;
        gridBagConstraints.insets = new java.awt.Insets(1, 1, 1, 1);
        add(jButtonRecordEnd, gridBagConstraints);

        jButtonRecordEnc.setText(Localization.getString("animseq.record_encapsulation"));
        jButtonRecordEnc.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                jButtonRecordEncActionPerformed(evt);
            }
        });

        gridBagConstraints = new java.awt.GridBagConstraints();
        gridBagConstraints.gridx = 0;
        gridBagConstraints.gridy = 2;
        gridBagConstraints.insets = new java.awt.Insets(1, 1, 1, 1);
        add(jButtonRecordEnc, gridBagConstraints);

        jButtonDeleteItem.setText(Localization.getString("animseq.delete_item"));
        jButtonDeleteItem.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                jButtonDeleteItemActionPerformed(evt);
            }
        });

        gridBagConstraints = new java.awt.GridBagConstraints();
        gridBagConstraints.gridx = 1;
        gridBagConstraints.gridy = 3;
        add(jButtonDeleteItem, gridBagConstraints);

        jButtonEditItem.setText(Localization.getString("animseq.edit_item"));
        jButtonEditItem.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                jButtonEditItemActionPerformed(evt);
            }
        });

        gridBagConstraints = new java.awt.GridBagConstraints();
        gridBagConstraints.gridx = 1;
        gridBagConstraints.gridy = 2;
        add(jButtonEditItem, gridBagConstraints);

        jButtonRecordPause.setText(Localization.getString("animseq.record_pause"));
        jButtonRecordPause.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                jButtonRecordPauseActionPerformed(evt);
            }
        });

        gridBagConstraints = new java.awt.GridBagConstraints();
        gridBagConstraints.gridx = 1;
        gridBagConstraints.gridy = 4;
        add(jButtonRecordPause, gridBagConstraints);

    }//GEN-END:initComponents

    private void jButtonNextItemActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_jButtonNextItemActionPerformed
        animSeq.nextItemSelected();
    }//GEN-LAST:event_jButtonNextItemActionPerformed

    private void jButtonEditItemActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_jButtonEditItemActionPerformed
        animSeq.editItemSelected();
    }//GEN-LAST:event_jButtonEditItemActionPerformed

    private void jButtonRecordPauseActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_jButtonRecordPauseActionPerformed
        animSeq.recordPauseSelected();
    }//GEN-LAST:event_jButtonRecordPauseActionPerformed

    private void jButtonDeleteItemActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_jButtonDeleteItemActionPerformed
        animSeq.deleteItemSelected();
    }//GEN-LAST:event_jButtonDeleteItemActionPerformed

    private void jList1MousePressed(java.awt.event.MouseEvent evt) {//GEN-FIRST:event_jList1MousePressed
        if (evt.getButton() != MouseEvent.BUTTON1) return;
        if (previousIndex == jList1.getSelectedIndex()) {
            animSeq.showIndex(jList1.getSelectedIndex());
        }
    }//GEN-LAST:event_jList1MousePressed

    private void jButtonRecordEncActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_jButtonRecordEncActionPerformed
        animSeq.recordEncSelected();
    }//GEN-LAST:event_jButtonRecordEncActionPerformed

    private void jButtonRecordStartActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_jButtonRecordStartActionPerformed
        animSeq.recordStartSelected();
    }//GEN-LAST:event_jButtonRecordStartActionPerformed

    private void jButtonRecordEndActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_jButtonRecordEndActionPerformed
        animSeq.recordEndSelected();
    }//GEN-LAST:event_jButtonRecordEndActionPerformed

    private void jToggleButtonRecordingButtonActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_jToggleButtonRecordingButtonActionPerformed
        animSeq.setRecordingMode(jToggleButtonRecordingButton.isSelected());
        setRecordingButtonsVisible(jToggleButtonRecordingButton.isSelected());
        refreshWidgets();
    }//GEN-LAST:event_jToggleButtonRecordingButtonActionPerformed

    private void jList1ValueChanged(javax.swing.event.ListSelectionEvent evt) {//GEN-FIRST:event_jList1ValueChanged
        if (!evt.getValueIsAdjusting() && !isRefreshing) {
            animSeq.showIndex(jList1.getSelectedIndex());
            previousIndex = jList1.getSelectedIndex();
        }
    }//GEN-LAST:event_jList1ValueChanged
    
    
    // Variables declaration - do not modify//GEN-BEGIN:variables
    private javax.swing.JButton jButtonDeleteItem;
    private javax.swing.JButton jButtonEditItem;
    private javax.swing.JButton jButtonNextItem;
    private javax.swing.JButton jButtonRecordEnc;
    private javax.swing.JButton jButtonRecordEnd;
    private javax.swing.JButton jButtonRecordPause;
    private javax.swing.JButton jButtonRecordStart;
    private javax.swing.JList jList1;
    private javax.swing.JPanel jPanelPlayList;
    private javax.swing.JScrollPane jScrollPane1;
    private javax.swing.JToggleButton jToggleButtonRecordingButton;
    // End of variables declaration//GEN-END:variables

    public void refreshWidgets() {
        isRefreshing = true;
        jList1.setListData(animSeq.getPlaylist());
        jList1.setSelectedIndex(animSeq.getIndexShown());
        jScrollPane1.setViewportView(jList1);
        jButtonRecordEnd.setEnabled(animSeq.isRecordEndPossible());
        jButtonRecordStart.setEnabled(animSeq.isRecordStartPossible());
        jButtonRecordEnc.setEnabled(animSeq.isRecordEncPossible());
        jButtonRecordPause.setEnabled(animSeq.isRecordPausePossible());
        jButtonNextItem.setEnabled(animSeq.isNextItemPossible());
        jButtonEditItem.setEnabled(animSeq.isEditItemPossible());
        isRefreshing = false;
    }
    
    public void setRecordingButtonsVisible(boolean enabled) {
        jButtonRecordStart.setVisible(enabled);
        jButtonRecordEnd.setVisible(enabled);
        jButtonRecordEnc.setVisible(enabled);
        jButtonDeleteItem.setVisible(enabled);
        jButtonEditItem.setVisible(enabled);
        jButtonRecordPause.setVisible(enabled);
    }

	/**
	 * Changes the setEnabled-option of record start and end.
	 */
	public void setStartAndEndEnability() {
		jButtonRecordStart.setEnabled(animSeq.isRecordStartPossible());
		jButtonRecordEnd.setEnabled(animSeq.isRecordEndPossible());
	}
    
}
