/*
 * ScenarioItemMSC.java
 *
 * Created on 5  2004 ., 12:42
 */

package fi.helsinki.dacopan.animseq;

import java.text.NumberFormat;

import fi.helsinki.dacopan.Localization;
import fi.helsinki.dacopan.ui.MainFrame;
import fi.helsinki.dacopan.ui.SettingsMSC;

/**
 * A list item for an MSC animation.
 *
 * <p>An MSC item in the list contains a SettingsMSC instance that in turn contains all information
 * necessary for presenting the MSC animation. Particularly, unlike in explore mode, now the
 * SettingsMSC object will also store a start time and an end time for this scenario item.
 */
public class ScenarioItemMSC 
    implements ScenarioItem
{
    
    private SettingsMSC settings;
    
    
    /** Creates a new instance of ScenarioItemMSC.
     *
     * <p><b>NOTE:</b> The ScenarioItemMSC will store a clone of the original settings object.
     */
    public ScenarioItemMSC(SettingsMSC settings, float startTime) {
        this.settings = (SettingsMSC)settings.clone();
        this.settings.setStartTime(startTime);
    }
    
    /** Instructs the MainFrame instance to display the MSC animation. If the SettingsMSC object
     * for this item return true from getScenarioAutoPlay(), the AnimatioTimeState instance is
     * also put to play mode.
     */
    public void doAction(MainFrame mainFrame, boolean inRecordingMode) {
        mainFrame.setupAnimationModeMSC(settings, true);
        if (!inRecordingMode && settings.getScenarioAutoPlay()) {
            mainFrame.getAnimationTimeState().play();
        }
    }
    
    public String getPlaylistText() {
        NumberFormat numberFormat = NumberFormat.getInstance(Localization.getCurrentLocale());
        numberFormat.setMaximumFractionDigits(4);
        numberFormat.setMinimumFractionDigits(4);
        
        StringBuffer sb = new StringBuffer(Localization.getString("animseq.item_msc")); //$NON-NLS-1$
        if (settings.getStartTime() >= 0f) {
            sb.append(" "+numberFormat.format(settings.getStartTime()));
        }
        sb.append("->");
        if (settings.getEndTime() > 0f) {
            sb.append(numberFormat.format(settings.getEndTime()));
        } else {
            sb.append("*");
        }
        sb.append(" ["+settings.getLayer().getName()+"]");
        sb.append(" {1/"+settings.getScaleMultiplier()+"}");
        if (settings.getScenarioAutoPlay()) {
            sb.append(Localization.getString("animseq.item_msc_auto_play")); //$NON-NLS-1$
        }
        
        return sb.toString();
    }
    
    /** Tries to record an end time and returns the result. */
    public boolean recordEnd(float nowTime) {
        try {
            settings.setEndTime(nowTime);
            return true;
        } catch (Exception e) {
            return false;
        }
    }    
    
    /** Always returns true. */
    public boolean getShowable() {
        return true;
    }
    
}
