/*
 * Created on 8.4.2004
 *
 */
package fi.helsinki.dacopan.contsig;

/** 
 * Describes the events related to the time state of the animation
 * that all animation panels should react to by changing their
 * visible state. In any view mode, there is exactly one
 * <code>AnimationTimeState</code> instance that manages the
 * time state and sends related events to all visible animation
 * panels. 
 * 
 * <p>It is the responsibility of the <code>MainFrame</code>
 * to register all visible animation panels as listeners to the
 * <code>AnimationTimeState</code>
 * 
 * @author vainio
 * @see fi.helsinki.dacopan.contsig.AnimationTimeState
 * @see fi.helsinki.dacopan.ui.MainFrame
 */
public interface ControlSignalsListener {

	/** 
	 * Signals the listener that it should visualise the network
	 * exchange data at the <code>nowTime</code> point in time,
	 * possibly as an incremental change to previous state. The
	 * change is given as <code>step</code> parameter.
	 * 
	 * <p>The listener is free to implement the <code>advance</code> 
	 * signal as if it was the same thing as receiving a <code>stepTo</code>
	 * signal.
	 * 
	 * <p>It is the responsibility of the listening object to call
	 * <code>repaint()</code> on itself as a result of receiving
	 * this signal.
	 * 
	 * @param step The incremental time step from the previous state.
	 * 
	 * @param nowTime The point of time in network exchange data
	 * the listener should visualize.
	 * 
	 * @author Vainio
	 */
	public void advance(float step, float nowTime);
	
	/** 
	 * Signals the listener that it should visualise the network
	 * exchange data at the <code>nowTime</code> point in time.
	 * 
	 * <p>It is the responsibility of the listening object to call
	 * <code>repaint()</code> on itself as a result of receiving
	 * this signal.
	 * 
	 * @param nowTime The point of time in network exchange data
	 * the listener should visualize.
	 * 
	 * @author Vainio
	 */
	public void stepTo(float nowTime);
	
	/** 
	 * Signals the listener that the animation has moved to PLAY mode.
	 * There are no strict responsibilities for the listener to react to
	 * this in any way. 
	 * 
	 * @author Vainio
	 */
	public void toPlayMode();
	
	/** Signals the listener that the animation has moved to PAUSE mode.
	 * There are no strict responsibilities for the listener to react to
	 * this in any way. 
	 * 
	 * @author Vainio
	 */
	public void toPauseMode();
  

}
