package fi.helsinki.dacopan.model;

import fi.helsinki.dacopan.Localization;

/**
 *
 * This class represents a single textual note. If it is an ENC
 * note, it is "attached" to a TransferUnit, if it's an MSC note,
 * it's "attached" to a specific time value. In both cases it is
 * stored in the NoteManager's hash maps.
 */
public class Note implements Comparable {
    
    public static final String NULL_ENC_NOTE_TEXT = Localization.getString("model.note.null_enc_note"); //$NON-NLS-1$
    
    private String text;
    private float time;
    private Layer layer;
    private TransferUnit transferUnit;
    
    /**
     * This constructor builds a note for MSC
     *
     * @param layer
     * @param time
     * @param text
     */
    public Note(Layer layer, float time, String text) {
        super();
        this.text = text;
        this.layer = layer;
        this.time = time;
        this.transferUnit = null;
    }
    
    /**
     * This constructor makes a note to use with encapsulation
     * @param transferUnit
     * @param time
     * @param text
     */
    public Note(TransferUnit transferUnit, String text) {
        super();
        this.text = text;
        this.layer = null;
        this.time = -1;
        this.transferUnit = transferUnit;
    }
    
    public void setText(String text){
        this.text = text;
    }
    
    /**
     * @return Returns the time.
     */
    public float getTime() {
        return time;
    }
    
    /**
     * @return Returns the text.
     */
    public String getText() {
        return text;
    }
    
    /**
     * @return Returns the layer.
     */
    public Layer getLayer() {
        return layer;
    }
    
    /**
     * @return Returns the transferUnit.
     */
    public TransferUnit getTransferUnit() {
        return transferUnit;
    }
    
    /**
     * We compare Notes by comparing their times.
     */
    public int compareTo(Object o) {
        return Float.compare(this.time,((Note)o).time);
    }
    
    public boolean equals(Object o) {
        if(!(o instanceof Note)) return false;
        Note other = (Note)o;
        if(text != null ? !text.equals(other.text) : other.text != null) return false;
        if(layer != null ? !layer.equals(other.layer) : other.layer != null) return false;
        if(transferUnit != null ? !transferUnit.equals(other.transferUnit) : other.transferUnit != null) return false;
        if(other.time != time) return false;
        return true;
    }
    
    public String toString() {
        return text + " / " + time;
    }
    
}

