package fi.helsinki.dacopan.model;

import java.util.LinkedList;
import java.util.List;

import fi.helsinki.dacopan.Localization;

/**
 * A network protocol.
 * 
 * @version $Id: Protocol.java,v 1.10 2005/05/02 11:18:59 mjvaanan Exp $
 */
public class Protocol implements Identifiable {
    
    /** unique identifier */
    protected long id;
    
    /** name of the protocol */
    protected String name;
    
    /** network layer to which the protocol belongs to */
    protected Layer layer;
    
    /** list of {@link VariableDefinition variable definitions} */
    protected List variables = new LinkedList();
    
    /** list of {@link StaticVariable static variables} */
    protected List staticVariables = new LinkedList();
    
    /**
     * Creates a protocol with the given id, name and network layer.
     */
    public Protocol(long id, String name, Layer layer) {
        if(name == null || layer == null) {
            throw new IllegalArgumentException("Name and layer must be specified");
        }
        this.id = id;
        this.name = name;
        this.layer = layer;
    }

    /**
     * Creates a protocol with the given values and variables.
     */
    public Protocol(long id, String name, Layer layer, List variables, List staticVariables) {
        this(id, name, layer);
        setVariables(variables);
        setStaticVariables(staticVariables);
    }

    public long getId() {
        return id;
    }
    
    public Layer getLayer() {
        return layer;
    }

    public String getName() {
        return Localization.getString("model.protocol." + name);
    }
    
    public String getRawName() {
    	return name;
    }
    
    /**
     * Returns any variable definitions associated with this protocol.
     */
    public List getVariables() {
        return variables;
    }
    
    public void setVariables(List variables) {
        if(variables == null) {
            throw new IllegalArgumentException("Variables must be non-null");
        }
        this.variables = variables;
    }

    /**
     * Returns any static variables associated with this protocol.
     */
    public List getStaticVariables() {
        return staticVariables;
    }
    
    public void setStaticVariables(List staticVariables) {
        if(staticVariables == null) {
            throw new IllegalArgumentException("Static variables must be non-null");
        }
        this.staticVariables = staticVariables;
    }

    public boolean equals(Object o) {
        if(this == o) return true;
        if(!(o instanceof Protocol)) return false;
        Protocol other = (Protocol)o;
        if(other.id != id) return false;
        if(!other.name.equals(name)) return false;
        if(!other.layer.equals(layer)) return false;
        if(!other.variables.equals(variables)) return false;
        if(!other.staticVariables.equals(staticVariables)) return false;
        return true;
    }
    
    // TODO: hashCode()
    
    public String toString() {
        return "Protocol {" + id + "/" + name + "/" + layer + "/" + variables + "/" + staticVariables + "}";
    }
    
}
