package fi.helsinki.dacopan.model;

/**
 * An iterator for stepping thru an animation sequence.
 * 
 * @see DataView#getStepIterator(Layer)
 * @version $Id: StepIterator.java,v 1.2 2004/04/26 17:57:19 vainio Exp $
 */
public interface StepIterator {

    /**
     * Returns true if the iterator can step forward.
     */
    boolean hasNext();

    /**
     * Returns true if the iterator can step back.
     */
    boolean hasPrevious();

    /**
     * Steps forward.
     * 
     * @throws IllegalStateException if at end
     */
    float next();

    /**
     * Steps back.
     * 
     * @throws IllegalStateException if at beginning
     */
    float previous();

    /**
     * Steps to beginning of the sequence.
     */
    float first();

    /**
     * Steps to end of the sequence.
     */
    float last();
    
    /**
     * Returns the current step.
     */
    float current();
    
    /**
     * Sets the iterator to the nearest time that is <i>after</i> the time given as parameter.
     * If given time is after the last possible step, the iterator is set to the last step.
     *
     * @return The new iterator value.
     */
    float getNextForTime(float time);
    
    /**
     * Sets the iterator to the nearest time that is <i>before</i> the time given as parameter.
     * If given time is before the first possible step, the iterator is set to the first step.
     *
     * @return The new iterator value.
     */
    float getPreviousForTime(float time);
    
}
