/*
 * Created on 23.2.2005
 *
 */
package fi.helsinki.dacopan.ui;

import java.util.ArrayList;
import java.util.Collections;
import java.util.Iterator;
import java.util.LinkedHashMap;
import java.util.List;
import java.util.ListIterator;
import java.util.Map;

import fi.helsinki.dacopan.model.DataView;
import fi.helsinki.dacopan.model.TransferUnit;

/**
 *
 * Calculates proper y-coordinates for events based on max and min stepping and visual scale.
 * Mainly used by MSC and related classes.
 * 
 */

public class CalcYCoord {
	
	private int maxStepping, minStepping, maxGap;
	
	private List events;
	private final SettingsMSC settings;
	
	// Well, maybe these will accelerate this a litle.
	private Map times = new LinkedHashMap();
	
	public CalcYCoord(DataView dataView, SettingsMSC settings) {
			TransferUnit unit;
			List units = dataView.getUnitsForLayer(settings.getLayer());
			Iterator unitIterator = units.iterator();
			
			events = new ArrayList();
			this.settings = settings;
			this.maxStepping = settings.getMaxStepping();
			this.minStepping = settings.getMinStepping();
			this.maxGap = maxStepping;
			
			
			// Copy timestamps to events.
			EventClass event;
			while (unitIterator.hasNext()) {
				unit = (TransferUnit)unitIterator.next();
				if (unit.isOneUnit()) {
					event = new EventClass(unit.getSendStart(), unit.getId());
					event.setSendStart(true);
					events.add(event);
					event = new EventClass(unit.getReceiveEnd(), unit.getId());
					event.setReceiveEnd(true);
					events.add(event);
				} else {
					event = new EventClass(unit.getSendStart(), unit.getId());
					event.setSendStart(true);
					event.setFatUnit(true);
					events.add(event);
					
					event = new EventClass(unit.getSendEnd(), unit.getId());
					event.setSendEnd(true);
					event.setFatUnit(true);
					events.add(event);
					
					event = new EventClass(unit.getReceiveStart(), unit.getId());
					event.setReceiveStart(true);
					event.setFatUnit(true);
					events.add(event);
					
					event = new EventClass(unit.getReceiveEnd(), unit.getId());
					event.setReceiveEnd(true);
					event.setFatUnit(true);
					events.add(event);
				}
			}
			Collections.sort(events);
			
			if (!events.isEmpty()) {
				if (((EventClass)events.get(0)).getTime() != 0.0f) {
					event = new EventClass(0.0f, getUniqueId(events));
					events.add(0, event);
				}
				if (((EventClass)events.get(events.size()-1)).getTime() != dataView.getEndTime())
					events.add(new EventClass(dataView.getEndTime(), getUniqueId(events)));
			}
			
			this.calculateYCoord();
	}

	private long getUniqueId(List events) {
		// TODO Too trivial implementation
		boolean isUnique = true;
		long id = 0;
		do {
			id--;
			isUnique = true;
			ListIterator iter = events.listIterator();
			while (iter.hasNext() && isUnique) {
				isUnique = (id != ((EventClass)iter.next()).getId());
			}
		} while (!isUnique);
		return id;
	}
	
	/**
	 * Calculates the y-coordinates. Implements the pseudo-code from design document.
	 * Should be run always when something related to y-coordinates changes (i.e. max or min
	 * stepping or visual scale).
	 */
	
	private void calculateYCoord() {
		ListIterator iterator = events.listIterator();
		ListIterator iterator_2;
		EventClass e, event;
		float firstTime;
		int tempY;
		
		// Calculate original Y-coordinates
		// If we have no events, we do not need to do anything.
		if (events.isEmpty())
			return;
		firstTime = ((EventClass)(events.get(0))).getTime();
		while (iterator.hasNext()) {
			e = (EventClass)iterator.next();
			e.setYCoord((int)((e.getTime() - firstTime) * settings.getVisualScale()));
		}
		
		
		if (!settings.isLinearMode()) {
			checkForMaximumDistance();
			checkForMinimumDistance();
			checkForMaximumGap();
		}
		
		// Add values to hash map.
		iterator = events.listIterator();
		while (iterator.hasNext()) {
			event = (EventClass)iterator.next();
			times.put(new Float(event.getTime()), event);
		}
	 	// All should be done now
	 	// Debug check
	 	/*
		System.out.println("minStepping = " + minStepping +" maxStepping = " + maxStepping 
			+ " maxGap = " + maxGap);
	 	iterator = events.listIterator();
	 	while (iterator.hasNext()) {
	 		e = (EventClass)iterator.next();
	 		System.out.println("id = "+e.getId()+", time = "+e.getTime()+", y-koord. = "+e.getYCoord());
	 	}
	 	*/
	}

	/**
	 * 
	 */
	private void checkForMaximumDistance() {
		ListIterator iterator, iterator_2;
		EventClass e, event;
		int tempY;
		
		// Check for maximum distance
		// TODO Might not work correctly... check
		iterator = events.listIterator();
		while (iterator.hasNext()) {

			e = (EventClass)iterator.next();
			iterator_2 = events.listIterator(iterator.nextIndex());
			if (iterator_2.hasNext()) { // We are not in the end of the list
				/* For fat units we have to check send[Start|End] and receive[Start|End] pairs
				 * instead of sendStart-receiveEnd. */
				if (e.isFatUnit()) {
					event = (EventClass)iterator_2.next();
					if (e.isReceiveStart()) {// Were looking at receiveEnd
						while (iterator_2.hasNext() && event.getId() != e.getId() 
								&& !event.isReceiveEnd()) {
							event = (EventClass)iterator_2.next();
						}
						if (e.getId() == event.getId() && event.isReceiveEnd() 
							&& event.getYCoord() - e.getYCoord() > maxStepping) {
							event.setYCoord(e.getYCoord() + maxStepping);
							// Move all previous events so that they are before this event!
							tempY = event.getYCoord();
							event = (EventClass)iterator_2.previous();
							while (event.getId() != e.getId()) {
								if (event.getYCoord() > tempY) {
									event.setYCoord(tempY);						
								}
								event = (EventClass)iterator_2.previous();
							}
						}
					} else if (e.isSendStart()) { // Were looking at sendEnd
						while (iterator_2.hasNext() && event.getId() != e.getId() && !event.isSendEnd()) {
							event = (EventClass)iterator_2.next();
						}
						if (e.getId() == event.getId() && event.isSendEnd() 
							&& event.getYCoord() - e.getYCoord() > maxStepping) {
							event.setYCoord(e.getYCoord() + maxStepping);
							// Move all previous events so that they are before this event!
							tempY = event.getYCoord();
							event = (EventClass)iterator_2.previous();
							while (event.getId() != e.getId()) {
								if (event.getYCoord() > tempY) {
									event.setYCoord(tempY);						
								}
								event = (EventClass)iterator_2.previous();
							}
						}						
					}
				} else {
					event = (EventClass)iterator_2.next();
					while (iterator_2.hasNext() && event.getId() != e.getId()) {
						event = (EventClass)iterator_2.next();
					}
		
					if (e.getId() == event.getId() 
							&& Math.abs(event.getYCoord() - e.getYCoord()) > maxStepping) {
						event.setYCoord(e.getYCoord() + maxStepping);
						// Move all previous events so that they are before this event!
						tempY = event.getYCoord();
						iterator_2.previous();
						event = (EventClass)iterator_2.previous();
						while (event.getId() != e.getId()) {
							if (event.getYCoord() > tempY) {
								event.setYCoord(tempY);						
							}
							event = (EventClass)iterator_2.previous();
						}
					}
				}
			}
		}

		
	}

	/**
	 * 
	 */
	private void checkForMinimumDistance() {
		ListIterator iterator;
		EventClass event, e;
		
		// Check for minimum distance
		iterator = events.listIterator();
		while (iterator.hasNext()) {
			e = (EventClass)iterator.next();
			if (iterator.hasNext()) {
				event = (EventClass)iterator.next();
				iterator.previous();
				if (e.getYCoord() > event.getYCoord()) {
					// Previous event has been transfered later than this event.
					event.setYCoord(e.getYCoord());
				}
				if (Math.abs(e.getYCoord() - event.getYCoord()) <= minStepping  
						&& e.getId() != event.getId()) {
					event.setYCoord(e.getYCoord() + minStepping);
				}
				else if (e.getId() == event.getId() 
						&& event.getYCoord() < e.getYCoord()) {
					event.setYCoord(e.getYCoord());
				}
			}
		}

		
	}

	/**
	 * 
	 */
	private void checkForMaximumGap() {
		ListIterator iterator, iterator_2;
		EventClass e, event;
		// Check for too long gaps.
		int gap;
		EventClass myEvent;
		iterator = events.listIterator();
		while (iterator.hasNext()) {
			e = (EventClass)iterator.next();
			if (iterator.hasNext()) {
				event = (EventClass)iterator.next();
				iterator.previous();
				if (event.getYCoord() - e.getYCoord() > maxGap) {
					// Some gap is too long.
					gap = event.getYCoord() - e.getYCoord() - maxGap;
					iterator_2 = events.listIterator(iterator.nextIndex());
					while (iterator_2.hasNext()) {
						myEvent = (EventClass)iterator_2.next();
						myEvent.setYCoord(myEvent.getYCoord() - gap);					
					}
				}
			}
		}
		
	}

	/**
	 * Getter for y-coordinate for given time. Y-coordinate either exactly the wanted (if 
	 * time == some event time) or lineary interpolated from nearest y-coordinates for events.
	 * 
	 * @param time
	 * @return y-coord for given time
	 */
	
	public int getYCoordForTime(float time) {
		ListIterator iterator = events.listIterator();
		int yCoord;
		EventClass event, preEvent;
		float prevTime, nextTime, percentOfTime;
		
		if (events.isEmpty()) 
			return 0;
		event = (EventClass)times.get(new Float(time));
		if (event != null) {
			return event.getYCoord();
		} else {
			// Map does not contain this time...
			event = (EventClass)iterator.next();
			while (iterator.hasNext() && event.getTime() < time) {
				event = (EventClass)iterator.next();
			}
			if (!iterator.hasNext()) {
				return event.getYCoord();
			} else {
				preEvent = (EventClass)iterator.previous();
				if (!iterator.hasPrevious()) {
					return 0;
				}
				preEvent = (EventClass)iterator.previous();
				prevTime = preEvent.getTime();
				nextTime = event.getTime();
				percentOfTime = (time - prevTime) / (nextTime - prevTime);
				return (int)(preEvent.getYCoord() + percentOfTime
						* (event.getYCoord() - preEvent.getYCoord()));
			}
		}
	}

	/**
	 * Getter for linear y-coord for given time, i.e. the ratio time/total time is equal to the
	 * ratio y-coord / greatest y-coord.
	 * 
	 * @param time
	 * @return y-coord for given time
	 */
	public int getLinearYCoordForTime(float time) {
		if (events.isEmpty()) 
			return 0;
		EventClass first = (EventClass)events.get(0);
		EventClass last = (EventClass)events.get(events.size()-1);
		float timeIntervall = last.getTime() - first.getTime();
		
		return (int)(last.getYCoord() * time / timeIntervall);
	}
	
	/**
	 * Getter for last y-coord in this layer.
	 * @return last y-coord
	 */
	
	public int getLastYCoord() {
		return (events.isEmpty()) ? 0 : ((EventClass)events.get(events.size()-1)).getYCoord();
	}
	
	/**
	 * Getter for first y-coord in this layer, normaly 0.
	 * @return firs y-coord
	 */
	
	public int getFirstYCoord() {
		return (events.isEmpty()) ? 0 : ((EventClass)events.get(0)).getYCoord();
	}
	
	// TODO use hash map.
	/**
	 * Getter for time for given y-coord. Time is calculated vice versa as in 
	 * getYCoordForTime.
	 * @param value
	 * @return time
	 */
	public float getTimeForYCoord(int yCoord) {
		ListIterator iterator = events.listIterator();
		int time;
		EventClass event, preEvent;
		int prevYCoord, nextYCoord; 
		float percentOfYCoord;
		
		if (events.isEmpty())
			return 0.0f;
		event = (EventClass)iterator.next();
		while (iterator.hasNext() && event.getYCoord() < yCoord) {
			event = (EventClass)iterator.next();
		}
		if (!iterator.hasNext()) { // time is more than in any event (excluding the last event)!
			return event.getTime();
		} else {
			preEvent = (EventClass)iterator.previous();
			if (!iterator.hasPrevious()) {
				return 0.0f; // We're at start
			}
			preEvent = (EventClass)iterator.previous();
			if (event.getYCoord() == yCoord) {
				return event.getTime();
			} else { // Should return something in between, interpolate
				prevYCoord = preEvent.getYCoord();
				nextYCoord = event.getYCoord();
				percentOfYCoord = ((float)yCoord - (float)prevYCoord) / 
					((float)nextYCoord - (float)prevYCoord);
				return preEvent.getTime() + percentOfYCoord * (event.getTime() - preEvent.getTime());
			}
		}
	}
	
	/**
	 * Getter for linear time for given y-coord. Time is calculated similary as in 
	 * getLinearYCoordForTime.
	 * @param yCoord
	 * @return time
	 */
	public float getLinearTimeForYCoord(int yCoord) {
		if (events.isEmpty())
			return 0.0f;
		EventClass first = (EventClass)events.get(0);
		EventClass last = (EventClass)events.get(events.size()-1);
		int yIntervall = last.getYCoord() - first.getYCoord();
		return last.getTime()  * (float)yCoord / (float)yIntervall;
	}

	/**
	 * Setter for maximum stepping. Sets also maxGap to be greater than maxStepping. 
	 * Re-calculates y-coordinates.
	 * @param maxStepping
	 */
	public void setMaxStepping(int maxStepping) {
		this.maxStepping = maxStepping;
		if (maxGap > this.maxStepping)
			maxGap = this.maxStepping + 10; // maxGap must be greater than maxStepping
		calculateYCoord();
	}
	
	/**
	 * Setter for minimum stepping. Re-calculates y-coordinates.
	 * @param minStepping
	 * @author pamietti
	 *
	 */
	public void setMinStepping(int minStepping) {
		this.minStepping = minStepping;
		calculateYCoord();
	}
	
	/**
	 * Setter for maximum gap. Re-calculates y-coordinates.
	 * @param maxGap
	 */
	public void setMaxGap(int maxGap) {
		this.maxGap = maxGap;
		calculateYCoord();
	}
	
	private class EventClass implements Comparable {

		private int yCoord;
		private float time;
		private long id;
		private boolean fatUnit = false;
		private boolean sendStart = false;
		private boolean sendEnd = false;
		private boolean receiveStart = false;
		private boolean receiveEnd = false;
		
		public EventClass() {
			this.time = 0.0f;
			this.yCoord = 0;
		}
		
		public EventClass(float time, long id) {
			this.time = time;
			this.yCoord = 0;
			this.id = id;
		}
		/* (non-Javadoc)
		 * @see java.lang.Comparable#compareTo(java.lang.Object)
		 */
		public int compareTo(Object o) {
			EventClass other = (EventClass)o;
			return Float.compare(time, other.getTime());
		}
		
		public boolean equals(EventClass e) {
			return (this.time == e.getTime() && this.yCoord == e.getYCoord() && this.id == e.getId());
		}
		
		public float getTime() {
			return time;
		}
		
		public int getYCoord() {
				return yCoord;
		}
		
		public long getId() {
			return id;
		}
		
		public void setTime(float time) {
			this.time = time;
		}
		
		public void setYCoord(int yCoord) {
			this.yCoord = yCoord;
		}
		
		public void setFatUnit(boolean fatUnit) {
			this.fatUnit = fatUnit;
		}
		
		public void setSendStart(boolean sendStart) {
			this.sendStart = sendStart;
		}
		
		public void setSendEnd(boolean sendEnd) {
			this.sendEnd = sendEnd;
		}
		
		public void setReceiveStart(boolean receiveStart) {
			this.receiveStart = receiveStart;
		}
		
		public void setReceiveEnd(boolean receiveEnd) {
			this.receiveEnd = receiveEnd;
		}
		
		public boolean isFatUnit() {
			return fatUnit;
		}
		
		public boolean isSendStart() {
			return sendStart;
		}
		
		public boolean isSendEnd() {
			return sendEnd;
		}
		
		public boolean isReceiveStart() {
			return receiveStart;
		}
		
		public boolean isReceiveEnd() {
			return receiveEnd;
		}
	}
	
}
