package fi.helsinki.dacopan.ui;

import java.awt.BorderLayout;
import java.awt.Container;
import java.awt.Frame;

import javax.swing.JDialog;
import javax.swing.JLabel;
import javax.swing.JProgressBar;
import javax.swing.SwingUtilities;

/**
 * Progress indicator based on ProgressMonitor, a standard Swing class.
 * 
 * @version $Id: DialogProgressIndicator.java,v 1.2 2005/04/14 19:58:30 korkki Exp $
 */
public class DialogProgressIndicator implements ProgressIndicator {
    
    private boolean disposed = false;
    private float step;
    private float progress;
    private JDialog dialog;
    private JProgressBar bar;
    private JLabel text;
    
    public DialogProgressIndicator(Frame owner, String title) {
        dialog = new JDialog(owner, title, true);
        text = new JLabel();
        bar = new JProgressBar(0, 100);
        bar.setStringPainted(true);
        Container c = dialog.getContentPane();
        c.setLayout(new BorderLayout(5, 5));
        c.add(text, BorderLayout.NORTH);
        c.add(bar, BorderLayout.CENTER);
        dialog.setSize(300, 100);
        dialog.setLocationRelativeTo(owner);
    }
    
    public void show() {
        if(!disposed) {
            dialog.setVisible(true);
        }
    }
    
    public void close() {
        dialog.setVisible(false);
        dialog.dispose();
        disposed = true;
    }
    
    public void setMessage(final String message) {
        SwingUtilities.invokeLater(new Runnable() {
            public void run() {
                text.setText(message);
            }
        });
    }

    public void setProgress(final float progress) {
        this.progress = progress;
        SwingUtilities.invokeLater(new Runnable() {
            public void run() {
                int p = (int)progress;
                bar.setValue(p);
                bar.setString(p + "%");
            }
        });
        if(progress >= 100.0f) {
            close();
        }
    }

    public void setStep(float step) {
        this.step = step;
    }
    
    public void step() {
        this.progress += step;
        setProgress(progress);
    }
    
}
