
package fi.helsinki.dacopan.ui;

import java.awt.BorderLayout;
import java.awt.GridBagConstraints;
import java.awt.GridBagLayout;
import java.awt.Insets;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.awt.event.ItemEvent;
import java.awt.event.ItemListener;
import java.util.Iterator;
import java.util.Vector;

import javax.swing.BorderFactory;
import javax.swing.ButtonGroup;
import javax.swing.JButton;
import javax.swing.JLabel;
import javax.swing.JPanel;
import javax.swing.JRadioButton;
import javax.swing.border.BevelBorder;
import javax.swing.border.EmptyBorder;
import javax.swing.border.TitledBorder;

import fi.helsinki.dacopan.Localization;
import fi.helsinki.dacopan.Localization.Language;
import fi.helsinki.dacopan.settings.GeneralSettings;

/**
 * A JDialog for loading, saving and restoring settings as well as changing
 * the language.
 */
public class SettingsPanelGeneral extends JPanel implements ActionListener {
	
	private JPanel contentsPanel;
	private JPanel languagePanel;
	private JPanel buttonPanel;
	private JLabel languageLabel;
	private JLabel manageLabel;
	private JButton buttonSaveAll;
	private JButton buttonRestore;
	private JButton buttonLoadAll;
	private ButtonGroup languageButtons;

	private Vector radioButtonItems;
	private Vector availableLanguages;
	private GeneralSettings generalSettings;
	private MainFrame mainFrame;
	private SettingsDialog settingsDialog;
	private static Language selectedLanguage = Localization.getCurrentLanguage();
	
	/**
	 * Constructor
	 * 
	 * @param generalSettings Temporary copy of GeneralSettings.
	 * @param mainFrame Needed for loading, saving &c. the settings and applying
	 * the chosen language.
	 * @param settingsDialog The current parent settings dialog for which the restored
	 * and loaded settings are applied before the user actually chooses to take them in
	 * use
	 */
	public SettingsPanelGeneral (GeneralSettings generalSettings, MainFrame mainFrame, SettingsDialog settingsDialog) {
		this.mainFrame = mainFrame;
		this.settingsDialog = settingsDialog;
		this.generalSettings = generalSettings;
		this.availableLanguages = new Vector(Localization.getAvailableLanguages());
		initComponents();
	}
	
	// initializes the swing components
	private void initComponents() {
		this.contentsPanel = new JPanel();
		contentsPanel.setLayout(new GridBagLayout());

		TitledBorder manageTitle;
		manageTitle = BorderFactory.createTitledBorder(Localization.getString("panel.settings.general.manage"));
		manageTitle.setTitleJustification(TitledBorder.CENTER);
		
		GridBagConstraints d = new GridBagConstraints();
		d.weightx = 1.0;
		d.weighty = 1.0;
		d.fill = GridBagConstraints.HORIZONTAL;
		d.gridwidth = GridBagConstraints.REMAINDER;
		
		buttonRestore = new JButton(Localization.getString("panel.settings.general.restore"));
		buttonSaveAll = new JButton(Localization.getString("panel.settings.general.saveall"));
		buttonLoadAll = new JButton(Localization.getString("panel.settings.general.loadall"));

		buttonRestore.setActionCommand("restore");
		buttonSaveAll.setActionCommand("save");
		buttonLoadAll.setActionCommand("load");
		
		buttonSaveAll.addActionListener(this);
		buttonLoadAll.addActionListener(this);
		buttonRestore.addActionListener(this);

		buttonPanel = new JPanel();
		buttonPanel.setLayout(new GridBagLayout());
		buttonPanel.add(buttonRestore, d);
		buttonPanel.add(buttonSaveAll, d);
		buttonPanel.add(buttonLoadAll, d);
		buttonPanel.setBorder(manageTitle);
		
		GridBagConstraints c = new GridBagConstraints();
		c.gridy = 0;
		contentsPanel.add(buttonPanel, c);

		this.languageButtons = new ButtonGroup();
		
		JPanel languagePanel = new JPanel();
		languagePanel.setLayout(new GridBagLayout());
		
		c = new GridBagConstraints();
		c.gridx = 0;
		c.gridy = 0;
		c.anchor = GridBagConstraints.WEST;
		
		for (Iterator i = Localization.getAvailableLanguages().iterator(); i.hasNext();) {
			final Language language = (Language)i.next();
			JRadioButton langItem =
				new JRadioButton(language.getName());
			if (selectedLanguage.equals(language))
				langItem.setSelected(true);
				
			langItem.addItemListener(new ItemListener() {
				public void itemStateChanged (ItemEvent e) {
					selectedLanguage=language;
				}
			}
				
			);
			
			languageButtons.add(langItem);
			languagePanel.add(langItem, c);
			c.gridy++;
			
		}

		TitledBorder languageTitle;
		languageTitle = BorderFactory.createTitledBorder(Localization.getString("panel.settings.general.language"));
		languageTitle.setTitleJustification(TitledBorder.CENTER);
		languagePanel.setBorder(languageTitle);
		languagePanel.setPreferredSize(buttonPanel.getPreferredSize());
		
		c.gridx = 0;
		c.gridy = 1;
		c.fill = GridBagConstraints.NONE;
		c.insets = new Insets(2,2,2,2);
		c.anchor = GridBagConstraints.NORTH;
		contentsPanel.add(languagePanel, c);		

		this.setLayout(new BorderLayout());
		this.setBorder(new EmptyBorder(3,0,0,0));
		contentsPanel.setBorder(new BevelBorder(BevelBorder.RAISED));
		this.add(contentsPanel);
		
	}
	
	
	/**
	 * Updates the language in use in the Animator.
	 * The language has to be updated differently than the 
	 * other settings because it is not stored in the settings
	 * objects.
	 */
	public void updateLanguage() {
		Localization.setCurrentLanguage(selectedLanguage);
		mainFrame.recreateUI();
	}
	
	public void actionPerformed(ActionEvent e) {
		
		String command = e.getActionCommand();
		
		if ("save".equals(command)) {
			// Apply and save the settings in a file
			settingsDialog.applySettings();
			mainFrame.saveAllSettings();
		}
		else if ("load".equals(command)) {
			// Loads the settings to this temporary generalSettings
			mainFrame.loadAllSettings(generalSettings);
			settingsDialog.setSettings(generalSettings);
		}
		else if ("restore".equals(command)) {
			// Restores these temporary settings to their defaults
			generalSettings = new GeneralSettings();
			mainFrame.applyDefaultSettings(generalSettings);
			settingsDialog.setSettings(generalSettings);
		}
	}
}