/**
 * Creates the panel to change settings related to the performance of the MSC animation
 *
 * @version $Id: SettingsPanelPerformance.java,v 1.13 2005/05/05 20:32:34 tituomin Exp $
 */
package fi.helsinki.dacopan.ui;

import java.awt.GridBagConstraints;

import fi.helsinki.dacopan.Localization;
import fi.helsinki.dacopan.settings.GeneralSettings;

/**
 * <p>
 * JPanel used to change settings related to the performance of the animation
 * </p>
 * <p>
 * These settings are:
 * <ul>
 * <li>refreshDelay.</li>
 * <li>Anti-aliasing.</li>
 * </ul>
 * </p>
 * <p>
 * This object uses a instance of GeneralSettings to access and modify the values of the performance
 * settings.
 * </p>
 * 
 * @version $Id: SettingsPanelPerformance.java,v 1.13 2005/05/05 20:32:34 tituomin Exp $
 */
public class SettingsPanelPerformance extends javax.swing.JPanel {
    
    
    /**
     * Creates new panel SettingsPanelPerformance
     * 
     * @param generalSettings
     *            GeneralSettings instance in which the values for the settings
     *            in about the performance are stored and modified.
     */
    public SettingsPanelPerformance(GeneralSettings generalSettings) {
        this.generalSettings = generalSettings;
        initComponents();
        this.updateValues();
    }
    
    /**
     * initialize the components in the panel
     */
    private void initComponents() {//GEN-BEGIN:initComponents
        java.awt.GridBagConstraints gridBagConstraints;

        buttonGroupScaleButtons = new javax.swing.ButtonGroup();
        slidersPanel = new javax.swing.JPanel();
        panelRefreshDelay = new javax.swing.JPanel();
        labelRefreshDelay = new javax.swing.JLabel();
        sliderRefreshDelay = new javax.swing.JSlider();
        tfRefreshDelay = new javax.swing.JTextField();
        panelTimeScale = new javax.swing.JPanel();
        checkboxAliasing = new javax.swing.JCheckBox();

        setLayout(new java.awt.GridBagLayout());
        
        GridBagConstraints c = new GridBagConstraints();
        c.insets = new java.awt.Insets(2,2,2,2);
        
        slidersPanel.setLayout(new java.awt.GridBagLayout());

        panelRefreshDelay.setLayout(new java.awt.GridBagLayout());

        labelRefreshDelay.setText(Localization.getString("panel.settings.performance.option_refresh"));
        labelRefreshDelay.setToolTipText(Localization.getString("panel.settings.performance.refresh_tooltip"));
        panelRefreshDelay.add(labelRefreshDelay, c);

        sliderRefreshDelay.setMaximum(GeneralSettings.MAX_REFRESH_DELAY);
        sliderRefreshDelay.setMinimum(GeneralSettings.MIN_REFRESH_DELAY);
        sliderRefreshDelay.setToolTipText(Localization.getString("panel.settings.performance.refresh_tooltip"));
        sliderRefreshDelay.setValue(this.generalSettings.getRefreshDelay());
        sliderRefreshDelay.addChangeListener(new javax.swing.event.ChangeListener() {
            public void stateChanged(javax.swing.event.ChangeEvent evt) {
                sliderRefreshDelayStateChanged(evt);
            }
        });

        panelRefreshDelay.add(sliderRefreshDelay, c);

        tfRefreshDelay.setColumns(10);
        tfRefreshDelay.setEditable(false);
        tfRefreshDelay.setHorizontalAlignment(javax.swing.JTextField.TRAILING);
        tfRefreshDelay.setToolTipText(Localization.getString("panel.settings.performance.refresh_tooltip"));
        panelRefreshDelay.add(tfRefreshDelay, c);

        slidersPanel.add(panelRefreshDelay, new java.awt.GridBagConstraints());

        checkboxAliasing.setSelected(generalSettings.getAntiAliasing());
        checkboxAliasing.setText(Localization.getString("panel.settings.performance.option_antialiasing"));
        checkboxAliasing.setToolTipText(Localization.getString("panel.settings.performance.antialiasing_tooltip"));
        checkboxAliasing.setHorizontalTextPosition(javax.swing.SwingConstants.LEADING);
        checkboxAliasing.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                checkboxAliasingActionPerformed(evt);
            }
        });

        panelTimeScale.add(checkboxAliasing);

        gridBagConstraints = new java.awt.GridBagConstraints();
        gridBagConstraints.gridx = 0;
        gridBagConstraints.gridy = 1;
        slidersPanel.add(panelTimeScale, gridBagConstraints);

        add(slidersPanel, new java.awt.GridBagConstraints());

    }//GEN-END:initComponents

    /**
     * Method called when an action is performed in the checkbox that sets the option of aliasing. 
     * 
     * @param evt 
     */
    private void checkboxAliasingActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_checkboxAliasingActionPerformed
        generalSettings.setAntiAliasing(checkboxAliasing.isSelected());
    }//GEN-LAST:event_checkboxAliasingActionPerformed
                

    /**
     * Method called when sliderRefreshDelay state is changed. 
     * 
     * @param evt 
     */
    private void sliderRefreshDelayStateChanged(javax.swing.event.ChangeEvent evt) {//GEN-FIRST:event_sliderRefreshDelayStateChanged
        updateValues();
    }//GEN-LAST:event_sliderRefreshDelayStateChanged
    
     /**
      * Update the value of the refreshDelay in the GeneralSettings attribute of the class and
      * refreshes the text displayed in the textfield with the new value of the refreshDelay.
      */
    private void updateValues(){
        generalSettings.setRefreshDelay(sliderRefreshDelay.getValue());
        tfRefreshDelay.setText(Localization.getString("panel.settings.performance.display_refresh", new Integer(generalSettings.getRefreshDelay()), new Integer(generalSettings.getRefreshRate()))); //$NON-NLS-1$
    }
    
    
    // Variables declaration - do not modify//GEN-BEGIN:variables
    private javax.swing.ButtonGroup buttonGroupScaleButtons;
    private javax.swing.JCheckBox checkboxAliasing;
    private javax.swing.JLabel labelRefreshDelay;
    private javax.swing.JPanel panelRefreshDelay;
    private javax.swing.JPanel panelTimeScale;
    private javax.swing.JSlider sliderRefreshDelay;
    private javax.swing.JPanel slidersPanel;
    private javax.swing.JTextField tfRefreshDelay;
    // End of variables declaration//GEN-END:variables
    
    /**
     * Reference to the GeneralSettings object that is accessed to read the values for the
     * performance settings and is modified with the new performance settings set by the user
     */
    private GeneralSettings generalSettings;
}
