package fi.helsinki.dacopan.ui;

import java.awt.BorderLayout;
import java.awt.Dialog;

import javax.swing.JPanel;
import javax.swing.JTabbedPane;

import fi.helsinki.dacopan.Localization;
import fi.helsinki.dacopan.model.DataView;
import fi.helsinki.dacopan.settings.GeneralSettings;
import fi.helsinki.dacopan.ui.tsc.SettingsTSC;

/**
 * JPanel which acts as a parent for all the different TSC settings tabs.
 */
public class SettingsPanelTSC extends JPanel {
	
	private SettingsTSC auxSettingsTSC;
	private JTabbedPane tabsSettingsPanel;
	private DataView dataView;
	private Dialog owner;
	
	/**
	 * Constructor.
	 * 
	 * @param dataView Current dataview
	 * @param generalSettings Reference to the temporary settings to be changed.
	 * @param owner Is the parent dialog of this panel, which is needed to display
	 * new dialog windows in the Notices panel.
	 */
	public SettingsPanelTSC (DataView dataView, GeneralSettings generalSettings, Dialog owner) {
		this.owner=owner;
		this.dataView = dataView;
		this.auxSettingsTSC = generalSettings.getSettingsTSC();
		this.setLayout(new BorderLayout());
		this.setBackground(UserInterface.DACOPAN_COLOR_PANE_BG);
		initComponents();
	}
	
	// Initializes the Swing components
	private void initComponents() {
		tabsSettingsPanel = new JTabbedPane();
		
		this.add(tabsSettingsPanel, BorderLayout.CENTER);

		JPanel panelElements = new SettingsPanelTSCElements(auxSettingsTSC, owner);
		this.tabsSettingsPanel.addTab(Localization.getString("panel.settings.tsc.elements"), panelElements);

		JPanel panelAnimation = new SettingsPanelTSCAnimation(auxSettingsTSC);
		this.tabsSettingsPanel.addTab(Localization.getString("panel.settings.tsc.animation"), panelAnimation);

		JPanel panelNotices = new SettingsPanelTSCNotices(auxSettingsTSC, dataView, owner);
		tabsSettingsPanel.addTab(Localization.getString("panel.settings.tsc.notices"), panelNotices);
		
		JPanel panelVariables = new SettingsPanelTSCVariables(dataView, auxSettingsTSC);
		this.tabsSettingsPanel.addTab(Localization.getString("panel.settings.tsc.variables"), panelVariables);
		
		
	}
	
	
}
