/*
 * Created on 14.4.2005
 * Modified last on 28.4.2005
 * 
 */

package fi.helsinki.dacopan.ui;

import java.awt.GridBagConstraints;
import java.awt.GridBagLayout;
import java.awt.Insets;
import java.awt.event.ItemEvent;
import java.awt.event.ItemListener;

import javax.swing.JCheckBox;
import javax.swing.JLabel;
import javax.swing.JPanel;
import javax.swing.JSlider;
import javax.swing.JTextField;
import javax.swing.event.ChangeEvent;
import javax.swing.event.ChangeListener;

import fi.helsinki.dacopan.Localization;
import fi.helsinki.dacopan.ui.tsc.SettingsTSC;

/**
 * @author korkki
 * 
 * Created by SettingsPanelTSC.java.
 * 
 * The TSC Animation settings tabbed panel which adjusts the Animation speed and Notice delay, 
 * and toggles between the absolute and relative SEQNO of a unit.
 * 
 * Straight forward generic Swing code. Widgets for the GUI, etc.
 * 
 * Uses the following methods from ui.tsc.SettingsTSC.java
 * 
 * - getAnimationSpeed()
 * - getNoticeDelaySeconds()
 * - isRelativeSEQNO()
 * - setAnimationSpeed(float)
 * - setNoticeDelaySeconds(int)
 * - setRelativeSEQNO(boolean)
 * 
 */

public class SettingsPanelTSCAnimation extends JPanel{
	
	private SettingsTSC auxSettingsTSC;
	
	private JLabel labelAnimationSpeed;
	private JSlider sliderAnimationSpeed;
	private JTextField tfAnimationSpeed;
	
	private JLabel labelNoticeDelay;
	private JSlider sliderNoticeDelay;
	private JTextField tfNoticeDelay;
	
	private JCheckBox checkboxRelativeSEQNO;
	
	
	public SettingsPanelTSCAnimation (SettingsTSC settingsTSC) {
		this.auxSettingsTSC = settingsTSC;
		initComponents();
	}
		
	private void initComponents() {
		
		JPanel slidersPanel = new JPanel();
		JPanel checkboxPanel = new JPanel();
		
        labelAnimationSpeed = new JLabel();
        sliderAnimationSpeed = new JSlider();
        tfAnimationSpeed = new JTextField();
        
        labelNoticeDelay = new JLabel();
        sliderNoticeDelay = new JSlider();
        tfNoticeDelay = new JTextField();
        
        checkboxRelativeSEQNO = new JCheckBox();
        
        GridBagConstraints c = new GridBagConstraints();
        slidersPanel.setLayout(new GridBagLayout());
        
        c.weightx = 1.0;
		c.weighty = 0.0;
		c.insets = new Insets(2,2,2,2);
		labelAnimationSpeed.setText(Localization.getString("panel.settings.animation.label_animation_speed"));
		labelAnimationSpeed.setToolTipText(Localization.getString("panel.settings.animation.animation_speed_tooltip"));
		slidersPanel.add(labelAnimationSpeed, c);
		c.fill = GridBagConstraints.HORIZONTAL;
		sliderAnimationSpeed.setToolTipText(Localization.getString("panel.settings.animation.animation_speed_tooltip"));
		sliderAnimationSpeed.setMinimum(1);
		sliderAnimationSpeed.setMaximum(200);
		slidersPanel.add(sliderAnimationSpeed, c);
		c.fill = GridBagConstraints.NONE;
		c.gridwidth = GridBagConstraints.REMAINDER;
		tfAnimationSpeed.setEditable(false);
		tfAnimationSpeed.setColumns(11);
		tfAnimationSpeed.setToolTipText(Localization.getString("panel.settings.animation.animation_speed_tooltip"));
		tfAnimationSpeed.setHorizontalAlignment(JTextField.TRAILING);
		slidersPanel.add(tfAnimationSpeed, c);
		
		c.gridwidth = 1;
		labelNoticeDelay.setText(Localization.getString("panel.settings.animation.label_notice_delay"));
		labelNoticeDelay.setToolTipText(Localization.getString("panel.settings.animation.notice_delay_tooltip"));
		slidersPanel.add(labelNoticeDelay, c);
		c.fill = GridBagConstraints.HORIZONTAL;
		sliderNoticeDelay.setToolTipText(Localization.getString("panel.settings.animation.notice_delay_tooltip"));
		sliderNoticeDelay.setMinimum(1);
		sliderNoticeDelay.setMaximum(10);
		slidersPanel.add(sliderNoticeDelay, c);
		c.fill = GridBagConstraints.NONE;
		c.gridwidth = GridBagConstraints.REMAINDER;
		tfNoticeDelay.setEditable(false);
		tfNoticeDelay.setColumns(11);
		tfNoticeDelay.setToolTipText(Localization.getString("panel.settings.animation.notice_delay_tooltip"));
		tfNoticeDelay.setHorizontalAlignment(JTextField.TRAILING);
		slidersPanel.add(tfNoticeDelay, c);
		
        
        GridBagConstraints d = new GridBagConstraints();
        checkboxPanel.setLayout(new GridBagLayout());
        
        d.gridx = 0;
        d.gridy = 1;
        d.gridwidth = GridBagConstraints.REMAINDER;
        checkboxRelativeSEQNO.setText(Localization.getString("panel.settings.animation.relative_seqno"));
        checkboxRelativeSEQNO.setToolTipText(Localization.getString("panel.settings.animation.relative_seqno_tooltip"));
        checkboxRelativeSEQNO.setHorizontalTextPosition(javax.swing.SwingConstants.LEADING);
        checkboxPanel.add(checkboxRelativeSEQNO, d);

        slidersPanel.add(checkboxPanel, c);
        add(slidersPanel, new GridBagConstraints());
		
		initSliderValues();
        updateSliderAnimationSpeed();
        updateSliderNoticeDelay();
        
        
        sliderAnimationSpeed.addChangeListener(new ChangeListener() {
        	public void stateChanged (ChangeEvent e) {
        		updateSliderAnimationSpeed();
				auxSettingsTSC.setAnimationSpeed((float) sliderAnimationSpeed.getValue() / 10.0f);
        	}
        });
        
        sliderNoticeDelay.addChangeListener(new ChangeListener() {
        	public void stateChanged (ChangeEvent e) {
        		updateSliderNoticeDelay();
				auxSettingsTSC.setNoticeDelaySeconds(sliderNoticeDelay.getValue());
        	}
        });
        
        checkboxRelativeSEQNO.addItemListener(new ItemListener() {
        	public void itemStateChanged(ItemEvent e) {
        		if (auxSettingsTSC.isRelativeSEQNO() == true) { 
        			auxSettingsTSC.setRelativeSEQNO(false);
        		}
				else {
					auxSettingsTSC.setRelativeSEQNO(true);
				}
        	}
        });
        
        this.setLayout(new GridBagLayout());
		this.add(slidersPanel);
	}
	
	private void initSliderValues() {
        sliderAnimationSpeed.setValue((int) (auxSettingsTSC.getAnimationSpeed() * 10));
        tfAnimationSpeed.setText(Localization.getString("panel.settings.animation.display_animation_speed", 
        (new Float(sliderAnimationSpeed.getValue() / 10.0f).toString())));
        
        sliderNoticeDelay.setValue(auxSettingsTSC.getNoticeDelaySeconds());
        tfNoticeDelay.setText(Localization.getString("panel.settings.animation.display_notice_delay",
        (new Float(sliderNoticeDelay.getValue() / 1.0f).toString())));
        
        checkboxRelativeSEQNO.setSelected(auxSettingsTSC.isRelativeSEQNO());
	}
	
	private void updateSliderAnimationSpeed() {
		tfAnimationSpeed.setText(Localization.getString("panel.settings.animation.display_animation_speed", 
		(new Float(sliderAnimationSpeed.getValue() / 10.0f).toString())));
	}
	
	private void updateSliderNoticeDelay() {
		tfNoticeDelay.setText(Localization.getString("panel.settings.animation.display_notice_delay", 
		(new Float(sliderNoticeDelay.getValue() / 1.0f).toString())));
	}
}