/*
 * Created on 14.4.2005
 * Modified last on 28.4.2005
 *
 */

package fi.helsinki.dacopan.ui;

import java.awt.BorderLayout;
import java.awt.Color;
import java.awt.Dialog;
import java.awt.Dimension;
import java.awt.FlowLayout;
import java.awt.GridBagConstraints;
import java.awt.GridBagLayout;
import java.awt.GridLayout;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.awt.event.ItemEvent;
import java.awt.event.ItemListener;

import javax.swing.BorderFactory;
import javax.swing.JButton;
import javax.swing.JCheckBox;
import javax.swing.JColorChooser;
import javax.swing.JDialog;
import javax.swing.JPanel;
import javax.swing.border.TitledBorder;

import fi.helsinki.dacopan.Localization;
import fi.helsinki.dacopan.ui.tsc.SettingsTSC;

/**
 * @author korkki
 * 
 * Created by SettingsPanelTSC.java.
 * 
 * The TSC Graphical elements settings tabbed panel that is used to select which elements are 
 * displayed. Also the colors for the graphical elements can be changed via the JColorChooser 
 * component.
 * 
 * Straight forward generic Swing code. Widgets for the GUI, etc.
 * 
 * Uses the following methods from ui.tsc.SettingsTSC.java
 * 
 * - getColor(String)
 * - getFINColor()
 * - getSYNColor() 
 * - highlightUnit(String, boolean)
 * - isElementDisplayed(String)
 * - isUnitHighlighted(String)
 * - setColor(String, Color)
 * - setElementDisplayed(String, boolean) 
 */

public class SettingsPanelTSCElements extends JPanel implements ActionListener {
	
	private Color chosenColor;
	private SettingsTSC auxSettingsTSC;
	private Dialog owner;
	
	private JButton buttonAdvertizedWindow;
    private JButton buttonConnectingLine;
    private JButton buttonDelayLine;
    private JButton buttonGrid;
    
    private JButton buttonACK;
    private JButton buttonSACK;
    private JButton buttonUnit;
    
    private JButton buttonFIN;
    private JButton buttonSYN;
    
    private JCheckBox checkboxDisplayCrosshair;
    private JCheckBox checkboxDisplayNotices;
    private JCheckBox checkboxDisplayAdvertizedWindow;
    private JCheckBox checkboxDisplayConnectingLine;
    private JCheckBox checkboxDisplayDelayLine;
    private JCheckBox checkboxDisplayGrid;
    
    private JCheckBox checkboxDisplayACK;
    private JCheckBox checkboxDisplayFIN;
    private JCheckBox checkboxDisplaySACK;
    private JCheckBox checkboxDisplaySYN;
    private JCheckBox checkboxDisplayUnit;
    
    private JCheckBox checkboxHighlightFIN;
    private JCheckBox checkboxHighlightSYN;
    
    
	public SettingsPanelTSCElements (SettingsTSC settingsTSC, Dialog owner) {
		
		this.owner = owner;
		auxSettingsTSC = settingsTSC;
		initComponents();
	}
	
	private void initComponents() {
		
		JPanel buttonsA1Panel = new JPanel();
		buttonsA1Panel.setLayout(new GridLayout(6,1,2,2));
		buttonsA1Panel.setPreferredSize(new Dimension(200,180));
		
		JPanel emptyPanelA = new JPanel();
        emptyPanelA.setPreferredSize(new Dimension(15,15));
        
        JPanel emptyPanelB = new JPanel();
        emptyPanelB.setPreferredSize(new Dimension(15,15));
        
        buttonAdvertizedWindow = new JButton();
        buttonAdvertizedWindow.setPreferredSize(new Dimension(15, 15));
        buttonAdvertizedWindow.addActionListener(this);
        buttonConnectingLine = new JButton();
        buttonConnectingLine.setPreferredSize(new Dimension(15, 15));
        buttonConnectingLine.addActionListener(this);
        buttonDelayLine = new JButton();
        buttonDelayLine.setPreferredSize(new Dimension(15, 15));
        buttonDelayLine.addActionListener(this);
        buttonGrid = new JButton();
        buttonGrid.setPreferredSize(new Dimension(15, 15));
        buttonGrid.addActionListener(this);
        
        checkboxDisplayCrosshair = new JCheckBox(Localization.getString("panel.settings.elements.crosshair"), null);
        checkboxDisplayNotices = new JCheckBox(Localization.getString("panel.settings.elements.notices"), null);
        checkboxDisplayAdvertizedWindow = new JCheckBox(Localization.getString("panel.settings.elements.advertized_window_size"), null);
        checkboxDisplayConnectingLine = new JCheckBox(Localization.getString("panel.settings.elements.connecting_line"), null);
        checkboxDisplayDelayLine = new JCheckBox(Localization.getString("panel.settings.elements.delay_line"), null);
        checkboxDisplayGrid = new JCheckBox(Localization.getString("panel.settings.elements.grid"), null);
        
        JPanel pA1 = new JPanel(new FlowLayout(FlowLayout.LEFT));
		pA1.add(emptyPanelA);
		pA1.add(checkboxDisplayCrosshair);
		buttonsA1Panel.add(pA1);
		
		JPanel pA2 = new JPanel(new FlowLayout(FlowLayout.LEFT));
		pA2.add(emptyPanelB);
		pA2.add(checkboxDisplayNotices);
		buttonsA1Panel.add(pA2);
		
        JPanel pA3 = new JPanel(new FlowLayout(FlowLayout.LEFT));
		pA3.add(buttonAdvertizedWindow);
		pA3.add(checkboxDisplayAdvertizedWindow);
		buttonsA1Panel.add(pA3);
		
		JPanel pA4 = new JPanel(new FlowLayout(FlowLayout.LEFT));
		pA4.add(buttonConnectingLine);
		pA4.add(checkboxDisplayConnectingLine);
		buttonsA1Panel.add(pA4);
		
		JPanel pA5 = new JPanel(new FlowLayout(FlowLayout.LEFT));
		pA5.add(buttonDelayLine);
		pA5.add(checkboxDisplayDelayLine);
		buttonsA1Panel.add(pA5);
		
		JPanel pA6 = new JPanel(new FlowLayout(FlowLayout.LEFT));
		pA6.add(buttonGrid);
		pA6.add(checkboxDisplayGrid);
		buttonsA1Panel.add(pA6);
		
		JPanel buttonsA2Panel = new JPanel();
		buttonsA2Panel.setLayout(new GridLayout(6,1,2,2));
		buttonsA2Panel.setPreferredSize(new Dimension(160,180));
		
		JPanel emptyPanelC = new JPanel();
        emptyPanelC.setPreferredSize(new Dimension(15,15));
        
        JPanel emptyPanelD = new JPanel();
        emptyPanelD.setPreferredSize(new Dimension(15,15));
        
		buttonACK = new JButton();
        buttonACK.setPreferredSize(new Dimension(15, 15));
        buttonACK.addActionListener(this);
        buttonSACK = new JButton();
        buttonSACK.setPreferredSize(new Dimension(15, 15));
        buttonSACK.addActionListener(this);
        buttonUnit = new JButton();
        buttonUnit.setPreferredSize(new Dimension(15, 15));
        buttonUnit.addActionListener(this);
        
		checkboxDisplayACK = new JCheckBox(Localization.getString("panel.settings.elements.ack"), null);
		checkboxDisplayFIN = new JCheckBox(Localization.getString("panel.settings.elements.fin"), null);
        checkboxDisplaySACK = new JCheckBox(Localization.getString("panel.settings.elements.sack"), null);
        checkboxDisplaySYN = new JCheckBox(Localization.getString("panel.settings.elements.syn"), null);
        checkboxDisplayUnit = new JCheckBox(Localization.getString("panel.settings.elements.unit"), null);
        
		JPanel pB1 = new JPanel(new FlowLayout(FlowLayout.LEFT));
		pB1.add(buttonACK);
		pB1.add(checkboxDisplayACK);
		buttonsA2Panel.add(pB1);
		
		JPanel pB2 = new JPanel(new FlowLayout(FlowLayout.LEFT));
		pB2.add(emptyPanelC);
		pB2.add(checkboxDisplayFIN);
		buttonsA2Panel.add(pB2);
		
		JPanel pB3 = new JPanel(new FlowLayout(FlowLayout.LEFT));
		pB3.add(buttonSACK);
		pB3.add(checkboxDisplaySACK);
		buttonsA2Panel.add(pB3);
		
		JPanel pB4 = new JPanel(new FlowLayout(FlowLayout.LEFT));
		pB4.add(emptyPanelD);
		pB4.add(checkboxDisplaySYN);
		buttonsA2Panel.add(pB4);
		
		JPanel pB5 = new JPanel(new FlowLayout(FlowLayout.LEFT));
		pB5.add(buttonUnit);
		pB5.add(checkboxDisplayUnit);
		buttonsA2Panel.add(pB5);
		
		
		JPanel buttonsAPanel = new JPanel();
		buttonsAPanel.setLayout(new BorderLayout());
		
		TitledBorder buttonsATitle;
		buttonsATitle = BorderFactory.createTitledBorder(Localization.getString("panel.settings.elements.border1_text"));
		buttonsATitle.setTitleJustification(TitledBorder.CENTER);
		
		buttonsAPanel.setBorder(buttonsATitle);
		buttonsAPanel.add(buttonsA1Panel, BorderLayout.WEST);
		buttonsAPanel.add(buttonsA2Panel, BorderLayout.EAST);

		
		JPanel buttonsBPanel = new JPanel();
		buttonsBPanel.setLayout(new GridLayout(2,1,2,2));
		buttonsBPanel.setPreferredSize(new Dimension(150,80));
		
		TitledBorder buttonsBTitle;
		buttonsBTitle = BorderFactory.createTitledBorder(Localization.getString("panel.settings.elements.border2_text"));
		buttonsBTitle.setTitleJustification(TitledBorder.CENTER);
		
		buttonsBPanel.setBorder(buttonsBTitle);
		
		buttonFIN = new JButton();
        buttonFIN.setPreferredSize(new Dimension(15, 15));
        buttonFIN.addActionListener(this);
        buttonSYN = new JButton();
        buttonSYN.setPreferredSize(new Dimension(15, 15));
        buttonSYN.addActionListener(this);
		
		checkboxHighlightFIN = new JCheckBox(Localization.getString("panel.settings.elements.fin"), null);
        checkboxHighlightSYN = new JCheckBox(Localization.getString("panel.settings.elements.syn"), null);
		
        JPanel pC1 = new JPanel(new FlowLayout(FlowLayout.LEFT));
		pC1.add(buttonFIN);
		pC1.add(checkboxHighlightFIN);
		buttonsBPanel.add(pC1);
		
		JPanel pC2 = new JPanel(new FlowLayout(FlowLayout.LEFT));
		pC2.add(buttonSYN);
		pC2.add(checkboxHighlightSYN);
		buttonsBPanel.add(pC2);
		
		JPanel contentsPanel = new JPanel();
		contentsPanel.setLayout(new GridBagLayout());
		contentsPanel.setMinimumSize(new Dimension(370, 200));

		GridBagConstraints g = new GridBagConstraints();
		g.gridwidth = GridBagConstraints.REMAINDER;
		contentsPanel.add(buttonsAPanel, g);
		contentsPanel.add(buttonsBPanel, g);
		add(contentsPanel);
		
		
        initButtons();
        
        
        checkboxDisplayCrosshair.addItemListener(new ItemListener() {
        	public void itemStateChanged(ItemEvent e) {
        		if (auxSettingsTSC.isElementDisplayed(SettingsTSC.ELEMENTKEY_CROSSHAIR) == true) {
        			auxSettingsTSC.setElementDisplayed(SettingsTSC.ELEMENTKEY_CROSSHAIR, false);
        		}
        		else {
        			auxSettingsTSC.setElementDisplayed(SettingsTSC.ELEMENTKEY_CROSSHAIR, true);
        		}
        	}
        });
        
        checkboxDisplayNotices.addItemListener(new ItemListener() {
        	public void itemStateChanged(ItemEvent e) {
        		if (auxSettingsTSC.isElementDisplayed(SettingsTSC.ELEMENTKEY_NOTICES) == true) {
        			auxSettingsTSC.setElementDisplayed(SettingsTSC.ELEMENTKEY_NOTICES, false);
        		}
        		else {
        			auxSettingsTSC.setElementDisplayed(SettingsTSC.ELEMENTKEY_NOTICES, true);
        		}
        	}
        });
        
        checkboxDisplayAdvertizedWindow.addItemListener(new ItemListener() {
        	public void itemStateChanged(ItemEvent e) {
        		if (auxSettingsTSC.isElementDisplayed(SettingsTSC.ELEMENTKEY_ADVERTIZED_WINDOW_LINE) == true) {
        			auxSettingsTSC.setElementDisplayed(SettingsTSC.ELEMENTKEY_ADVERTIZED_WINDOW_LINE, false);
        		}
        		else {
        			auxSettingsTSC.setElementDisplayed(SettingsTSC.ELEMENTKEY_ADVERTIZED_WINDOW_LINE, true);
        		}
        	}
        });
        
        checkboxDisplayConnectingLine.addItemListener(new ItemListener() {
        	public void itemStateChanged(ItemEvent e) {
        		if (auxSettingsTSC.isElementDisplayed(SettingsTSC.ELEMENTKEY_CONNECTING_LINE) == true) {
        			auxSettingsTSC.setElementDisplayed(SettingsTSC.ELEMENTKEY_CONNECTING_LINE, false);
        		}
        		else {
        			auxSettingsTSC.setElementDisplayed(SettingsTSC.ELEMENTKEY_CONNECTING_LINE, true);
        		}
        	}
        });
        
        checkboxDisplayDelayLine.addItemListener(new ItemListener() {
        	public void itemStateChanged(ItemEvent e) {
        		if (auxSettingsTSC.isElementDisplayed(SettingsTSC.ELEMENTKEY_DELAY_LINE) == true) {
        			auxSettingsTSC.setElementDisplayed(SettingsTSC.ELEMENTKEY_DELAY_LINE, false);
        		}
        		else {
        			auxSettingsTSC.setElementDisplayed(SettingsTSC.ELEMENTKEY_DELAY_LINE, true);
        		}
        	}
        });
        
        checkboxDisplayGrid.addItemListener(new ItemListener() {
        	public void itemStateChanged(ItemEvent e) {
        		if (auxSettingsTSC.isElementDisplayed(SettingsTSC.ELEMENTKEY_GRID) == true) {
        			auxSettingsTSC.setElementDisplayed(SettingsTSC.ELEMENTKEY_GRID, false);
        		}
        		else {
        			auxSettingsTSC.setElementDisplayed(SettingsTSC.ELEMENTKEY_GRID, true);
        		}
        	}
        });
        
        checkboxDisplayACK.addItemListener(new ItemListener() {
        	public void itemStateChanged(ItemEvent e) {
        		if (auxSettingsTSC.isElementDisplayed(SettingsTSC.ELEMENTKEY_ACK) == true) {
        			auxSettingsTSC.setElementDisplayed(SettingsTSC.ELEMENTKEY_ACK, false);
        		}
        		else {
        			auxSettingsTSC.setElementDisplayed(SettingsTSC.ELEMENTKEY_ACK, true);
        		}
        	}
        });
        
        checkboxDisplayFIN.addItemListener(new ItemListener() {
        	public void itemStateChanged(ItemEvent e) {
        		if (auxSettingsTSC.isElementDisplayed(SettingsTSC.ELEMENTKEY_FIN) == true) {
        			auxSettingsTSC.setElementDisplayed(SettingsTSC.ELEMENTKEY_FIN, false);
        		}
        		else {
        			auxSettingsTSC.setElementDisplayed(SettingsTSC.ELEMENTKEY_FIN, true);
        		}
        	}
        });
        
        checkboxDisplaySACK.addItemListener(new ItemListener() {
        	public void itemStateChanged(ItemEvent e) {
        		if (auxSettingsTSC.isElementDisplayed(SettingsTSC.ELEMENTKEY_SACK) == true) {
        			auxSettingsTSC.setElementDisplayed(SettingsTSC.ELEMENTKEY_SACK, false);
        		}
        		else {
        			auxSettingsTSC.setElementDisplayed(SettingsTSC.ELEMENTKEY_SACK, true);
        		}
        	}
        });
        
        checkboxDisplaySYN.addItemListener(new ItemListener() {
        	public void itemStateChanged(ItemEvent e) {
        		if (auxSettingsTSC.isElementDisplayed(SettingsTSC.ELEMENTKEY_SYN) == true) {
        			auxSettingsTSC.setElementDisplayed(SettingsTSC.ELEMENTKEY_SYN, false);
        		}
        		else {
        			auxSettingsTSC.setElementDisplayed(SettingsTSC.ELEMENTKEY_SYN, true);
        		}
        	}
        });
        
        checkboxDisplayUnit.addItemListener(new ItemListener() {
        	public void itemStateChanged(ItemEvent e) {
        		if (auxSettingsTSC.isElementDisplayed(SettingsTSC.ELEMENTKEY_SEGMENT) == true) {
        			auxSettingsTSC.setElementDisplayed(SettingsTSC.ELEMENTKEY_SEGMENT, false);
        		}
        		else {
        			auxSettingsTSC.setElementDisplayed(SettingsTSC.ELEMENTKEY_SEGMENT, true);
        		}
        	}
        });
        
        checkboxHighlightFIN.addItemListener(new ItemListener() {
        	public void itemStateChanged(ItemEvent e) {
        		if (checkboxHighlightFIN.isSelected() == true) {
        			auxSettingsTSC.highlightUnit(SettingsTSC.ELEMENTKEY_FIN, true);
        		}
        		else {
        			auxSettingsTSC.highlightUnit(SettingsTSC.ELEMENTKEY_FIN, false);
        		}	
        	}
        });
         
        checkboxHighlightSYN.addItemListener(new ItemListener() {
        	public void itemStateChanged(ItemEvent e) {
        		if (checkboxHighlightSYN.isSelected() == true) {
        			auxSettingsTSC.highlightUnit(SettingsTSC.ELEMENTKEY_SYN, true);
        		}
        		else {
        			auxSettingsTSC.highlightUnit(SettingsTSC.ELEMENTKEY_SYN, false);
        		}
        	}
        });
	}
        
    public void actionPerformed(ActionEvent e) {
    	JButton button = (JButton) e.getSource();
    	if (button == buttonACK) {
    		ColorPalette colorPalette = new ColorPalette(buttonACK.getBackground());
    		buttonACK.setBackground(chosenColor);
    		auxSettingsTSC.setColor(SettingsTSC.COLORKEY_ACK, chosenColor);
    	} 
    	else if (button == buttonSACK) {
    		ColorPalette colorPalette = new ColorPalette(buttonSACK.getBackground());
    		buttonSACK.setBackground(chosenColor);
    		auxSettingsTSC.setColor(SettingsTSC.COLORKEY_SACK, chosenColor);
    	} 
    	else if (button == buttonAdvertizedWindow) {
    		ColorPalette colorPalette = new ColorPalette(buttonAdvertizedWindow.getBackground());
    		buttonAdvertizedWindow.setBackground(chosenColor);
    		auxSettingsTSC.setColor(SettingsTSC.COLORKEY_ADVERTIZED_WINDOW_LINE, chosenColor);
    	} 
    	else if (button == buttonConnectingLine) {
    		ColorPalette colorPalette = new ColorPalette(buttonConnectingLine.getBackground());
    		buttonConnectingLine.setBackground(chosenColor);
    		auxSettingsTSC.setColor(SettingsTSC.COLORKEY_CONNECTING_LINE, chosenColor);
    	} 
    	else if (button == buttonDelayLine) {
    		ColorPalette colorPalette = new ColorPalette(buttonDelayLine.getBackground());
    		buttonDelayLine.setBackground(chosenColor);
    		auxSettingsTSC.setColor(SettingsTSC.COLORKEY_DELAY_LINE, chosenColor);
    	} 
    	else if (button == buttonGrid) {
    		ColorPalette colorPalette = new ColorPalette(buttonGrid.getBackground());
    		buttonGrid.setBackground(chosenColor);
    		auxSettingsTSC.setColor(SettingsTSC.COLORKEY_GRID_MAJOR, chosenColor);
    		//Minor Grid lines are drawn brighter using the formula: R+(255-R)/2
    		chosenColor = new Color((int) (chosenColor.getRed()+255)/2, (int) (chosenColor.getGreen()+255)/2, 
    				(int) (chosenColor.getBlue()+255)/2);
    		auxSettingsTSC.setColor(SettingsTSC.COLORKEY_GRID_MINOR, chosenColor);
    	}
    	else if (button == buttonUnit) {
    		ColorPalette colorPalette = new ColorPalette(buttonUnit.getBackground());
    		buttonUnit.setBackground(chosenColor);
    		auxSettingsTSC.setColor(SettingsTSC.COLORKEY_SEGMENT, chosenColor);
    	} 
    	else if (button == buttonFIN) {
    		ColorPalette colorPalette = new ColorPalette(buttonFIN.getBackground());
    		buttonFIN.setBackground(chosenColor);
    		auxSettingsTSC.setColor(SettingsTSC.COLORKEY_FIN, chosenColor);	
    	}
    	else if (button == buttonSYN) {
    		ColorPalette colorPalette = new ColorPalette(buttonSYN.getBackground());
        	buttonSYN.setBackground(chosenColor);
        	auxSettingsTSC.setColor(SettingsTSC.COLORKEY_SYN, chosenColor);
    	}
    }
	
	private void initButtons() {
		
	 	buttonAdvertizedWindow.setBackground(auxSettingsTSC.getColor(SettingsTSC.COLORKEY_ADVERTIZED_WINDOW_LINE));
		buttonConnectingLine.setBackground(auxSettingsTSC.getColor(SettingsTSC.COLORKEY_CONNECTING_LINE));
		buttonDelayLine.setBackground(auxSettingsTSC.getColor(SettingsTSC.COLORKEY_DELAY_LINE));
		buttonGrid.setBackground(auxSettingsTSC.getColor(SettingsTSC.COLORKEY_GRID_MAJOR));
		
		buttonACK.setBackground(auxSettingsTSC.getColor(SettingsTSC.COLORKEY_ACK));
		buttonSACK.setBackground(auxSettingsTSC.getColor(SettingsTSC.COLORKEY_SACK));
		buttonUnit.setBackground(auxSettingsTSC.getColor(SettingsTSC.COLORKEY_SEGMENT));
		
		buttonFIN.setBackground(auxSettingsTSC.getFINColor());
		buttonSYN.setBackground(auxSettingsTSC.getSYNColor());
		
		checkboxDisplayCrosshair.setSelected(auxSettingsTSC.isElementDisplayed(SettingsTSC.ELEMENTKEY_CROSSHAIR));
		checkboxDisplayNotices.setSelected(auxSettingsTSC.isElementDisplayed(SettingsTSC.ELEMENTKEY_NOTICES));
		checkboxDisplayAdvertizedWindow.setSelected(auxSettingsTSC.isElementDisplayed(SettingsTSC.ELEMENTKEY_ADVERTIZED_WINDOW_LINE));
		checkboxDisplayConnectingLine.setSelected(auxSettingsTSC.isElementDisplayed(SettingsTSC.ELEMENTKEY_CONNECTING_LINE));
		checkboxDisplayDelayLine.setSelected(auxSettingsTSC.isElementDisplayed(SettingsTSC.ELEMENTKEY_DELAY_LINE));
		checkboxDisplayGrid.setSelected(auxSettingsTSC.isElementDisplayed(SettingsTSC.ELEMENTKEY_GRID));
		
		checkboxDisplayACK.setSelected(auxSettingsTSC.isElementDisplayed(SettingsTSC.ELEMENTKEY_ACK));
		checkboxDisplayFIN.setSelected(auxSettingsTSC.isElementDisplayed(SettingsTSC.ELEMENTKEY_FIN));
		checkboxDisplaySACK.setSelected(auxSettingsTSC.isElementDisplayed(SettingsTSC.ELEMENTKEY_SACK));
		checkboxDisplaySYN.setSelected(auxSettingsTSC.isElementDisplayed(SettingsTSC.ELEMENTKEY_SYN));
		checkboxDisplayUnit.setSelected(auxSettingsTSC.isElementDisplayed(SettingsTSC.ELEMENTKEY_SEGMENT));
		
		checkboxHighlightFIN.setSelected(auxSettingsTSC.isUnitHighlighted(SettingsTSC.ELEMENTKEY_FIN));
		checkboxHighlightSYN.setSelected(auxSettingsTSC.isUnitHighlighted(SettingsTSC.ELEMENTKEY_SYN));	
	}
	
	private class ColorPalette extends JDialog implements ActionListener {
		private JPanel buttonPanel;
		private JButton acceptButton;
		private JButton cancelButton;
		private JColorChooser tcc;
		private Color initialColor;
		
		public ColorPalette(Color initialColor) {	
			super(owner, Localization.getString("panel.settings.elements.colorchooser"), true);
			this.initialColor = initialColor;
			initComponents();
		}
		
		private void initComponents() {
			this.getContentPane().setLayout(new BorderLayout());
			
			tcc = new JColorChooser();
			tcc.setColor(initialColor);
			tcc.setBorder(BorderFactory.createTitledBorder("Select Color"));
			
			buttonPanel = new JPanel();
			acceptButton = new JButton(Localization.getString("ui.button_accept"));
			cancelButton = new JButton(Localization.getString("ui.button_cancel"));
			acceptButton.setActionCommand("accept");
			cancelButton.setActionCommand("cancel");
			acceptButton.addActionListener(this);
			cancelButton.addActionListener(this);
			
			buttonPanel.setLayout(new FlowLayout());
			buttonPanel.add(acceptButton);
			buttonPanel.add(cancelButton);
			
			this.getContentPane().add(tcc, BorderLayout.NORTH);
			this.getContentPane().add(buttonPanel, BorderLayout.SOUTH);

			this.pack();
			this.setLocationRelativeTo(owner);
			this.setVisible(true);
		}

		public void actionPerformed(ActionEvent e) {
			String command = e.getActionCommand();
			
			if ("accept".equals(command)) {
				chosenColor = tcc.getColor();
				}
			if ("cancel".equals(command)) {
				chosenColor = initialColor;
			}
			this.setVisible(false);
		}
	}
}