package fi.helsinki.dacopan.ui;

import java.awt.BorderLayout;
import java.awt.Component;
import java.awt.Dialog;
import java.awt.FlowLayout;
import java.awt.GridBagConstraints;
import java.awt.GridBagLayout;
import java.awt.Insets;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.awt.event.ItemEvent;
import java.awt.event.ItemListener;
import java.util.Collection;
import java.util.Iterator;
import java.util.List;
import java.util.Vector;

import javax.swing.JButton;
import javax.swing.JComboBox;
import javax.swing.JDialog;
import javax.swing.JLabel;
import javax.swing.JList;
import javax.swing.JPanel;
import javax.swing.JScrollPane;
import javax.swing.JTextField;
import javax.swing.event.ListSelectionEvent;
import javax.swing.event.ListSelectionListener;
import javax.swing.plaf.basic.BasicComboBoxRenderer;

import fi.helsinki.dacopan.Localization;
import fi.helsinki.dacopan.model.DataView;
import fi.helsinki.dacopan.model.Layer;
import fi.helsinki.dacopan.model.VariableDefinition;
import fi.helsinki.dacopan.ui.tsc.NoticeTrigger;
import fi.helsinki.dacopan.ui.tsc.SettingsTSC;

/**
 * JPanel for adding, editing, removing and enabling/disabling triggers which trigger
 * notices to be displayed in the TSC animation.
 */
public class SettingsPanelTSCNotices extends JPanel implements ActionListener {

	private SettingsTSC auxSettingsTSC;
	private DataView dataView;
	private Layer layer;

	// Current triggers.
	private Vector triggers;
	// Possible variables to test in the triggers
	private List variables;
	
	// Swing components
	private JPanel contentsPanel;
	private JList listNotices;
	private JScrollPane scrollPaneList;
	private JTextField noticeText;
	private JButton buttonAdd;
	private JButton buttonEdit;
	private JButton buttonRemove;
	private JButton buttonEnableDisable;
	private JPanel panelButtons;
	private Dialog owner;
	
	
	/**
	 * Constructor.
	 * 
	 * @param settingsTSC Temporary SettingsTSC object to modify
	 * @param dataview Required to get the available variables
	 * @param owner Reference to the SettingsDialog is required to be able to display the
	 * NoticeDialog
	 */
	public SettingsPanelTSCNotices (SettingsTSC settingsTSC, DataView dataview, Dialog owner) {
		this.owner = owner;
		this.dataView = dataview;
		auxSettingsTSC = settingsTSC;
		triggers = auxSettingsTSC.getVectorNoticeTriggers();
		initVariableList();
		initComponents();
	}
	
	// Gets the variables for TCP layer
	private void initVariableList() {
		layer = null;
		List layerList = dataView.getLayers();
		for (Iterator j = layerList.iterator(); j.hasNext();) {
			layer = (Layer)j.next();
			if (layer.getId() == 2)
				break;
		}
		variables = dataView.getAvailableVariables(layer);
	}
	
	// Initializes the swing components
	private void initComponents() {
		this.contentsPanel = new JPanel();
		contentsPanel.setLayout(new GridBagLayout());
		contentsPanel.add(new JLabel(Localization.getString("panel.settings.tsc.notices")));
		panelButtons =  new JPanel();
		panelButtons.setLayout(new GridBagLayout());
		GridBagConstraints c = new GridBagConstraints();
		
		buttonAdd = new JButton (Localization.getString("panel.settings.tsc.add"));
		buttonAdd.setActionCommand("add");
		buttonEdit = new JButton (Localization.getString("panel.settings.tsc.edit"));
		buttonEdit.setActionCommand("edit");
		buttonRemove = new JButton (Localization.getString("panel.settings.tsc.remove"));
		buttonRemove.setActionCommand("remove");

		buttonEnableDisable = new JButton();

		c.gridy = 0;
		c.gridwidth = GridBagConstraints.REMAINDER;
		c.fill = GridBagConstraints.HORIZONTAL;
		c.weightx = 1.0;
		c.weighty = 1.0;
		panelButtons.add(buttonAdd, c);
		c.gridy = 1;
		c.gridwidth = GridBagConstraints.REMAINDER;
		panelButtons.add(buttonEdit, c);
		c.gridy = 2;
		c.gridwidth = GridBagConstraints.REMAINDER;
		panelButtons.add(buttonRemove, c);
		c.gridy = 3;
		c.gridwidth = GridBagConstraints.REMAINDER;
		panelButtons.add(buttonEnableDisable, c);
		
		initListNotices();
		
		c = new GridBagConstraints();
		c.weightx = 1.0;
		c.weighty = 1.0;
		c.gridx = 0;
		c.gridy = 0;
		
		scrollPaneList = new JScrollPane (listNotices);
		c = new GridBagConstraints();
		c.weightx = 1.0;
		c.weighty = 1.0;
		c.gridx = 0;
		c.gridy = 1;
		c.anchor = GridBagConstraints.NORTHEAST;
		c.weighty = 1.0;
		c.weightx = 1.0;
		contentsPanel.add(scrollPaneList, c);
		
		c = new GridBagConstraints();

		c.gridy = 1;
		c.gridx = 1;
		c.fill = GridBagConstraints.HORIZONTAL;
		c.weightx = 0.0;
		c.weighty = 1.0;
		c.anchor = GridBagConstraints.NORTHWEST;
		c.fill = GridBagConstraints.NONE;
		contentsPanel.add(panelButtons, c);

		this.setLayout(new GridBagLayout());
		this.add(contentsPanel);
		
		buttonRemove.addActionListener(this);
        buttonEnableDisable.addActionListener(this);
        buttonAdd.addActionListener(this);
        buttonEdit.addActionListener(this);
        
	}
	
	// listens for different buttons which are pressed
	public void actionPerformed (ActionEvent e) {
		String command = e.getActionCommand();
		NoticeTrigger trigger = (NoticeTrigger)(listNotices.getSelectedValue());
		
		if (trigger != null) {
			if ("enable".equals(command)) {
				(trigger).setEnabled(true);
				buttonDisable();
			}
			else if ("disable".equals(command))
			{
				(trigger).setEnabled(false);
				buttonEnable();
			}
			else if ("remove".equals(command)) {
				triggers.remove(trigger);
				listNotices.clearSelection();
			}
			else if ("edit".equals(command)) {
				NoticeDialog noticeDialog = new NoticeDialog(trigger);
			}
		}
		if ("add".equals(command)) {
			if (!variables.isEmpty()) {
				NoticeDialog noticeDialog = new NoticeDialog();
			}
		}
		this.updateUI();
	}

	// initializes the JList for the current notices
	private void initListNotices() {
		listNotices = new JList(triggers);
		listNotices.setPrototypeCellValue("Just a value to fix the width of the component");
		listNotices.setSelectionMode(javax.swing.ListSelectionModel.SINGLE_SELECTION);
		buttonEnable();
		
		listNotices.addListSelectionListener(new ListSelectionListener() {
			public void valueChanged (ListSelectionEvent e) {
				if (!e.getValueIsAdjusting() && !listNotices.isSelectionEmpty()) {
					NoticeTrigger current = (NoticeTrigger)listNotices.getSelectedValue();
					if (current.isEnabled()) {
						buttonDisable();
					}
					else {
						buttonEnable();
					}
				}
					
			}
		});
	}

	private void buttonEnable() {
		buttonEnableDisable.setText(
				Localization.getString("panel.settings.tsc.enable"));
		buttonEnableDisable.setActionCommand("enable");
	}
	
	private void buttonDisable() {
		buttonEnableDisable.setText(
				Localization.getString("panel.settings.tsc.disable"));
			buttonEnableDisable.setActionCommand("disable");
	}

	// private dialog for editing or adding a notice trigger
	private class NoticeDialog extends JDialog implements ActionListener {

		private NoticeTrigger tempTrigger;
		private NoticeTrigger trigger;
		private boolean editing;
		
		private JPanel buttonPanel;
		private JPanel fieldsPanel;
		private JLabel noticeLabel;
		private JButton acceptButton;
		private JButton cancelButton;
		private JTextField noticeText;
		private JLabel valueLabel;
		private JTextField valueText;
		private JLabel errorField;
		private JLabel variableLabel;
		private JComboBox menuVariables;
		private JComboBox menuTests;
		private String [] tests = new String[6];
		
		// add notice
		public NoticeDialog() {
			super (owner, Localization.getString("panel.settings.tsc.notice_add"), true);

			initComponents();

			this.editing = false;
		}
		
		// edit notice
		public NoticeDialog(NoticeTrigger trigger) {
			super (owner, Localization.getString("panel.settings.tsc.notice_edit"), true);
			this.editing = true;
			this.trigger = trigger;
			this.tempTrigger = new NoticeTrigger(trigger);

			initComponents();
		}
		
		// possible tests the trigger can utilize
		private void initTests() {
			tests[NoticeTrigger.EQUALS] = Localization.getString("panel.settings.tsc.notice_equals");
			tests[NoticeTrigger.EXISTS] = Localization.getString("panel.settings.tsc.notice_exists");
			tests[NoticeTrigger.GREATER] = Localization.getString("panel.settings.tsc.notice_greater");
			tests[NoticeTrigger.GREATER_OR_EQUALS] = Localization.getString("panel.settings.tsc.notice_greater_or_equals");
			tests[NoticeTrigger.SMALLER] = Localization.getString("panel.settings.tsc.notice_smaller");
			tests[NoticeTrigger.SMALLER_OR_EQUALS] = Localization.getString("panel.settings.tsc.notice_smaller_or_equals");
		}
		
		// Initializes the Swing components
		private void initComponents() {
			initTests();
			
			this.getContentPane().setLayout(new BorderLayout());
			fieldsPanel = new JPanel();

			noticeLabel = new JLabel(Localization.getString("panel.settings.tsc.notice_text"));
			noticeText = new JTextField(20);
			variableLabel = new JLabel(Localization.getString("panel.settings.tsc.notice_variable"));
			
			Vector variablesVector = new Vector((Collection)variables);
			menuVariables = new JComboBox(variablesVector);
			menuVariables.setRenderer(new NoticeDialogCellRenderer());
			menuTests = new JComboBox((Object [])tests);
			
			errorField = new JLabel();
			errorField.setForeground(java.awt.Color.RED);

			valueLabel = new JLabel(Localization.getString("panel.settings.tsc.notice_value"));
			valueText = new JTextField(6);
			
			fieldsPanel.setLayout(new GridBagLayout());
			GridBagConstraints c = new GridBagConstraints();
	        c.insets = new Insets(2,2,2,2);

			c.fill = GridBagConstraints.VERTICAL;
			c.weightx = 1.0;
			c.anchor = GridBagConstraints.EAST;
			c.gridy = 0;
			c.gridx = 0;
			fieldsPanel.add(noticeLabel, c);
			c.anchor = GridBagConstraints.WEST;
			c.gridx = 1;
			fieldsPanel.add(noticeText, c);
			c.anchor = GridBagConstraints.EAST;
			c.gridy = 1;
			c.gridx = 0;
			fieldsPanel.add(variableLabel, c);
			c.anchor = GridBagConstraints.WEST;
			c.gridx = 1;
			fieldsPanel.add(menuVariables, c);
			c.gridy = 2;
			c.gridx = 1;
			fieldsPanel.add(menuTests, c);
			c.anchor = GridBagConstraints.EAST;
			c.gridy = 3;
			c.gridx = 0;
			fieldsPanel.add(valueLabel, c);
			c.anchor = GridBagConstraints.WEST;
			c.gridx = 1;
			fieldsPanel.add(valueText, c);
			valueText.setPreferredSize(menuVariables.getPreferredSize());
			noticeText.setPreferredSize(menuVariables.getPreferredSize());
			c.gridy = 4;
			c.gridx = 1;
			c.anchor = GridBagConstraints.WEST;
			fieldsPanel.add(errorField, c);
			errorField.setText(" ");

			if (editing) 
				// fill the fields of the dialog
				initDialogValues();
				
			buttonPanel = new JPanel();
			acceptButton = new JButton(Localization.getString("ui.button_accept"));
			cancelButton = new JButton(Localization.getString("ui.button_cancel"));
			acceptButton.setActionCommand("accept");
			cancelButton.setActionCommand("cancel");
			acceptButton.addActionListener(this);
			cancelButton.addActionListener(this);
			
			buttonPanel.setLayout(new FlowLayout());
			buttonPanel.add(acceptButton);
			buttonPanel.add(cancelButton);

			menuTests.addItemListener(new ItemListener() {
				public void itemStateChanged (ItemEvent e) {
					if (menuTests.getSelectedIndex() == NoticeTrigger.EXISTS)
						valueText.setEnabled(false);
					else
						valueText.setEnabled(true);
				}
			});
			
			this.getContentPane().add(fieldsPanel, BorderLayout.CENTER);
			this.getContentPane().add(buttonPanel, BorderLayout.SOUTH);

			this.pack();
			this.setLocationRelativeTo(owner);
			this.setVisible(true);
		}

		// fills the fields of the dialog
		private void initDialogValues() {
			noticeText.setText(tempTrigger.getNotice());
			valueText.setText(tempTrigger.getValue());
			menuVariables.setSelectedItem(tempTrigger.getVariable());
			menuTests.setSelectedItem(tests[tempTrigger.getType()]);
		}
		
		public void actionPerformed(ActionEvent e) {
			String command = e.getActionCommand();
			
			if ("accept".equals(command)) {
				if (verifyTriggerValue() || menuTests.getSelectedIndex()==NoticeTrigger.EXISTS) {
					if (editing) {
						this.trigger.setVariable((VariableDefinition)(menuVariables.getSelectedItem()));
						this.trigger.setNotice(noticeText.getText());
						this.trigger.setType(menuTests.getSelectedIndex());
						this.trigger.setValue(valueText.getText());
					}
					else {
						auxSettingsTSC.addNoticeTrigger(new NoticeTrigger(
								(VariableDefinition)(menuVariables.getSelectedItem()),
								valueText.getText(),
								menuTests.getSelectedIndex(),
								noticeText.getText()));
					}
					triggers = auxSettingsTSC.getVectorNoticeTriggers();
					listNotices.updateUI();
					this.setVisible(false);
				}
				else {
					// Given value could not be parsed to a number
					errorField.setText(Localization.getString("panel.settings.tsc.error_noticevalue"));
				}
			}
			else if ("cancel".equals(command)) {
				this.setVisible(false);
			}
		}

		// Tests if the user given value can be parsed into a number
		private boolean verifyTriggerValue() {
			String value = valueText.getText();
			try {
				Double.parseDouble(value);
				return true;
			}
			catch (Exception e) {
				if (e instanceof NumberFormatException) {
					return false;
				}
				else {
					e.printStackTrace();
					return false;
				}
			}
		}
		
		// renders the variable field list
	    class NoticeDialogCellRenderer extends BasicComboBoxRenderer {

	        public NoticeDialogCellRenderer() {
	            super();
	        }
	        
	        public Component getListCellRendererComponent(
	            JList list,
	            Object value,
	            int index,
	            boolean isSelected,
	            boolean cellHasFocus)
	        {
	            super.getListCellRendererComponent(list,value,index,isSelected,cellHasFocus);
	            super.setText(((VariableDefinition)value).getFullName());
	            return this;
	        }      
	    }
	}
}
