/*
 * Created on 14.4.2005
 */
package fi.helsinki.dacopan.ui;

import java.awt.BorderLayout;
import java.awt.Component;
import java.awt.GridLayout;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.util.HashMap;
import java.util.HashSet;
import java.util.Iterator;
import java.util.List;
import java.util.Map;
import java.util.Set;
import java.util.Vector;

import javax.swing.DefaultListCellRenderer;
import javax.swing.JButton;
import javax.swing.JLabel;
import javax.swing.JList;
import javax.swing.JPanel;
import javax.swing.JScrollPane;
import javax.swing.border.EmptyBorder;

import fi.helsinki.dacopan.Localization;
import fi.helsinki.dacopan.model.DataView;
import fi.helsinki.dacopan.model.Layer;
import fi.helsinki.dacopan.model.Protocol;
import fi.helsinki.dacopan.model.VariableDefinition;
import fi.helsinki.dacopan.ui.tsc.SettingsTSC;

/**
 * @author mjvaanan
 */
public class SettingsPanelTSCVariables extends JPanel{
	
	private SettingsTSC settingsTSC;
	private DataView data;
	private Map selections = new HashMap(); // Integer -> Boolean
	
	public SettingsPanelTSCVariables (DataView data, SettingsTSC settingsTSC) {
		this.data = data;
		this.settingsTSC = settingsTSC;
		initComponents();
	}
	
	private void initComponents() {
		setLayout(new BorderLayout());
		Vector tcpVars = getVariables(2, "tcp");
		final JList tcp_list = new JList(tcpVars);
		tcp_list.setCellRenderer(new SettingsPanelCellRenderer());
		JScrollPane scrollPane2 = new JScrollPane();
		scrollPane2.setViewportView(tcp_list);
		tcp_list.addMouseListener(new java.awt.event.MouseAdapter() {
            public void mousePressed(java.awt.event.MouseEvent evt) {
            	selectListSelection(tcp_list);
            	updateList(tcp_list);
            	updateSettings(tcp_list);
            }
        });
		tcp_list.addMouseMotionListener(new java.awt.event.MouseMotionAdapter() {
            public void mouseDragged(java.awt.event.MouseEvent evt) {
            	updateList(tcp_list);	// undoes the dragging selection effect
            }
        });
		JPanel btnPanel = new JPanel(new GridLayout(2,1));
		JButton btnSelectAll = new JButton(Localization.getString("panel.settings.tsc.variables.select_all"));
		JButton btnDeselectAll = new JButton(Localization.getString("panel.settings.tsc.variables.deselect_all"));
		btnSelectAll.addActionListener(new ActionListener() {
			public void actionPerformed(ActionEvent arg) {
				selectAll(tcp_list);
            	updateList(tcp_list);
            	updateSettings(tcp_list);
			}
		});
		btnDeselectAll.addActionListener(new ActionListener() {
			public void actionPerformed(ActionEvent arg) {
				deselectAll(tcp_list);
            	updateList(tcp_list);
            	updateSettings(tcp_list);
			}
		});
		btnPanel.add(btnSelectAll);
		btnPanel.add(btnDeselectAll);
		JPanel btnPanel2 = new JPanel();
		btnPanel2.add(btnPanel);
		
		JPanel panel = new JPanel(new BorderLayout());
		panel.setBorder(new EmptyBorder(5, 5, 5, 5));
		panel.add(new JLabel(Localization.getString("panel.settings.tsc.variables")), BorderLayout.NORTH);
		panel.add(scrollPane2, BorderLayout.CENTER);
		panel.add(btnPanel2, BorderLayout.EAST);
		
		add(panel, BorderLayout.WEST);
		add(new JPanel(), BorderLayout.CENTER);
		
		VariableDefinition[] all_defs = (VariableDefinition[]) tcpVars.toArray(new VariableDefinition[0]);
		initSelections(all_defs);
		updateList(tcp_list);
	}
	
	private void selectAll(JList variablesList) {
		Integer[] keys = (Integer[]) selections.keySet().toArray(new Integer[0]);
		for (int i = 0; i < keys.length; i++) {
			Integer key = keys[i];
			selections.put(key, new Boolean(true));
		}
	}
	
	private void deselectAll(JList variablesList) {
		Integer[] keys = (Integer[]) selections.keySet().toArray(new Integer[0]);
		for (int i = 0; i < keys.length; i++) {
			Integer key = keys[i];
			selections.put(key, new Boolean(false));
		}
	}
		
	private void selectListSelection(JList variablesList) {
		int i = variablesList.getSelectedIndex();
		if (i == -1) {
			return;
		}
		Integer index = new Integer(i);
		Boolean value = (Boolean) selections.get(index);
		boolean newValue = !value.booleanValue();
		selections.put(index, new Boolean(newValue));
	}
	
	private void updateSettings(JList variablesList) {
		Object[] aux = variablesList.getSelectedValues();
		String[] vars = new String[aux.length];
		for (int i = 0; i < aux.length; i++) {
			VariableDefinition def = (VariableDefinition) aux[i];
			vars[i] = def.getRawName();
		}
		settingsTSC.setDisplayedData(vars);
    }
    
    private void updateList(JList variablesList) {
    	Integer[] keys = (Integer[]) selections.keySet().toArray(new Integer[0]);
    	Vector result = new Vector();
    	for (int i = 0; i < keys.length; i++) {
    		Boolean value = (Boolean) selections.get(keys[i]);
    		if (value.booleanValue()) {
    			result.add(keys[i]);
    		}
		}
    	int[] selected = new int[result.size()];
    	for (int i = 0; i < selected.length; i++) {
    		Integer key = (Integer) result.elementAt(i);
    		selected[i] = key.intValue();
    	}
		variablesList.setSelectedIndices(selected);
    }
    
    private void initSelections(VariableDefinition[] all_defs) {
    	String[] variables = settingsTSC.getDisplayedData();
		Set enabled = new HashSet();
    	for (int i = 0; i < variables.length; i++) {
    		enabled.add(variables[i]);
    	}
    	for (int i = 0; i < all_defs.length; i++) {
			VariableDefinition def = (VariableDefinition) all_defs[i];
			String rawname = def.getRawName(); 
			if (enabled.contains(rawname)) {
				selections.put(new Integer(i), new Boolean(true));
			} else {
				selections.put(new Integer(i), new Boolean(false));
			}
		}
    }
	
	private Vector getVariables(int layer_id, String protocol_name) {
		List layers = data.getLayers();
		Iterator iterator = layers.iterator();
		Layer net_layer = null;
		while (iterator.hasNext()) {
			Layer layer = (Layer) iterator.next();
			if (layer.getId() == layer_id) {
				net_layer = layer;
				break;
			}
		}
		Vector result = new Vector();
		if (net_layer != null) {
			List variableDefinitionList = data.getAvailableVariables(net_layer);
			Iterator var_iter = variableDefinitionList.iterator();
			while (var_iter.hasNext()) {
				VariableDefinition var = (VariableDefinition) var_iter.next();
				Protocol protocol = var.getProtocol();
				if (protocol.getRawName().equalsIgnoreCase(protocol_name)) {
					result.add(var);
				}
			}
		}
		return result;
	}

	
	/** 
     * Internal Class that contains information about how the JList should be rendered
     */
    class SettingsPanelCellRenderer extends DefaultListCellRenderer {

        public SettingsPanelCellRenderer() {
            super();
        }
        
        public Component getListCellRendererComponent(
            JList list,
            Object value,
            int index,
            boolean isSelected,
            boolean cellHasFocus)
        {
            super.getListCellRendererComponent(list,value,index,isSelected,cellHasFocus);
            super.setText(((VariableDefinition)value).getProtocol().getName()+"-"+((VariableDefinition)value).getFullName());
            return this;         
        }      
    }
}
