/*
 * TimePanel.java
 *
 * Created on 5  2004 ., 11:27
 */

package fi.helsinki.dacopan.ui;

import java.awt.BorderLayout;
import java.text.NumberFormat;

import javax.swing.BorderFactory;
import javax.swing.JLabel;

import fi.helsinki.dacopan.Localization;

/**
 * A very simple animation panel just showing 
 * the "now time" as text.
 */
public class TimePanel extends AbstractAnimationPanel {
    private static TimePanel instance;
    
    private NumberFormat numberFormat;
    private JLabel label;
    
    private MainFrame mainFrame;
    
    /** Creates a new instance of TimePanel */
    private TimePanel(MainFrame mainFrame, float startTime) {
        super(mainFrame);
               
        this.mainFrame = mainFrame;
        this.numberFormat = NumberFormat.getInstance(Localization.getCurrentLocale());
        this.numberFormat.setMaximumFractionDigits(4);
        this.numberFormat.setMinimumFractionDigits(4);

        setBorder(BorderFactory.createEtchedBorder());        
        setLayout(new BorderLayout());
        this.label = new JLabel(Localization.getString("panel.time.display", this.numberFormat.format(new Float(startTime))));        //$NON-NLS-1$
        add(this.label, BorderLayout.CENTER);
        label.setFont(StatusBar.STATUS_BAR_FONT);
        
        // This means that this instance of the time panel 
        // should be used only as a placeholder for empty time
        if(mainFrame == null || !(mainFrame.isInMSCMode() || mainFrame.isInENCMode())) {
            this.label.setEnabled(false);
        }
    }
    
    public static TimePanel getInstance(MainFrame mainFrame, float startTime) {
    	if (instance == null) {
    		instance = new TimePanel(mainFrame, startTime);
    	}
    	instance.stepTo(startTime);
    	return instance;
    }
    
    public void stepTo(float nowTime) {
    	if (mainFrame != null && (mainFrame.isInMSCMode() || mainFrame.isInENCMode())) {
    		this.label.setEnabled(true);
    		float animationTime = mainFrame.getMSCPanel().getRealTime();
	        String newText = Localization.getString("panel.time.display", this.numberFormat.format(animationTime)); //$NON-NLS-1$
            this.label.setText(newText);
    	} else {
    		this.label.setText(Localization.getString("panel.time.display", "-.-"));
    		this.label.setEnabled(false);
    	}
    }
    
}
