package fi.helsinki.dacopan.ui;

import java.awt.BorderLayout;
import java.awt.Color;
import java.awt.Component;
import java.awt.Container;
import java.awt.Dimension;
import java.awt.Font;
import java.awt.Toolkit;
import java.awt.Window;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.awt.event.KeyEvent;
import java.io.File;
import java.io.IOException;
import java.io.InputStream;
import java.io.InputStreamReader;
import java.net.URL;
import java.util.Iterator;

import javax.swing.ButtonGroup;
import javax.swing.ImageIcon;
import javax.swing.JButton;
import javax.swing.JCheckBoxMenuItem;
import javax.swing.JComponent;
import javax.swing.JDialog;
import javax.swing.JEditorPane;
import javax.swing.JFileChooser;
import javax.swing.JFrame;
import javax.swing.JLabel;
import javax.swing.JMenu;
import javax.swing.JMenuBar;
import javax.swing.JMenuItem;
import javax.swing.JOptionPane;
import javax.swing.JPanel;
import javax.swing.JRadioButtonMenuItem;
import javax.swing.JScrollPane;
import javax.swing.JSplitPane;
import javax.swing.JTabbedPane;
import javax.swing.JTextArea;
import javax.swing.JToggleButton;
import javax.swing.JToolBar;
import javax.swing.KeyStroke;
import javax.swing.MenuElement;
import javax.swing.SwingConstants;
import javax.swing.UIDefaults;
import javax.swing.UIManager;
import javax.swing.event.HyperlinkEvent;
import javax.swing.event.HyperlinkListener;
import javax.swing.text.html.HTMLDocument;
import javax.swing.text.html.HTMLFrameHyperlinkEvent;

import fi.helsinki.dacopan.Localization;
import fi.helsinki.dacopan.contsig.AnimationTimeState;
import fi.helsinki.dacopan.model.Layer;

/**
 * This class handles the user interface part of the Main Frame so that 
 * the main frame itself can concentrate in controlling the flow of the program.
 */
public class UserInterface {
     
    /** The title font used in all animation panels */
    public static final Font DACOPAN_FONT_TITLE = new Font("Verdana", Font.BOLD, 12);

    /** The font to be used in the buttons */    
    public static final Font DACOPAN_FONT_BUTTON = new Font("Verdana", Font.PLAIN, 11);
    
    /** The font for the menu */
    public static final Font DACOPAN_FONT_MENU = new Font("Verdana", Font.PLAIN, 12);

    /** The size for the splitpane divider in pixels */
    public static final int DACOPAN_DIVIDER_SIZE = 5;
    
    /** The default width of the main window */
    public static final int DEFAULT_DISP_MODE_WIDTH = 800; 
    
    /** The default height of the main window */
    public static final int DEFAULT_DISP_MODE_HEIGHT = 570; 
    
    /** The default ratio of UFO to NotePanel */
    public static final float DEFAULT_RATIO_UFO_2_NOTE = 0.55f;
    
    /** The default ratio of the main panel to the right panel */ 
    public static final float DEFAULT_RATIO_MAIN_2_RIGHT = 0.70f; 
    
    /** Width of the help window */
    public static final int HELP_DIALOG_WIDTH = 900;
    
    /** Height of the help window */    
    public static final int HELP_DIALOG_HEIGHT = 700;
    
    // Icons for the menu and tool bar
    private static final String DACOPAN_ICON_LANG = "/toolbarButtonGraphics/navigation/Home16.gif";
    private static final String DACOPAN_ICON_SETTINGS = "/toolbarButtonGraphics/general/Preferences24.gif";
    private static final String DACOPAN_ICON_SETTINGS16 = "/toolbarButtonGraphics/general/Preferences16.gif";
    private static final String DACOPAN_ICON_LOAD = "/toolbarButtonGraphics/general/Open24.gif";
    private static final String DACOPAN_ICON_LOAD16 = "/toolbarButtonGraphics/general/Open16.gif";
    private static final String DACOPAN_ICON_SAVE = "/toolbarButtonGraphics/general/Save24.gif";
    private static final String DACOPAN_ICON_SAVE16 = "/toolbarButtonGraphics/general/Save16.gif";
    //private static final String DACOPAN_ICON_DEVELOPERS = "/team.jpg";
    private static final String DACOPAN_ICON_LOGO = "/logo-header.jpg";
    private static final String DACOPAN_ICON_STEPFORWARD = "/toolbarButtonGraphics/media/StepForward24.gif";
    private static final String DACOPAN_ICON_STEPBACK = "/toolbarButtonGraphics/media/StepBack24.gif";
    private static final String DACOPAN_ICON_FORWARD = "/toolbarButtonGraphics/media/FastForward24.gif";
    private static final String DACOPAN_ICON_STOP = "/toolbarButtonGraphics/media/Pause24.gif";
    private static final String DACOPAN_ICON_PLAY = "/toolbarButtonGraphics/media/Play24.gif";
    private static final String DACOPAN_ICON_REWIND = "/toolbarButtonGraphics/media/Rewind24.gif";
    private static final String DACOPAN_ICON_STEPFORWARD16 = "/toolbarButtonGraphics/media/StepForward16.gif";
    private static final String DACOPAN_ICON_STEPBACK16 = "/toolbarButtonGraphics/media/StepBack16.gif";
    private static final String DACOPAN_ICON_FORWARD16 = "/toolbarButtonGraphics/media/FastForward16.gif";
    private static final String DACOPAN_ICON_STOP16 = "/toolbarButtonGraphics/media/Pause16.gif";
    private static final String DACOPAN_ICON_PLAY16 = "/toolbarButtonGraphics/media/Play16.gif";
    private static final String DACOPAN_ICON_REWIND16 = "/toolbarButtonGraphics/media/Rewind16.gif";
    private static final String DACOPAN_ICON_TIPS16 = "/toolbarButtonGraphics/general/TipOfTheDay16.gif";
//addition
	private static final String DACOPAN_ICON_TSC="/toolbarButtonGraphics/media/test1.png";
	private static final String DACOPAN_ICON_MSC="/toolbarButtonGraphics/media/test2.png";
	
	// The default colors used for the GUI widgets.
	public static final Color DACOPAN_COLOR_ACTIVE_CAPTION_TEXT = new Color(255,255,255);
	public static final Color DACOPAN_COLOR_INACTIVE_CAPTION_TEXT = new Color(0,0,0);
	public static final Color DACOPAN_COLOR_ACTIVE_CAPTION_BG = new Color(50,70,160);
	public static final Color DACOPAN_COLOR_INACTIVE_CAPTION_BG = new Color(128,128,128);
	public static final Color DACOPAN_COLOR_HIGHLIGHT = new Color(160,160,160);
	public static final Color DACOPAN_COLOR_LTHIGHLIGHT = new Color(230,230,230);
	public static final Color DACOPAN_COLOR_DKHIGHLIGHT = new Color(74,74,74);
	public static final Color DACOPAN_COLOR_MAIN_PANE_BG = new Color(50,70,160);
	public static final Color DACOPAN_COLOR_PANE_BG = new Color(196,196,196);
	public static final Color DACOPAN_COLOR_PANE_TEXT = new Color(0,0,0);
	public static final Color DACOPAN_COLOR_WINDOW = new Color(196,196,196);
//	
    private static final String DACOPAN_LICENSE_RESOURCE = "/dacopan-license.txt";

    /**
     * This private class is only used to encapsulate
     * UI controls to one place so that they are easier
     * to update from different parts of the User Interface. 
     */
    private class UIControls {
        JButton saveButton;
        JButton loadButton;
        JButton rewindButton;
        JButton forwardButton;
        JButton stepBackButton;
        JButton stepForwardButton;
        JButton settingsButton;
        JButton stopButton;
        JButton playButton;
        JButton mscButton;
        JButton tscButton;
        JPanel  layerPanel;
        
        JRadioButtonMenuItem menuItemExploreMode;
        JRadioButtonMenuItem menuItemScenarioMode;
        JMenuItem menuItemSave;
        JMenuItem menuItemSaveAs;

        JMenuItem menuItemPlay;
        JMenuItem menuItemPause;
        JMenuItem menuItemRewind;
        JMenuItem menuItemForward;
        JMenuItem menuItemStepForward;
        JMenuItem menuItemStepBackward;
        
        JMenu menuSelectLayer;
        
        JMenuItem menuItemSettings;
        JMenuItem menuItemGeneralSettings;
        JMenuItem menuItemMSCSettigs;
        JMenuItem menuItemTSCSettings;
        JMenuItem menuItemSaveAllSettings;
    }
    
    /**
     * This private class is used to listen the action events 
     * from the layer change buttons.
     */
    private class LayerButtonActionListener implements ActionListener {
        private Layer targetLayer;

        public LayerButtonActionListener(Layer layer) {
            targetLayer = layer;            
        }

        public void actionPerformed(ActionEvent e) {
            mainFrame.changeLayer(targetLayer);
        }
    }

    /** The main frame reference */
    private MainFrame mainFrame;
    
    /** The tool bar */
    private JToolBar toolBar;
    
    /** The status bar */
    private StatusBar statusBar;
    
    /** The menu */
    private JMenuBar menuBar;
    
    /** A "container" for all UI controls */
    private UIControls uiControls;
    
    /** The split pane for separating UFO from Notes */
    private JSplitPane verticalSplitPane = null;
    
    /** 
     * The split pane for separating the main area from the 
     * secondary area (special forces) 
     */
    private JSplitPane mainSplitPane = null;

    /** the settings dialog */
    private SettingsDialog settingsDialog;
    
	private static final int HELP_MODE_USER_MANUAL = 0;
	private static final int HELP_MODE_FAQ = 1;

    /**
     * Creates the UI for the animator.
     * 
     * @param frame A reference to the MainFrame
     */
    public UserInterface(MainFrame frame) {

        mainFrame = frame;
        uiControls = new UIControls();
        
        // Do the general UI settings
        frame.setSize(DEFAULT_DISP_MODE_WIDTH, DEFAULT_DISP_MODE_HEIGHT);
        frame.setTitle(Localization.getString("ui.frame_title")); //$NON-NLS-1$
        frame.setDefaultCloseOperation(JFrame.EXIT_ON_CLOSE);
        frame.setBackground(DACOPAN_COLOR_WINDOW);
        frame.setForeground(DACOPAN_COLOR_WINDOW);
        
        // center frame on the screen
        centerWindow(mainFrame);

        // Colors
        setupDefaultColors();
              
        // Create the layout
        Container contentPane = frame.getContentPane();
        contentPane.setLayout(new BorderLayout());
        
        // Main frame UI
        mainSplitPane = createMainLayout();
        mainSplitPane.setBackground(DACOPAN_COLOR_PANE_BG);
        mainSplitPane.setForeground(DACOPAN_COLOR_PANE_BG);
        mainSplitPane.setOpaque(true);
        mainSplitPane.setResizeWeight(1);
        contentPane.add(mainSplitPane, BorderLayout.CENTER);

        // building a toolbar
        toolBar = createToolBar();
        toolBar.setBackground(DACOPAN_COLOR_PANE_BG);
        contentPane.add(toolBar, BorderLayout.NORTH);

        // Add the menu bar and all its actions
        menuBar = createMenuBar();
        frame.setJMenuBar(menuBar);

        // Add the status bar
        statusBar = new StatusBar(mainFrame);
        contentPane.add(statusBar, BorderLayout.SOUTH);
    }
    
    /**
     * Centers a window on the screen.
     */
    private void centerWindow(Window window) {
        Dimension dim = Toolkit.getDefaultToolkit().getScreenSize();
        int w = window.getSize().width;
        int h = window.getSize().height;
        int x = (dim.width-w)/2;
        int y = (dim.height-h)/2;
        window.setLocation(x, y);
    }

    /**
     * Sets up the default colors so that all UI components
     * have the same look and feel.
     */
    private void setupDefaultColors() {
               
        UIDefaults uiDefaults = UIManager.getDefaults();
               
        uiDefaults.put("Menu.border", DACOPAN_COLOR_PANE_BG);
        uiDefaults.put("Button.background", DACOPAN_COLOR_PANE_BG);
        uiDefaults.put("Button.highlight", DACOPAN_COLOR_LTHIGHLIGHT);
        uiDefaults.put("Separator.highlight", DACOPAN_COLOR_LTHIGHLIGHT);
        uiDefaults.put("Separator.foreground", DACOPAN_COLOR_PANE_BG);
        uiDefaults.put("ToolBar.dockingBackground", DACOPAN_COLOR_PANE_BG);
        uiDefaults.put("TabbedPane.background", DACOPAN_COLOR_PANE_BG);
        uiDefaults.put("List.selectionBackground", DACOPAN_COLOR_HIGHLIGHT);
        
    }
    
    /**
     * Creates the split panes and their internal structure
     * @return
     */
    private JSplitPane createMainLayout() {
        /*
         * When the animator is started, show a welcome panel on 
         * the empty place where the MSC or ENC animation will be
         * once the file is loaded
         */
        JEditorPane welcomePane = null;
        URL welcomeURL = mainFrame.getClass().getResource("/welcome.html");
        
        try {
            welcomePane = new JEditorPane(welcomeURL);
            welcomePane.setEditable(false);
            welcomePane.setOpaque(true);
        } catch (IOException e) {
            System.out.println("The welcome text wasn't found");
        }

        /*
         * Divide the frame in two parts: main animation area and "special area" :) 
         * The secondary area (right pane) is then divided again in two parts.
         * 
         * We don't put anything in the frames, as no data has been loaded.
         * 
         * Probably it would be a good idea not to even add the panes to the 
         * MainFrame until some data is loaded
         */
        verticalSplitPane = new JSplitPane(JSplitPane.VERTICAL_SPLIT, null, null);
        verticalSplitPane.setDividerSize(DACOPAN_DIVIDER_SIZE);
        verticalSplitPane.setDividerLocation((int) (DEFAULT_DISP_MODE_HEIGHT * DEFAULT_RATIO_UFO_2_NOTE));
        verticalSplitPane.setResizeWeight(0.5);

        mainSplitPane = new JSplitPane(JSplitPane.HORIZONTAL_SPLIT, welcomePane, verticalSplitPane);
        mainSplitPane.setOneTouchExpandable(true);
        mainSplitPane.setDividerLocation((int) (DEFAULT_DISP_MODE_WIDTH - 290));// (DEFAULT_RATIO_MAIN_2_RIGHT)));

        return mainSplitPane;
    }

    /**
     * Creates the tool bar and adds buttons to it
     */
    private JToolBar createToolBar() {
               
        JToolBar toolBar = new JToolBar(JToolBar.HORIZONTAL);
        addToolBarButtons(toolBar);
        
        toolBar.setFloatable(false); // user cannot move it
        toolBar.setRollover(true); 

        return toolBar;
    }

    /**
     * Adds buttons to the tool bar
     */ 
    private void addToolBarButtons(final JToolBar toolBar) {

        uiControls.loadButton = createButton(mainFrame, DACOPAN_ICON_LOAD, null);
        uiControls.loadButton.setToolTipText(Localization.getString("ui.tooltip_load")); //$NON-NLS-1$
        uiControls.loadButton.addActionListener(new ActionListener() {
            public void actionPerformed(ActionEvent arg0) {
                loadFile();
            }
        });
        toolBar.add(uiControls.loadButton);

        uiControls.saveButton = createButton(mainFrame, DACOPAN_ICON_SAVE, null);
        uiControls.saveButton.setToolTipText(Localization.getString("ui.tooltip_save")); //$NON-NLS-1$
        uiControls.saveButton.addActionListener(new ActionListener() {
            public void actionPerformed(ActionEvent arg0) {
                mainFrame.saveAnimationFile(false);
            }
        });
        toolBar.add(uiControls.saveButton);

        toolBar.addSeparator();

        uiControls.rewindButton = createButton(mainFrame, DACOPAN_ICON_REWIND, null);
        uiControls.rewindButton.setToolTipText(Localization.getString("ui.tooltip_rewind")); //$NON-NLS-1$
        uiControls.rewindButton.addActionListener(new ActionListener() {
            public void actionPerformed(ActionEvent arg0) {
                mainFrame.rewindAnimation();
            }
        });
        toolBar.add(uiControls.rewindButton);

        uiControls.playButton = createButton(mainFrame, DACOPAN_ICON_PLAY, null);
        uiControls.playButton.setToolTipText(Localization.getString("ui.tooltip_play")); //$NON-NLS-1$
        uiControls.playButton.addActionListener(new ActionListener() {
            public void actionPerformed(ActionEvent arg0) {
                mainFrame.playAnimation();
            }
        });
        toolBar.add(uiControls.playButton);

        uiControls.stopButton = createButton(mainFrame, DACOPAN_ICON_STOP, null);
        uiControls.stopButton.setToolTipText(Localization.getString("ui.tooltip_stop")); //$NON-NLS-1$
        uiControls.stopButton.addActionListener(new ActionListener() {
            public void actionPerformed(ActionEvent arg0) {
                mainFrame.pauseAnimation();
            }
        });
        toolBar.add(uiControls.stopButton);

        uiControls.forwardButton = createButton(mainFrame, DACOPAN_ICON_FORWARD, null);
        uiControls.forwardButton.setToolTipText(Localization.getString("ui.tooltip_forward")); //$NON-NLS-1$
        uiControls.forwardButton.addActionListener(new ActionListener() {
            public void actionPerformed(ActionEvent arg0) {
                mainFrame.fastForwardAnimation();
            }
        });
        toolBar.add(uiControls.forwardButton);

        toolBar.addSeparator();

        uiControls.stepBackButton = createButton(mainFrame, DACOPAN_ICON_STEPBACK, null);
        uiControls.stepBackButton.setToolTipText(Localization.getString("ui.tooltip_step_back")); //$NON-NLS-1$
        uiControls.stepBackButton.addActionListener(new ActionListener() {
            public void actionPerformed(ActionEvent arg0) {
                mainFrame.stepInAnimation(MainFrame.DACOPAN_STEP_BACKWARD);
            }
        });
        toolBar.add(uiControls.stepBackButton);

        uiControls.stepForwardButton = createButton(mainFrame, DACOPAN_ICON_STEPFORWARD, null);
        uiControls.stepForwardButton.setToolTipText(Localization.getString("ui.tooltip_step_forward")); //$NON-NLS-1$
        uiControls.stepForwardButton.addActionListener(new ActionListener() {
            public void actionPerformed(ActionEvent arg0) {
                mainFrame.stepInAnimation(MainFrame.DACOPAN_STEP_FORWARD);
            }
        });
        toolBar.add(uiControls.stepForwardButton);

        toolBar.addSeparator();

        uiControls.settingsButton = createButton(mainFrame, DACOPAN_ICON_SETTINGS, null);
        uiControls.settingsButton.setToolTipText(Localization.getString("ui.tooltip_change_settings")); //$NON-NLS-1$
        uiControls.settingsButton.addActionListener(new ActionListener() {
            public void actionPerformed(ActionEvent e) {
                changeSettings();
            }
        });
        toolBar.add(uiControls.settingsButton);
        toolBar.addSeparator();
        
        uiControls.tscButton = createButton(mainFrame, DACOPAN_ICON_TSC, null);
        uiControls.tscButton.setToolTipText(Localization.getString("ui.tooltip_tsc_button"));
        uiControls.tscButton.addActionListener(new ActionListener() {
            public void actionPerformed(ActionEvent e) {
                mainFrame.setupAnimationModeTSC();
                toolBar.remove(uiControls.layerPanel);
            }
        });
        toolBar.add(uiControls.tscButton);

		uiControls.mscButton = createButton(mainFrame, DACOPAN_ICON_MSC, null);
        uiControls.mscButton.setToolTipText(Localization.getString("ui.tooltip_msc_button"));
        uiControls.mscButton.addActionListener(new ActionListener() {
            public void actionPerformed(ActionEvent e) {
                mainFrame.continueAnimationModeMSC();
                toolBar.add(uiControls.layerPanel);
            }
        });
        toolBar.add(uiControls.mscButton);
        
        toolBar.addSeparator();
        toolBar.addSeparator();

        addLayerButtons(toolBar);
    }

    /**
     * Adds the layer buttons to the tool bar.  
     */
    private void addLayerButtons(JToolBar toolBar) {
        if (mainFrame.isInTSCMode()) {
        	return;
        }
    	
        if(uiControls.layerPanel != null) {
            toolBar.remove(uiControls.layerPanel);
            uiControls.layerPanel = null;
        }
                
        uiControls.layerPanel = new JPanel();                      
        uiControls.layerPanel.setBackground(DACOPAN_COLOR_PANE_BG);
        
        // Get the layers from the data and show them on the tool bar
        Iterator i = mainFrame.getDataView().getLayers().iterator();
        while (i.hasNext()) {
            Layer layer = (Layer) i.next();
            
            String layerName = layer.getName();
            
            JToggleButton layerButton = new JToggleButton(layerName);
            layerButton.setToolTipText(Localization.getString("ui.tooltip_show_layer", layer.getName()));
            layerButton.addActionListener(new LayerButtonActionListener(layer));
            setDefaultButtonFont(layerButton);            
            if(mainFrame.getCurrentLayer() != null && mainFrame.getCurrentLayer().equals(layer)) {
            	layerButton.setSelected(true);                   
            }
            
            uiControls.layerPanel.add(layerButton);
        }
        
        toolBar.add(uiControls.layerPanel);   
    }

    /**
     * Creates a menu bar to be used in the Main Frame and returns a 
     * reference to it so that it can easily be added to the frame.
     * 
     * @return a reference to the created menu bar
     */
    private JMenuBar createMenuBar() {
        JMenuBar menuBar;
        JMenu menu;
        JMenu submenu;

        JMenuItem menuItem;
        JRadioButtonMenuItem radioButtonMenuItem;
        JCheckBoxMenuItem checkBoxMenuItem;

        // Create the menu bar.
        menuBar = new JMenuBar();
        menuBar.setBackground(DACOPAN_COLOR_PANE_BG);

        // Build the "file" menu
        menu = createMenu("ui.menu_file");
        menuBar.add(menu);

        menuItem = createMenuItem("ui.menu_file_open", DACOPAN_ICON_LOAD16); //$NON-NLS-1$
		menuItem.setAccelerator(KeyStroke.getKeyStroke(KeyEvent.VK_O, ActionEvent.CTRL_MASK));
        menuItem.addActionListener(new ActionListener() {
            public void actionPerformed(ActionEvent e) {
                loadFile();
            }
        });
        menu.add(menuItem);

        uiControls.menuItemSave = createMenuItem("ui.menu_file_save", DACOPAN_ICON_SAVE16); //$NON-NLS-1$
        uiControls.menuItemSave.setAccelerator(KeyStroke.getKeyStroke(KeyEvent.VK_S, ActionEvent.CTRL_MASK));
        uiControls.menuItemSave.addActionListener(new ActionListener() {
            public void actionPerformed(ActionEvent e) {
                mainFrame.saveAnimationFile(false);
            }
        });
        menu.add(uiControls.menuItemSave);


        uiControls.menuItemSaveAs = createMenuItem("ui.menu_file_save_as", null); //$NON-NLS-1$        
        uiControls.menuItemSaveAs.addActionListener(new ActionListener() {
            public void actionPerformed(ActionEvent e) {
                mainFrame.saveAnimationFile(true);
            }
        });
        menu.add(uiControls.menuItemSaveAs);

        menu.addSeparator();
/*
        uiControls.menuItemSettings = createMenuItem("ui.menu_file_settings", DACOPAN_ICON_SETTINGS16); //$NON-NLS-1$
        uiControls.menuItemSettings.addActionListener(new ActionListener(){
            public void actionPerformed(ActionEvent arg0) {
            	changeSettings();
            }            
        });
        menu.add(uiControls.menuItemSettings);

        menu.addSeparator();
*/
        menuItem = createMenuItem("ui.menu_file_exit", null); //$NON-NLS-1$
        menuItem.addActionListener(new ActionListener(){
			public void actionPerformed(ActionEvent arg0) {
                mainFrame.dispose();
			}            
        });
        menu.add(menuItem);

        // Build the "animation" menu
        menu = createMenu("ui.menu_animation");
        menuBar.add(menu);

        uiControls.menuItemPlay = createMenuItem("ui.menu_animation_play", DACOPAN_ICON_PLAY16); //$NON-NLS-1$
        uiControls.menuItemPlay.addActionListener(new ActionListener() {
            public void actionPerformed(ActionEvent arg0) {
                mainFrame.playAnimation();
            }
        });
        menu.add(uiControls.menuItemPlay);

        uiControls.menuItemPause = createMenuItem("ui.menu_animation_pause", DACOPAN_ICON_STOP16); //$NON-NLS-1$
        uiControls.menuItemPause.addActionListener(new ActionListener() {
            public void actionPerformed(ActionEvent arg0) {
                mainFrame.pauseAnimation();
            }
        });
        menu.add(uiControls.menuItemPause);

        uiControls.menuItemRewind = createMenuItem("ui.menu_animation_rewind", DACOPAN_ICON_REWIND16); //$NON-NLS-1$
        uiControls.menuItemRewind.addActionListener(new ActionListener() {
            public void actionPerformed(ActionEvent arg0) {
                mainFrame.rewindAnimation();
            }
        });
        menu.add(uiControls.menuItemRewind);

        uiControls.menuItemForward = createMenuItem("ui.menu_animation_forward", DACOPAN_ICON_FORWARD16); //$NON-NLS-1$
        uiControls.menuItemForward.addActionListener(new ActionListener() {
            public void actionPerformed(ActionEvent arg0) {
                mainFrame.fastForwardAnimation();
            }
        });
        menu.add(uiControls.menuItemForward);

        menu.addSeparator();

        uiControls.menuItemStepForward = createMenuItem("ui.menu_animation_step_forward", DACOPAN_ICON_STEPFORWARD16); //$NON-NLS-1$
        uiControls.menuItemStepForward.addActionListener(new ActionListener() {
            public void actionPerformed(ActionEvent arg0) {
                mainFrame.stepInAnimation(MainFrame.DACOPAN_STEP_FORWARD);
            }
        });
        menu.add(uiControls.menuItemStepForward);

        uiControls.menuItemStepBackward = createMenuItem("ui.menu_animation_step_backward", DACOPAN_ICON_STEPBACK16); //$NON-NLS-1$
        uiControls.menuItemStepBackward.addActionListener(new ActionListener() {
            public void actionPerformed(ActionEvent arg0) {
                mainFrame.stepInAnimation(MainFrame.DACOPAN_STEP_BACKWARD);
            }
        });
        menu.add(uiControls.menuItemStepBackward);

        menu.addSeparator();

        // The animation mode selection
        ButtonGroup group = new ButtonGroup();
        
        uiControls.menuItemScenarioMode = createRadioButtonMenuItem("ui.mode_scenario", true);
        uiControls.menuItemScenarioMode.addActionListener(new ActionListener() {
            public void actionPerformed(ActionEvent event) {
                if (mainFrame.isInScenarioMode() == false) {
                    mainFrame.setAnimatorMode(MainFrame.DACOPAN_SCENARIO_MODE);
                    uiControls.tscButton.setEnabled(false);
                }
            }
        });
        uiControls.menuItemScenarioMode.setEnabled(false);
        group.add(uiControls.menuItemScenarioMode);
        menu.add(uiControls.menuItemScenarioMode);

        uiControls.menuItemExploreMode = createRadioButtonMenuItem("ui.mode_explore", false);
        uiControls.menuItemExploreMode.addActionListener(new ActionListener() {
            public void actionPerformed(ActionEvent event) {
                if (mainFrame.isInExploreMode() == false) {
                    mainFrame.setAnimatorMode(MainFrame.DACOPAN_EXPLORE_MODE);
                }
            }
        });
        uiControls.menuItemExploreMode.setEnabled(false);
        uiControls.menuItemExploreMode.setSelected(true);
        group.add(uiControls.menuItemExploreMode);
        menu.add(uiControls.menuItemExploreMode);

        // Build the "layer" menu
        uiControls.menuSelectLayer = createMenu("ui.menu_layer");
        menuBar.add(uiControls.menuSelectLayer);

        // Build the "settings" menu.
        menu = createMenu("ui.menu_settings");
        menu.setBackground(DACOPAN_COLOR_PANE_BG);
        menu.setFont(DACOPAN_FONT_MENU);
        menuBar.add(menu);
        


        uiControls.menuItemSettings = createMenuItem("ui.menu_settings_settings", DACOPAN_ICON_SETTINGS16); //$NON-NLS-1$
        uiControls.menuItemSettings.addActionListener(new ActionListener(){
            public void actionPerformed(ActionEvent arg0) {
            	changeSettings();
            }            
        });
        menu.add(uiControls.menuItemSettings);

        menu.addSeparator();

        
        uiControls.menuItemMSCSettigs = createMenuItem("ui.menu_settings_MSC", null);
        uiControls.menuItemMSCSettigs.addActionListener(new ActionListener(){
        	public void actionPerformed(ActionEvent arg0) {
        		changeSettingsMode(SettingsDialog.MODE_MSC);
        	}
        });
        menu.add(uiControls.menuItemMSCSettigs);
        
        uiControls.menuItemTSCSettings = createMenuItem("ui.menu_settings_TSC", null);
        uiControls.menuItemTSCSettings.addActionListener(new ActionListener(){
        	public void actionPerformed(ActionEvent arg0) {
        		changeSettingsMode(SettingsDialog.MODE_TSC);
        	}
        });
        menu.add(uiControls.menuItemTSCSettings);

        uiControls.menuItemGeneralSettings = createMenuItem("ui.menu_settings_general", null);
        uiControls.menuItemGeneralSettings.addActionListener(new ActionListener(){
        	public void actionPerformed(ActionEvent arg0) {
        		changeSettingsMode(SettingsDialog.MODE_GENERAL);
        	}
        });
        menu.add(uiControls.menuItemGeneralSettings);
        
        uiControls.menuItemSaveAllSettings = createMenuItem("ui.menu_settings_save", null);
        uiControls.menuItemSaveAllSettings.addActionListener(new ActionListener(){ 
        	public void actionPerformed(ActionEvent arg0) {
        		mainFrame.saveAllSettings();
        	}
        });
        menu.add(uiControls.menuItemSaveAllSettings);
        
        menu.addSeparator();
        
        // Build the "language" submenu.
        submenu = createMenu("ui.menu_language"); //$NON-NLS-1$
        submenu.setBackground(DACOPAN_COLOR_PANE_TEXT);
        submenu.setFont(DACOPAN_FONT_MENU);
        //menuBar.add(menu);

        ButtonGroup langGroup = new ButtonGroup();
        
        for(Iterator i = Localization.getAvailableLanguages().iterator(); i.hasNext(); ) {
            final Localization.Language lang = (Localization.Language)i.next();
            JRadioButtonMenuItem langItem = new JRadioButtonMenuItem(lang.getName());
            langItem.setSelected(Localization.getCurrentLanguage().equals(lang));
            setMenuLooks(langItem);
            submenu.add(langItem);
            langGroup.add(langItem);
            langItem.addActionListener(new ActionListener() {
                public void actionPerformed(ActionEvent e) {
                    Localization.setCurrentLanguage(lang);
                    recreateUI();
                }
            });
        }
        menu.add(submenu);

        // Build the "help" menu
        menu = createMenu("ui.menu_help");
        menuBar.add(menu);
        //menuBar.setHelpMenu(menu);

        menuItem = createMenuItem("ui.menu_help_contents", null); //$NON-NLS-1$
        menuItem.addActionListener(new ActionListener(){
			public void actionPerformed(ActionEvent arg0) {
                showHelp(HELP_MODE_USER_MANUAL);
			}            
        });
        menu.add(menuItem);

        menuItem = createMenuItem("ui.menu_faq", null); //$NON-NLS-1$
        menuItem.addActionListener(new ActionListener(){
            public void actionPerformed(ActionEvent arg0) {
                showHelp(HELP_MODE_FAQ);
            }            
        });
        menu.add(menuItem);
        
        menuItem = createMenuItem("ui.menu_help_about", null); //$NON-NLS-1$
        menuItem.addActionListener(new ActionListener() {
            public void actionPerformed(ActionEvent arg0) {
                showAboutBox();
            }
        });
        menu.add(menuItem);
        
        return menuBar;
    }

    /**
	 * Opens the HTML help in the current locale in a new 
     * window. 
	 */
	protected void showHelp(int mode) {
        JDialog helpDialog = new JDialog(); 
        JScrollPane scrollPane;        
        String filename = null;
        
        if(mode == HELP_MODE_USER_MANUAL) {
        	filename = "/help/man_animator.html";
        }
        else {
            filename = "/help/man_faq.html";            
        }
        
        try {
            URL helpURL = mainFrame.getClass().getResource(filename);
        	JEditorPane htmlPane = new JEditorPane(helpURL);
            htmlPane.setEditable(false);
            htmlPane.addHyperlinkListener(new HyperlinkListener(){          
                public void hyperlinkUpdate(HyperlinkEvent e) {
                      if (e.getEventType() == HyperlinkEvent.EventType.ACTIVATED) {
                      JEditorPane pane = (JEditorPane) e.getSource();
                      if (e instanceof HTMLFrameHyperlinkEvent) {
                          HTMLFrameHyperlinkEvent  evt = (HTMLFrameHyperlinkEvent)e;
                          HTMLDocument doc = (HTMLDocument)pane.getDocument();
                          doc.processHTMLFrameHyperlinkEvent(evt);
                      } else {
                          try {
                          pane.setPage(e.getURL());
                          } catch (Throwable t) {
                          t.printStackTrace();
                          }
                      }
                      }
                  }                
            });
            
            scrollPane = new JScrollPane(htmlPane);
            scrollPane.setVerticalScrollBarPolicy(JScrollPane.VERTICAL_SCROLLBAR_AS_NEEDED);
            helpDialog.getContentPane().add(scrollPane, BorderLayout.CENTER);            
        } catch (IOException e) {            
        }
        helpDialog.setTitle(Localization.getString("dialog.help.title"));
        helpDialog.pack();
  
        helpDialog.setSize(HELP_DIALOG_WIDTH,HELP_DIALOG_HEIGHT);
        helpDialog.setLocationRelativeTo(mainFrame);
        helpDialog.setVisible(true);
	}

	/**
     * Gets the requested icon from the resources
     * 
     * @param user
     * @param imgLocation
     * @return the requested image if it is found, otherwise null.
     */
    private ImageIcon getImageIcon(JFrame user, String imgLocation) {
        URL imageURL = user.getClass().getResource(imgLocation);
        if (imageURL != null) {
            return new ImageIcon(imageURL);
        } else {
            return null;
        }
    }

    /**
     * Creates a menu item that can be then added to the menu bar
     * 
     * @param key of the localized menu text
     * @param imgLocation
     *            The URL for the image, null if no image is wanted
     * @return The menu item
     */
    private JMenuItem createMenuItem(String key, String imgLocation) {

        JMenuItem menuItem = new JMenuItem();
        Localization.localizeMenuItem(menuItem, key);
		setMenuLooks(menuItem);
        
        if (imgLocation != null) {
            menuItem.setIcon(getImageIcon(mainFrame, imgLocation));
        }

        return menuItem;
    }
    
    /**
     * Creates a top-level menu item
     */
    private JMenu createMenu(String key) {
        JMenu menu = new JMenu();
        Localization.localizeMenuItem(menu, key);
        setMenuLooks(menu);
        return menu;
    }

    /** 
     * Creates a new radio button menu item
     */    
    private JRadioButtonMenuItem createRadioButtonMenuItem(String key, boolean selected) {
        JRadioButtonMenuItem item = new JRadioButtonMenuItem();
        item.setSelected(selected);
        Localization.localizeMenuItem(item, key);
        setMenuLooks(item);
        return item;
    }
    
    private void setMenuLooks(JMenuItem item) {
        item.setBackground(DACOPAN_COLOR_PANE_BG);
        item.setFont(DACOPAN_FONT_MENU);
    }

    /**
     * Makes a button with either icon and text or 
     * only text / image. 
     *  
     * @param user
     * @param imgLocation MUST NOT BE NULL, "" is ok.
     * @param buttonText
     * @return @throws
     *         IllegalArgumentException
     */
    private JButton createButton(JFrame user, String imgLocation, String buttonText) throws IllegalArgumentException {
        JButton button = null;

        if (imgLocation != null) {
            URL imageURL = user.getClass().getResource(imgLocation);

            if (buttonText != null) {
                button = new JButton(buttonText, new ImageIcon(imageURL));
                button.setVerticalTextPosition(SwingConstants.CENTER);
                button.setHorizontalTextPosition(SwingConstants.RIGHT);
            } else {
                button = new JButton(new ImageIcon(imageURL));
            }
        } else {
            if (buttonText != null) {
                button = new JButton(buttonText);
                button.setFont(DACOPAN_FONT_BUTTON);
            } else {
                throw new IllegalArgumentException("Buttons need text or image!");
            }
        }
        
        button.setBackground(DACOPAN_COLOR_PANE_BG);
        
        return button;
    }

    /**
     * Creates and shows the about box for the whole DaCoPAn product
     */
    private void showAboutBox() {
        
        /*JLabel teamLabel = new JLabel();
        URL teamImageURL = MainFrame.class.getResource(DACOPAN_ICON_DEVELOPERS);
        if (teamImageURL == null) {
            System.err.println("Resource not found");
        } else {
            teamLabel.setIcon(new ImageIcon(teamImageURL));
        }
        
        teamLabel.setVerticalTextPosition(JLabel.BOTTOM);
        teamLabel.setHorizontalTextPosition(JLabel.CENTER);
        teamLabel.setText(Localization.getString("ui.about_team")); //$NON-NLS-1$

        JPanel teamPanel = new JPanel();
        teamLabel.setFont(DACOPAN_FONT_MENU);
        teamPanel.add(teamLabel);*/

        // The main about pane        
        JPanel aboutPanel = new JPanel(new BorderLayout());
        aboutPanel.setBackground(new Color(177,177,177));
        URL logoImageURL = MainFrame.class.getResource(DACOPAN_ICON_LOGO);
        if (logoImageURL == null) {
            System.err.println("Resource not found");
        } else {
            aboutPanel.add(new JLabel(new ImageIcon(logoImageURL)), BorderLayout.NORTH);                
        }
        
        JTextArea textLabel = new JTextArea(Localization.getString("ui.about_text"));
        textLabel.setLineWrap(true); 
        textLabel.setEditable(false);       
        aboutPanel.add(new JScrollPane(textLabel), BorderLayout.CENTER); 

        // The license panel
        JPanel licensePanel = new JPanel(new BorderLayout());
        JTextArea licenseText = new JTextArea();
        String text = readString(mainFrame.getClass().getResourceAsStream(DACOPAN_LICENSE_RESOURCE));
        licenseText.setText(text);
        licenseText.setLineWrap(true);
        licenseText.setWrapStyleWord(true);
        licenseText.setEditable(false);
        JScrollPane scrollPane = new JScrollPane(licenseText);
        licensePanel.add(scrollPane, BorderLayout.CENTER);       
        
        JTabbedPane aboutTabbedPane = new JTabbedPane();
        aboutTabbedPane.setAutoscrolls(true);
        aboutTabbedPane.add(aboutPanel);       
        //aboutTabbedPane.add(teamPanel);
        aboutTabbedPane.add(scrollPane);    
        aboutTabbedPane.setTitleAt(0, Localization.getString("ui.about_about"));
        //aboutTabbedPane.setTitleAt(1, Localization.getString("ui.about_team"));
        aboutTabbedPane.setTitleAt(1, Localization.getString("ui.about_license"));                
                
        final JDialog aboutDialog = new JDialog(mainFrame);
        aboutDialog.getContentPane().add(aboutTabbedPane, BorderLayout.CENTER);
        
        JPanel okPanel = new JPanel();
        JButton okButton = new JButton("OK");
        okButton.addActionListener(new ActionListener(){

            JDialog dlg = aboutDialog;
   			public void actionPerformed(ActionEvent arg0) {
			    dlg.setVisible(false);			
			}
            
        });
        okPanel.add(okButton);
        
        aboutDialog.getContentPane().add(okPanel, BorderLayout.SOUTH);
        aboutDialog.setSize(600, 400);
        aboutDialog.setTitle(Localization.getString("ui.menu_help_about"));
        centerWindow(aboutDialog);
        aboutDialog.setVisible(true);
        
    }
    
    private String readString(InputStream is) {
        try {
            InputStreamReader reader = new InputStreamReader(is);
            StringBuffer result = new StringBuffer();
            char[] buff = new char[4096];
            int length;
            while((length = reader.read(buff)) != -1) {
                result.append(buff, 0, length);
            }
            reader.close();
            return result.toString();
        } catch (IOException e) {
            throw new RuntimeException("Couldn't read resource", e);
        }
    }

	/**
     * Refreshes the toolbar by adding the layer buttons there
     */
    private void refreshToolBar() {       
        if (mainFrame.isInTSCMode() == false) {
        	addLayerButtons(toolBar);
        }
        toolBar.repaint();
    }

    /**
     * Updates the title bar to show if there is a file in memory and 
     * if it has been modified since it has been saved or loaded.
     */
    private void refreshTitleBar() {

        String titleString = Localization.getString("ui.frame_title"); //$NON-NLS-1$
        FileInformation fileInfo = mainFrame.getFileInformation();
        
        if(fileInfo.isLoaded() == true)
        {
        	titleString += Localization.getString("ui.frame_title_separator") + fileInfo.getFile().getName(); //$NON-NLS-1$
            
            if(fileInfo.isModified()) {
            	titleString += Localization.getString("ui.frame_title_modified"); //$NON-NLS-1$
            }
        }

        mainFrame.setTitle(titleString);
    }

    /**
     * Updates the menu by setting the correct layer selections
     */
    private void refreshLayerMenu() {
        
        MenuElement[] topMenus = menuBar.getSubElements();
               
        uiControls.menuSelectLayer.removeAll();
        
        ButtonGroup group = new ButtonGroup();
        JRadioButtonMenuItem radioButtonMenuItem = null;
        
        // add new buttons
        Iterator i = mainFrame.getDataView().getLayers().iterator();
        while (i.hasNext()) {

            Layer layer = (Layer) i.next();
            
            String layerName = layer.getName();
            
            radioButtonMenuItem = new JRadioButtonMenuItem(layerName);
            radioButtonMenuItem.setBackground(DACOPAN_COLOR_PANE_BG);
            radioButtonMenuItem.setFont(DACOPAN_FONT_MENU);
            radioButtonMenuItem.addActionListener(new LayerButtonActionListener(layer));
            
            if(layer == mainFrame.getCurrentLayer()){
            	radioButtonMenuItem.setSelected(true);
            }
            group.add(radioButtonMenuItem);
            uiControls.menuSelectLayer.add(radioButtonMenuItem);             
        }                    
    }

    /**
     * Enables the controls that need to be enabled in
     * current situation and disables the rest. 
     * 
     * This is called by the refresh() method.
     */
    private void refreshControlActivations() {
        
        FileInformation fileInfo = mainFrame.getFileInformation();
        if(fileInfo.isLoaded() == false) {
            setControlsEnabledAnimationControl(false);
            setControlsEnabledSave(false);
            setControlsEnabledSaveAs(false);
            setControlsEnabledAnimatorMode(false);
            setControlsEnabledTSC(false);
            setControlsEnabledMSC(false);
        }
        else {
            setControlsEnabledSaveAs(true);

            // Animation controls
            if(mainFrame.isInENCMode()) {
                setControlsEnabledAnimationControl(false);                
            }
            else {                
                // First enable everything
                setControlsEnabledAnimationControl(true);
                setControlsEnabledAnimatorMode(true);
                setControlsEnabledMSC(true);
                setControlsEnabledTSC(true);
                
                if (mainFrame.isInMSCMode()) {
                	setControlsEnabledMSC(false);
                	// disable tsc button in scenario mode
                    if (mainFrame.isInScenarioMode()) {
                    	uiControls.tscButton.setEnabled(false);
                    } else {
                    	uiControls.tscButton.setEnabled(true);
                    }
                } else if (mainFrame.isInTSCMode()) {
                	setControlsEnabledTSC(false);
                }
                
                // Then see if something needs to be disabled
                if(mainFrame.isPaused()) {
                    // Paused
                    setControlsEnabledPause(false);
                    
                    /* These tests can be added if the issue of 
                     * when to call this method is solved... We didn't
                     * have the time to test it thoroughly so we decided
                     * to leave this part out of the current version.
                     * 
                    if(mainFrame.isAtBeginning()) {
                        setControlsEnabledRewind(false);
                        setControlsEnabledStepBack(false);
                    }
                    else if(mainFrame.isAtEnd()) {
                        setControlsEnabledFastForward(false);
                        setControlsEnabledStepForward(false);
                        setControlsEnabledPlay(false);
                    }
                    */
                }
                else {
                    // Playing
                    setControlsEnabledPlay(false);
                    setControlsEnabledStepBack(false);
                    setControlsEnabledStepForward(false);
                    setControlsEnabledLayerSelection(false);
                    
                    setControlsEnabledRewind(false);
                    setControlsEnabledFastForward(false);
                    setControlsEnabledSettings(false);
                }
            }
            
            // Saving
            if(fileInfo.isLoaded() && fileInfo.isModified()) {
                setControlsEnabledSave(true);
            }
            else {
                setControlsEnabledSave(false);
            }
        }        
    }   
    
    /**
     * Handles the load file action coming from the tool bar or the menu.
     */
    private void loadFile() {

        try {
            /*
             * Show a file chooser dialog to the user and if he selects a file, ask the main frame to load that file.
             * If the user presses cancel, the main frame doesn't have to know about this.
             */
            JFileChooser fc = MainFrame.getFileChooser();
            fc.setForeground(DACOPAN_COLOR_PANE_BG);

            // Add file filters to accept only the types of files we want to use
            fc.addChoosableFileFilter(new DaCoPAnFileFilter(MainFrame.DACOPAN_EXTENSION_PEF, Localization.getString("ui.filetype_pef"))); //$NON-NLS-1$
            fc.addChoosableFileFilter(
                new DaCoPAnFileFilter(MainFrame.DACOPAN_EXTENSION_SCENARIO, Localization.getString("ui.filetype_animseq"))); //$NON-NLS-1$

			DaCoPAnFileFilter filter = new DaCoPAnFileFilter(MainFrame.DACOPAN_EXTENSION_PEF, Localization.getString("ui.filetype_all"));
			filter.addExtension(MainFrame.DACOPAN_EXTENSION_SCENARIO);
			fc.addChoosableFileFilter(filter);

            // Show the dialog
            int returnValue = fc.showOpenDialog(mainFrame);

            // Handle the return values
            if (returnValue == JFileChooser.APPROVE_OPTION) {
                File file = fc.getSelectedFile();
                mainFrame.loadAnimationFile(file);
            }
        } catch (Throwable t) {
            JOptionPane.showMessageDialog(mainFrame, Localization.getString("mainframe.title_loading_pef_failed"),  //$NON-NLS-1$
                Localization.getString("ui.title_error"), JOptionPane.ERROR_MESSAGE); //$NON-NLS-1$
        }
    }

    /**
     * Creates and dispaly the dialog with the general and MSC settings
     */
    protected void changeSettings() {

        // First pause the animation
        mainFrame.pauseAnimation();

        /* check if the panel can be created or display just a message of error */
        if (mainFrame.getSettings() == null){
            JOptionPane.showMessageDialog(mainFrame, Localization.getString("ui.msg_no_file_loaded"),  //$NON-NLS-1$
                            Localization.getString("ui.title_error"), JOptionPane.ERROR_MESSAGE); //$NON-NLS-1$
            return;
        }
        
        /* Create the dialog that will contain the settings and buttons for acept and cancel */
        settingsDialog = new SettingsDialog(mainFrame, Localization.getString("ui.title_change_settings"), true); //$NON-NLS-1$
        
        /* changes the location of the settings panel to the center of the main frame */
        settingsDialog.setLocationRelativeTo(mainFrame);

        settingsDialog.setVisible(true);
    }
    
	/**
	 * Creates and displays the settings dialog with given mode selected.
	 * @param mode
	 */
    protected void changeSettingsMode(int mode) {

        // First pause the animation
        mainFrame.pauseAnimation();

        /* check if the panel can be created or display just a message of error */
        if (mainFrame.getSettings() == null){
            JOptionPane.showMessageDialog(mainFrame, Localization.getString("ui.msg_no_file_loaded"),  //$NON-NLS-1$
                            Localization.getString("ui.title_error"), JOptionPane.ERROR_MESSAGE); //$NON-NLS-1$
            return;
        }
        
        /* Create the dialog that will contain the settings and buttons for acept and cancel */
        settingsDialog = new SettingsDialog(mainFrame, Localization.getString("ui.title_change_settings"), true, mode); //$NON-NLS-1$
        
        /* changes the location of the settings panel to the center of the main frame */
        settingsDialog.setLocationRelativeTo(mainFrame);

        settingsDialog.setVisible(true);
    }
    
    /**
     * Sets the given JComponent to the UFO Panel 
     * space in the UI.
     * 
     * @param ufoPanel
     */
    public void setUfoPanel(JComponent ufoPanel) {
		int oldLocation = verticalSplitPane.getDividerLocation();
		int oldHorizontalLocation = mainSplitPane.getDividerLocation();
		if (oldLocation < (int)ufoPanel.getMinimumSize().getHeight())
	        oldLocation = (int) (DEFAULT_DISP_MODE_HEIGHT * DEFAULT_RATIO_UFO_2_NOTE);
		verticalSplitPane.setTopComponent(ufoPanel);
		verticalSplitPane.setDividerLocation(oldLocation);
		mainSplitPane.setDividerLocation(oldHorizontalLocation);
		if (ufoPanel instanceof UFOPanel) {
			((UFOPanel)(ufoPanel)).updateButtons();
		}
    }

    /**
     * Sets the given JComponent to the Note Panel 
     * space in the UI.
     * 
     * @param ufoPanel
     */
    public void setNotePanel(JComponent panel) {
        int oldLocation = verticalSplitPane.getDividerLocation();
		if (oldLocation < (int)panel.getMinimumSize().getHeight())
	        oldLocation = (int) (DEFAULT_DISP_MODE_HEIGHT * DEFAULT_RATIO_UFO_2_NOTE);
        verticalSplitPane.setBottomComponent(panel);
        verticalSplitPane.setDividerLocation(oldLocation);
    }

    /**
     * Sets the given JComponent to the main panel 
     * space in the UI.
     * 
     * @param ufoPanel
     */
    public void setMainPanel(JComponent panel) {
        int oldLocation = mainSplitPane.getDividerLocation();
        mainSplitPane.setLeftComponent(panel);
        mainSplitPane.setDividerLocation(oldLocation);
    }

    /**
     * Sets the given AbstractAnimationPanel to the Time Panel 
     * space in the UI.
     * 
     * @param ufoPanel
     */
    public void setTimePanel(AbstractAnimationPanel panel) {
        statusBar.setTimePanel(panel);
    }
    
	/**
     * Sets the animator mode information in the user interface.
     * This should only be called through MainFrame.setAnimatorMode and
     * refreshSelections.
     * 
	 * @param mode The mode to which the animator is changed
	 */
	public void setAnimatorModeInfo(int mode) {
		               
        if(mode == MainFrame.DACOPAN_EXPLORE_MODE) {
            uiControls.menuItemScenarioMode.setSelected(false);
            uiControls.menuItemExploreMode.setSelected(true);
        }
        else if(mode == MainFrame.DACOPAN_SCENARIO_MODE) {
            uiControls.menuItemScenarioMode.setSelected(true);
            uiControls.menuItemExploreMode.setSelected(false);
        }
        else {
            uiControls.menuItemScenarioMode.setSelected(false);
            uiControls.menuItemExploreMode.setSelected(false);
        }
        
        statusBar.setStateMode(mode);
	}

	/** Sets the enabled value for the animator mode selection */
	public void setControlsEnabledAnimatorMode(boolean value) {
        uiControls.menuItemExploreMode.setEnabled(value);
        uiControls.menuItemScenarioMode.setEnabled(value);
    }    
    
    /** Sets the enabled value for the settings button */
    public void setControlsEnabledSettings(boolean value) {
        uiControls.settingsButton.setEnabled(value);
        uiControls.menuItemSettings.setEnabled(value);
        uiControls.menuItemGeneralSettings.setEnabled(value);
        uiControls.menuItemMSCSettigs.setEnabled(value);
        uiControls.menuItemTSCSettings.setEnabled(value);
        uiControls.menuItemSaveAllSettings.setEnabled(value);
    }

    /** 
     * Sets the enabled value for the play button, also  
     * sets the pause button to the complement of value 
     */
    public void setControlsEnabledPlayAndPause(boolean value) {
    	setControlsEnabledPlay(value);
        
        // Then the pause button
        if(value) {
            setControlsEnabledPause(false);
        }
        else {
            setControlsEnabledPause(true);
        }
    }

    /** Sets the enabled value for the play control */    
    public void setControlsEnabledPlay(boolean value) {
        uiControls.playButton.setEnabled(value);
        uiControls.menuItemPlay.setEnabled(value);
    }
    
    /** Sets the enabled value for the pause control */    
    public void setControlsEnabledPause(boolean value) {
        uiControls.stopButton.setEnabled(value);
        uiControls.menuItemPause.setEnabled(value);
    }
        
    /** Sets the enabled value for the layer selection controls */
    public void setControlsEnabledLayerSelection(boolean value) {
        // Toolbar buttons
        uiControls.layerPanel.setEnabled(value);
        Component[] components = uiControls.layerPanel.getComponents();
        
        for(int i = 0; i < uiControls.layerPanel.getComponentCount(); i++)
        {
        	components[i].setEnabled(value);
        }
        
        // Menu
        uiControls.menuSelectLayer.setEnabled(value);
    }
    
    /** Sets the enabled value for the stepping backward */    
    public void setControlsEnabledStepBack(boolean value) {
        uiControls.stepBackButton.setEnabled(value);       
        uiControls.menuItemStepBackward.setEnabled(value);
    }

    /** Sets the enabled value for stepping forward */
    public void setControlsEnabledStepForward(boolean value) {
    	uiControls.stepForwardButton.setEnabled(value);
    	uiControls.menuItemStepForward.setEnabled(value);
    }
    
    /** Sets the enabled value for rewind controls */
    public void setControlsEnabledRewind(boolean value) {
        uiControls.rewindButton.setEnabled(value);
        uiControls.menuItemRewind.setEnabled(value);
    }

    /** Sets the enabled value for fast forward controls */
    public void setControlsEnabledFastForward(boolean value) {
        uiControls.forwardButton.setEnabled(value);
        uiControls.menuItemForward.setEnabled(value);
    }

    /** Sets the enabled value for saving controls */
    public void setControlsEnabledSave(boolean value) {
        uiControls.saveButton.setEnabled(value);
        uiControls.menuItemSave.setEnabled(value);
    }
    
    /** Sets the enabled value for saving as controls */
    public void setControlsEnabledSaveAs(boolean value) {
        uiControls.menuItemSaveAs.setEnabled(value);
    }
    
    public void setControlsEnabledMSC(boolean value) {
    	uiControls.mscButton.setEnabled(value);
    }
    
    public void setControlsEnabledTSC(boolean value) {
    	uiControls.tscButton.setEnabled(value);
    	uiControls.menuItemScenarioMode.setEnabled(value);
    	uiControls.menuSelectLayer.setEnabled(value);
    }
    
    /** 
     * Sets the enabled value for all controls related to controlling
     * the control signal framework. Used when switching to enc and back
     */
    public void setControlsEnabledAnimationControl(boolean value) {
    	setControlsEnabledRewind(value);
        setControlsEnabledFastForward(value);
        setControlsEnabledAnimatorMode(value);
        setControlsEnabledLayerSelection(value);
        setControlsEnabledSettings(value);
        setControlsEnabledPlay(value);
        setControlsEnabledPause(value);
        setControlsEnabledStepBack(value);
        setControlsEnabledStepForward(value);
    }    
    
    /**
     * Refreshes the UI to reflect any changes in the MainFrame 
     * that affect the visual look and feel of the software
     */
    public void refresh() {        
        refreshTitleBar();       
        refreshToolBar();
        
        refreshControlActivations();
        refreshSelections();

        // Status bar               
        if(mainFrame.isAnimationSequenceDialogActive() || 
           mainFrame.isInScenarioMode() == false) 
        {            
            statusBar.setRestoreScenarioDialogButton(false);
        }
        else {
            statusBar.setRestoreScenarioDialogButton(true);
        }
        statusBar.repaint();
        
        // Repaint all panels
        mainSplitPane.getLeftComponent().repaint();
        mainSplitPane.getRightComponent().repaint();
        
        //SwingUtilities.updateComponentTreeUI(toolBar);
        toolBar.updateUI();
        toolBar.setRollover(true);
    }
    
    
    /**
	 * 
	 */
	private void refreshSelections() {
		// Refresh the animator mode
        setAnimatorModeInfo(mainFrame.getMode());
        
        // Refresh layer selection
        refreshLayerMenu();       
	}

	/**
     * Re-creates the user interface for example when the 
     * language has been changed
     */
    protected void recreateUI() {
        
        AnimationTimeState timeState = mainFrame.getAnimationTimeState();
        float currentNowTime = 0f;
        
        if(timeState != null)
        { 
            timeState.pause();
            currentNowTime = timeState.getNowTime();
        }
        
        // menu        
        menuBar = createMenuBar();
        mainFrame.setJMenuBar(menuBar);
        
        // title bar
        refreshTitleBar();

        // Recreate the toolbar
        Container contentPane = mainFrame.getContentPane();
        contentPane.remove(toolBar);        
        toolBar = createToolBar();
        toolBar.setBackground(DACOPAN_COLOR_PANE_BG);
        contentPane.add(toolBar, BorderLayout.NORTH);               
        
        // create the status bar so that the animation mode
        // setup methods can then set it up with real information
        if(statusBar != null) {
            mainFrame.getContentPane().remove(statusBar);
        }
        statusBar = new StatusBar(mainFrame);
        mainFrame.getContentPane().add(statusBar, BorderLayout.SOUTH);
                
        // animation mode specific information
        if(mainFrame.isInENCMode()) {
            mainFrame.setupAnimationModeENC(mainFrame.getCurrentENCUnit());
        }
        else if(mainFrame.isInMSCMode()){
            mainFrame.setupAnimationModeMSC(mainFrame.getCurrentMSCSettings(), false);            
        } 
        else if (mainFrame.isInTSCMode()) {
        	mainFrame.setupAnimationModeTSC();
        }
        
        if(mainFrame.isInScenarioMode()) {
            mainFrame.refreshScenarioSettings();           
        }                       

        // Finally set the current now time back
        if(timeState != null) {
        	timeState.setNowTime(currentNowTime);
        }
        
        refresh();
    }
    
    /** 
     * This method can be used to set the default button font to
     * the given button 
     */
    public static void setDefaultButtonFont(Component component) {
        component.setFont(DACOPAN_FONT_BUTTON);
    }
}
