/*
 * Created on 18.2.2005
 */
package fi.helsinki.dacopan.ui.tsc;

import java.awt.Color;
import java.awt.Graphics;
import java.awt.GridBagConstraints;
import java.awt.GridBagLayout;
import java.awt.Image;
import java.awt.image.BufferedImage;

import javax.swing.BoxLayout;
import javax.swing.ImageIcon;
import javax.swing.JLabel;
import javax.swing.JPanel;
import javax.swing.SwingUtilities;

import fi.helsinki.dacopan.Localization;

public class LegendPanel extends JPanel {
    public static final int UNIT_WIDTH = 20;
    public static final int UNIT_HEIGHT = 20;
	
    private SettingsTSC settings;
    
    private GridBagConstraints gbc;
    private GridBagLayout gbl;
    
    
    private JLabel lblSegment;
    private JLabel lblSelectedSegment;
    private JLabel lblDroppedSegment;
    private JLabel lblWindowSize;
    
    private JLabel lblSyn;
    private JLabel lblAck;
    private JLabel lblFin;
    private JLabel lblDelay;
    private JLabel lblSack;
    
    private JPanel container;
    
	public LegendPanel(SettingsTSC settings)
	{
        this.settings = settings;
		initComponents();
	}
    
	private void initComponents() {        
        this.setLayout(new GridBagLayout());
        
        container = new JPanel();
        container.setLayout(new BoxLayout(container,BoxLayout.Y_AXIS));
        
        createLabels();        
        
        this.add(container);
        
	}
        
    private void createLabels() {
        lblSegment = createLabel("panel.settings.elements.segment");
        lblSegment.setVisible(settings.isElementDisplayed(SettingsTSC.ELEMENTKEY_SEGMENT));
        
        lblSelectedSegment = createLabel("panel.settings.elements.active_segment");
        lblSelectedSegment.setVisible(settings.isElementDisplayed(SettingsTSC.ELEMENTKEY_SEGMENT));
        
        lblDroppedSegment = createLabel("panel.settings.elements.dropped_segment");
        lblDroppedSegment.setVisible(settings.isElementDisplayed(SettingsTSC.ELEMENTKEY_SEGMENT));
        
        lblAck = createLabel("panel.settings.elements.ack");
        lblAck.setVisible(settings.isElementDisplayed(SettingsTSC.ELEMENTKEY_ACK));
        
        lblFin = createLabel("panel.settings.elements.fin");
        lblFin.setVisible(settings.isElementDisplayed(SettingsTSC.ELEMENTKEY_FIN));
        
        lblSyn = createLabel("panel.settings.elements.syn");
        lblSyn.setVisible(settings.isElementDisplayed(SettingsTSC.ELEMENTKEY_SYN));
        
        lblDelay = createLabel("panel.settings.elements.delay_line");
        lblDelay.setVisible(settings.isElementDisplayed(SettingsTSC.ELEMENTKEY_DELAY_LINE));
        
        lblSack = createLabel("panel.settings.elements.sack");
        lblSack.setVisible(settings.isElementDisplayed(SettingsTSC.ELEMENTKEY_SACK));
        
        lblWindowSize = createLabel("panel.settings.elements.advertized_window_size");
        lblWindowSize.setVisible(settings.isElementDisplayed(SettingsTSC.ELEMENTKEY_ADVERTIZED_WINDOW_LINE));
        
        drawUnitSymbols();
    }
    
    public void paintComponent(Graphics g) {
        super.paintComponent(g);        
    }
    
    
    private JLabel createLabel(String elementKey) {
        JLabel lbl = new JLabel(Localization.getString(elementKey));
        //gbl.setConstraints(lbl,gbc);
        container.add(lbl);
        return lbl;    	
    }
    private void drawUnitSymbols() {
        int offset_x = 5;
        int offset_y = 5;
        
        Image img;
        Graphics g;
        
        
        img = drawUnit(SettingsTSC.COLORKEY_SEGMENT,offset_x,offset_y);
        lblSegment.setIcon(new ImageIcon(img));
        
        
        //Draw active unit
        img = drawUnit(SettingsTSC.COLORKEY_SEGMENT,offset_x,offset_y);
        g = img.getGraphics();
        g.setColor(settings.getColor(SettingsTSC.COLORKEY_ACTIVE));
        g.drawRect(offset_x-2, offset_y-2, UNIT_WIDTH-1+4, UNIT_HEIGHT-1+4);
        lblSelectedSegment.setIcon(new ImageIcon(img));
        
        //Draw dropped unit
        img = drawUnit(SettingsTSC.COLORKEY_SEGMENT,offset_x,offset_y);
        g = img.getGraphics();
        g.setColor(Color.RED);
        g.drawLine(offset_x-5, offset_y-5, offset_x+UNIT_WIDTH+5, offset_y+UNIT_HEIGHT+5);
        g.drawLine(offset_x+UNIT_WIDTH+5, offset_y-5, offset_x-5, offset_y+UNIT_HEIGHT+5);
        lblDroppedSegment.setIcon(new ImageIcon(img));
        
        //Draw syn unit
        img = drawUnit(SettingsTSC.COLORKEY_SYN, offset_x,offset_y);
        lblSyn.setIcon(new ImageIcon(img));
        
        //Draw fin unit
        img = drawUnit(SettingsTSC.COLORKEY_FIN, offset_x,offset_y);
        lblFin.setIcon(new ImageIcon(img));
        
        //Draw ack
        img = createUnitImage();
        g = img.getGraphics();        
        g.setColor(settings.getColor(SettingsTSC.COLORKEY_ACK));        
        g.fillRect(offset_x,offset_y, UNIT_WIDTH, UNIT_HEIGHT/3);
        g.setColor(Color.BLACK);           
        g.drawRect(offset_x, offset_y, UNIT_WIDTH-1, UNIT_HEIGHT/3-1);
        lblAck.setIcon(new ImageIcon(img));


        lblDelay.setIcon(createLineIcon(SettingsTSC.COLORKEY_DELAY_LINE, SwingUtilities.HORIZONTAL));
        lblSack.setIcon(createLineIcon(SettingsTSC.COLORKEY_SACK));        
        lblWindowSize.setIcon(createLineIcon(SettingsTSC.COLORKEY_ADVERTIZED_WINDOW_LINE));           
        
    }
    
    private ImageIcon createLineIcon(String colorKey) {
    	return createLineIcon(colorKey,SwingUtilities.VERTICAL);
    }
    
    private ImageIcon createLineIcon(String colorKey, int direction) {
        if(colorKey == null || settings.getColor(colorKey)==null) {
        	throw new NullPointerException("colorKey or value was null" + colorKey + " " + settings.getColor(colorKey));
        }
        
        Image img = createUnitImage();        
        Graphics g = img.getGraphics();        
        g.setColor(settings.getColor(colorKey));        
        
        if(SwingUtilities.VERTICAL == direction) {
            g.drawLine(img.getWidth(this)/2,2,img.getWidth(this)/2,img.getHeight(this)-2);
        } else {
            g.drawLine(2,img.getHeight(this)/2,img.getWidth(this)-2,img.getHeight(this)/2);    
        }
        
        return new ImageIcon(img);
    }
    
    private Image createUnitImage() {
        Image img = new BufferedImage((int)(UNIT_WIDTH*1.5),(int)(UNIT_HEIGHT*1.5),BufferedImage.TRANSLUCENT);        
        return img;
    }
    
    private Image drawUnit(String colorKey,int offset_x,int offset_y) {
    	Image img = createUnitImage();
        Graphics g = img.getGraphics();
        
        g.setColor(settings.getColor(colorKey));        
        g.fillRect(offset_x,offset_y, UNIT_WIDTH, UNIT_HEIGHT);
        g.setColor(Color.BLACK);           
        g.drawRect(offset_x, offset_y, UNIT_WIDTH-1, UNIT_HEIGHT-1);
        
        return img;
    }
}
