/*
 * Created on 18.2.2005
 */
package fi.helsinki.dacopan.ui.tsc;

import java.awt.BorderLayout;
import java.awt.Dimension;
import java.awt.FlowLayout;
import java.awt.Graphics;
import java.awt.GridBagConstraints;
import java.awt.GridBagLayout;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.util.Iterator;

import javax.swing.BoxLayout;
import javax.swing.ButtonGroup;
import javax.swing.JComboBox;
import javax.swing.JPanel;
import javax.swing.JRadioButton;
import javax.swing.JScrollPane;
import javax.swing.JSlider;
import javax.swing.JSplitPane;
import javax.swing.JViewport;
import javax.swing.ScrollPaneConstants;
import javax.swing.border.TitledBorder;
import javax.swing.event.ChangeEvent;
import javax.swing.event.ChangeListener;

import fi.helsinki.dacopan.Localization;
import fi.helsinki.dacopan.model.DataView;
import fi.helsinki.dacopan.model.Flow;
import fi.helsinki.dacopan.model.Host;
import fi.helsinki.dacopan.ui.MainFrame;
import fi.helsinki.dacopan.ui.TitlePanel;

/**
 * @author mjvaanan
 */
public class MainPanel extends JPanel {
	private MainFrame main;
	private DataView data;
	private DrawingArea drawingArea;
	private PacketInfoPanel infoPanel;
    private NoteBar noteBar;    
    private JPanel chartAndNoteBar;
    private Yaxis yAxis = new Yaxis();
	private Xaxis xAxis = new Xaxis();
	private volatile boolean isRunning = false;
	private AnimationThread thread;
	private int hostid_a;
	private int hostid_b;
	
    
    
	public MainPanel(MainFrame main, DataView data, PacketInfoPanel infoPanel) {
		this.main = main;
		this.data = data;
		this.infoPanel = infoPanel;
		if (data.getHosts().size() >= 2) {
			Host host_a = (Host) data.getHosts().get(0);
			Host host_b = (Host) data.getHosts().get(1);
			hostid_a = (int) host_a.getId();
			hostid_b = (int) host_b.getId();
			initDefaults(main.getSettings().getSettingsTSC());
		}
		initComponents();
	}
	
	private void initDefaults(SettingsTSC settings) {
		if (settings.getHostId() == -1) {
			settings.setHostId(hostid_a);
		}
		if (settings.getFlowId() == -1) {
			if (data.getFlows().size() > 0) {
				Flow flow = (Flow) data.getFlows().get(0);
				settings.setFlowId(flow.getId());
			}
		}
		if (settings.getXScale() == -1) {
			settings.setXScale(1);
		}
		if (settings.getYScale() == -1) {
			settings.setYScale(1);
		}
	}
	
	MainFrame getMainFrame() {
		return main;
	}
	
	private SettingsTSC getSettings() {
		return main.getSettings().getSettingsTSC();
	}
	
	private void initComponents() {
		removeAll();
		SettingsTSC settings = getSettings();
		drawingArea = new DrawingArea(this, data, settings.getHostId(), settings.getFlowId(), xAxis, 
				yAxis, infoPanel);

        infoPanel.setDrawingArea(drawingArea);
        
		setLayout(new BorderLayout());
		String title = Localization.getString("panel.tsc.title");
		add(new TitlePanel(title, null), BorderLayout.NORTH);
		
		JSplitPane splitPane = new JSplitPane(JSplitPane.VERTICAL_SPLIT);
		splitPane.setOneTouchExpandable(true);
		add(splitPane, BorderLayout.CENTER);
		
		// top
		splitPane.setTopComponent(createToolPanel());

		// bottom
        //Give handle to the notebar so that the infoPanel can tell the panel to display the packet notes.
		noteBar = new NoteBar(this, data, drawingArea,main);
        infoPanel.setNoteBar(noteBar);
        
        
		chartAndNoteBar = new JPanel(new BorderLayout());
		chartAndNoteBar.add(createChartPanel(), BorderLayout.CENTER);
		chartAndNoteBar.add(noteBar, BorderLayout.SOUTH);
		splitPane.setBottomComponent(chartAndNoteBar);
	}

	
	private JPanel createToolPanel()
	{
		JPanel toolPanel = new JPanel();
		toolPanel.setLayout(new BoxLayout(toolPanel, BoxLayout.X_AXIS));

		ButtonGroup group = new ButtonGroup();
		JRadioButton radio1 = new JRadioButton(Localization.getString("panel.tsc.hostchooser.host_a"));
		radio1.setSelected(true);
		JRadioButton radio2 = new JRadioButton(Localization.getString("panel.tsc.hostchooser.host_b"));
		group.add(radio1);
		group.add(radio2);
		JPanel radioPanel = new JPanel(new BorderLayout());
		radioPanel.add(radio1, BorderLayout.NORTH);
		radioPanel.add(radio2, BorderLayout.SOUTH);
		radioPanel.setBorder(new TitledBorder(Localization.getString("panel.tsc.hostchooser.title")));
		toolPanel.add(radioPanel);
		radio1.addActionListener(new ActionListener() {
			public void actionPerformed(ActionEvent arg0) {
				chooseHost(hostid_a);
			}
		});
		radio2.addActionListener(new ActionListener() {
			public void actionPerformed(ActionEvent arg0) {
				chooseHost(hostid_b);
			}
		});
		SettingsTSC settings = getSettings();
		if (settings.getHostId() == hostid_a) {
			radio1.setSelected(true);
			radio2.setSelected(false);
		} else {
			radio1.setSelected(false);
			radio2.setSelected(true);
		}
		
		JPanel flowPanel = new JPanel();
		JComboBox comboBox = new JComboBox();
		Iterator flow_iterator = data.getFlows().iterator();
		while (flow_iterator.hasNext()) {
			Flow flow = (Flow) flow_iterator.next();
			String item = "" + flow.getId();
			comboBox.addItem(item);
		}
		comboBox.setSelectedItem("" + settings.getFlowId());
		comboBox.addActionListener(new ActionListener() {
			public void actionPerformed(ActionEvent arg0) {
				JComboBox source = (JComboBox) arg0.getSource();
				String item = (String) source.getSelectedItem();
				chooseFlow(Long.parseLong(item));
			}
		});
		flowPanel.setBorder(new TitledBorder(Localization.getString("panel.tsc.flowchooser.title")));
		flowPanel.add(comboBox);
		toolPanel.add(flowPanel);

		final JSlider xSlider = new JSlider(0, 100);
		xSlider.setPreferredSize(new Dimension(180, 45));
		xSlider.setPaintLabels(true);
		xSlider.setPaintTicks(true);
		xSlider.setPaintTrack(true);
		xSlider.setMinorTickSpacing(5);
		xSlider.setMajorTickSpacing(20);
		int x_scale = 100;
		if (settings.getXScale() != -1) {
			x_scale = (int) Math.round(settings.getXScale() * 100);
			if (x_scale > 100) x_scale = 100;
			if (x_scale < 0) x_scale = 0;
		}
		xSlider.setValue(x_scale);
		xSlider.addChangeListener(new ChangeListener() {
			public void stateChanged(ChangeEvent e) {
				int value = xSlider.getValue();
				drawingArea.setXScale(value / 100.0);
			}
		});
		JPanel p1 = new JPanel();
		p1.add(xSlider);
		p1.setBorder(new TitledBorder(Localization.getString("panel.tsc.x_slider.title")));
		toolPanel.add(p1);
		
		final JSlider ySlider = new JSlider(0, 100);
		ySlider.setPreferredSize(new Dimension(180, 45));
		ySlider.setPaintLabels(true);
		ySlider.setPaintTicks(true);
		ySlider.setPaintTrack(true);
		ySlider.setMinorTickSpacing(5);
		ySlider.setMajorTickSpacing(20);
		int y_scale = 100;
		if (settings.getYScale() != -1) {
			y_scale = (int) Math.round(settings.getYScale() * 100);
			if (y_scale > 100) y_scale = 100;
			if (y_scale < 0) y_scale = 0;
		}
		ySlider.setValue(y_scale);
		ySlider.addChangeListener(new ChangeListener() {
			public void stateChanged(ChangeEvent e) {
				int value = ySlider.getValue();
				drawingArea.setYScale(value / 100.0);
			}
		});
		JPanel p2 = new JPanel();
		p2.add(ySlider);
		p2.setBorder(new TitledBorder(Localization.getString("panel.tsc.y_slider.title")));
		toolPanel.add(p2);

		JPanel panel = new JPanel(new FlowLayout(FlowLayout.LEFT));
		panel.add(toolPanel);
		return panel;
	}
	
	private JPanel createChartPanel()
	{
		JPanel panel = new JPanel(new GridBagLayout());
		JScrollPane scrollPane = new JScrollPane(drawingArea, 
				ScrollPaneConstants.VERTICAL_SCROLLBAR_ALWAYS,
				ScrollPaneConstants.HORIZONTAL_SCROLLBAR_ALWAYS);
		scrollPane.getHorizontalScrollBar().setUnitIncrement(20);
		scrollPane.getVerticalScrollBar().setUnitIncrement(20);
		JViewport view = scrollPane.getViewport();
		//view.setScrollMode(JViewport.BACKINGSTORE_SCROLL_MODE);
		view.setScrollMode(JViewport.BLIT_SCROLL_MODE);
		drawingArea.setViewport(view);

		GridBagConstraints c = new GridBagConstraints();
		
		c.fill = GridBagConstraints.BOTH;
		c.weightx = 0;
		c.weighty = 0;
		c.gridx = 0;
		c.gridy = 0;
		panel.add(yAxis, c);
		
		c.weightx = 1;
		c.weighty = 1;
		c.gridx = 1;
		panel.add(scrollPane, c);

		c.weightx = 0;
		c.weighty = 0;
		c.gridx = 0;
		c.gridy = 1;
		panel.add(new JPanel(), c);	// corner component
		
		c.gridx = 1;
		panel.add(xAxis, c);
		
		return panel;
	}
	
	private void chooseHost(int host_id) {
		SettingsTSC settings = getSettings();
		if (host_id != settings.getHostId()) {
			toStart();
			settings.setHostId(host_id);
			initComponents();
			validate();
		}
	}
	
	private void chooseFlow(long flow_id) {
		SettingsTSC settings = getSettings();
		if (flow_id != settings.getFlowId()) {
			toStart();
			settings.setFlowId(flow_id);
			initComponents();
			validate();
		}
	}
	
	public void toPlayMode() {
		if (!isRunning) {
			isRunning = true;
			thread = new AnimationThread();
			thread.start();
		}
	}
	
	public void toPauseMode() {
		isRunning = false;
		try {
			// interrupts sleep
			thread.interrupt();
		} catch (Exception e) {}
		main.toPauseMode();
	}
	
    public void toPauseMode(int seconds) {
        if(isRunning) {
            try {
            	thread.pauseMillis = seconds * 1000;         
            } catch(Exception exp) {}
            
        }
    }
    
	public void stepForward() {
		toPauseMode();
		drawingArea.stepForward();
		repaint();
	}
	
	public void stepBackward() {
		toPauseMode();
		drawingArea.stepBackward();
		repaint();
	}
	
	public void toStart() {
		toPauseMode();
		drawingArea.toStart();
		repaint();
	}
	
	public void toEnd() {
		toPauseMode();
		drawingArea.toEnd();
		repaint();
	}
	
	public boolean isPlaying() {
		return isRunning;
	}
	
	private void step() {
		boolean canDo = drawingArea.stepForward();
		repaint();
		if (!canDo) {
			toPauseMode();
			main.toPauseMode();
		}
	}
	
    public void paint(Graphics g) {
    	super.paint(g);
        
        //Check if the notebar is supposed to be visible. If not, remove it.
		SettingsTSC settings = getSettings();
		boolean noteBarEnabled = settings.isElementDisplayed(SettingsTSC.ELEMENTKEY_NOTICES);
        if(!noteBarEnabled && noteBar.isVisible()) {            
        	noteBar.setVisible(false);
            chartAndNoteBar.validate();            
        } else
        if(noteBarEnabled && !noteBar.isVisible()) {
        	noteBar.setVisible(true);
            chartAndNoteBar.validate();
        }        
        
        noteBar.paintNoticeLines(g);
    }
    
    public void pauseForSeconds(long seconds) {
    	if(thread != null) thread.setPause(seconds);        
    }
    
	private class AnimationThread extends Thread {
        private boolean paused = false;
        private long pauseMillis = -1;
        private long sleepTime = -1;
        
        
		public void run() {
			while (isRunning) {
				step();
				SettingsTSC settings = getSettings();
				int delay = Math.round(1000 / settings.getAnimationSpeed());
				try {
                    sleep(delay);                    
				} catch (InterruptedException e) {}
                
                try {
                    sleep(pauseMillis);
                    pauseMillis = 0;
                } catch(Exception exp) { }
                
			}
		}
        
        public void setPause(long seconds) {
            if(seconds > 0) {
            	pauseMillis = (seconds-1)*1000; //-1 if for the minimum pause setting in settings
                System.out.println("MainPanel pausing for " + pauseMillis);
            }
        }
        
	}
	
	public boolean containsData() {
		return drawingArea.getEventCount() > 0;
	}
}
