package fi.helsinki.dacopan.ui.tsc;

import java.awt.BorderLayout;
import java.awt.Color;
import java.awt.Dimension;
import java.awt.FlowLayout;
import java.awt.Graphics;
import java.awt.Point;
import java.awt.Rectangle;
import java.awt.event.AdjustmentEvent;
import java.awt.event.AdjustmentListener;
import java.util.Collection;
import java.util.HashMap;
import java.util.HashSet;
import java.util.Iterator;
import java.util.LinkedList;
import java.util.List;
import java.util.Map;

import javax.swing.JPanel;
import javax.swing.JScrollPane;
import javax.swing.JTextArea;
import javax.swing.JViewport;
import javax.swing.SwingUtilities;
import javax.swing.border.EtchedBorder;
import javax.swing.border.LineBorder;

import fi.helsinki.dacopan.model.DataView;
import fi.helsinki.dacopan.model.TransferUnit;
import fi.helsinki.dacopan.ui.MainFrame;

public class NoteBar extends JPanel {
	//Instance variables

	/** Handle to our parent MainPanel * */
	private MainPanel mainPanel;

	/** Handle to the drawingarea */
	private DrawingArea drawingArea;

	/** Handle to packet data */
	private DataView dataView;

	/** Stores NoteWidget lists that are in the notebar, key is transferunit */
	private HashMap unitMap;

    /** Is the bar currently moving? **/
    private boolean isAdjusting =  false;
    
    private NoteWidget selectedNotice;
    private NoteWidget prevNote;
    private TransferUnit selectedUnit;    
        
    private Runnable viewPortMover;
    
    private MainFrame main;
    
	//GUI Widgets
	private JScrollPane scrollPane = null;;

	private JPanel noteContainer = null;

	public NoteBar(MainPanel main, DataView data, DrawingArea area,MainFrame frame) {
		this.mainPanel = main;
		this.drawingArea = area;
		this.dataView = data;
		this.unitMap = new HashMap();
        this.main = frame;
        
		initComponents();       
         
	}

    
    
    private SettingsTSC getSettings() {
    	return main.getSettings().getSettingsTSC();
    }
    
	public NoteBar() {
		this.unitMap = new HashMap();
		initComponents();
	}

	private void initComponents() {
		setBorder(new EtchedBorder());
		this.setLayout(new BorderLayout());

		//Setup scrollpane and the panel that contains the note gui widgets
		noteContainer = new JPanel();
		noteContainer.setLayout(new FlowLayout(FlowLayout.LEFT, 2, 0));
        scrollPane = new JScrollPane(noteContainer);
        scrollPane.getViewport().setScrollMode(JViewport.BLIT_SCROLL_MODE);       
        scrollPane.getHorizontalScrollBar().addAdjustmentListener(new AdjustmentListener() {
			public void adjustmentValueChanged(AdjustmentEvent e) {
				mainPanel.repaint();
			}
            
        
        
        });

		this.add(scrollPane, BorderLayout.CENTER);
		setPreferredSize(new Dimension(1, 80));
        
        viewPortMover = new Runnable() {
            public void run() {
                if(selectedUnit != null) {
                    //Get a note for the unit
                    NoteWidget[] n = getNoticesForUnit(selectedUnit);
                    if(n.length > 0) {                        
                        NoteWidget w = n[n.length-1];
                    	scrollPane.getViewport().scrollRectToVisible(
                                w.getBounds());
                        
                        scrollPane.getViewport().setViewPosition(w.getLocation());
                        
                        /*
                        System.out.println("Screen position: "  + scrollPane.getViewport().getViewRect());                        
                        System.out.println("Scrolling to "  + w.getBounds());
                        */
                    	repaint();                        
                    } 
                }
            }            
        };
        

	}

	public void displayUnitNotice(TransferUnit unit) {
		if(unit != null) {
			//Add this if necessary			    
			selectedUnit = unit;
            SwingUtilities.invokeLater(viewPortMover);
            
            //Check if we should pause between packets
            SettingsTSC settings = getSettings();
            int pause = settings.getNoticeDelaySeconds();
            
            List widgets = (List) unitMap.get(unit);
   
            if(pause > 0 && settings.getTriggeredNotices(unit).size() > 0)  {             
            	mainPanel.toPauseMode(pause);
            }
        }
	}

    public void paintNoticeLines(Graphics g) {
        if(this.isVisible() && unitMap != null && unitMap.size() > 0) {
        	Iterator mapIter = unitMap.values().iterator();           
            
            while(mapIter.hasNext()) {
                List list = (List) mapIter.next();
                
                if(list == null || list.size()==0) continue;
                
                Iterator listIter = list.iterator();
                while(listIter.hasNext()) {
                	NoteWidget w = (NoteWidget) listIter.next();
                    
                    if(w == null) {
                    	continue;
                    }                    
                    Rectangle a = scrollPane.getViewport().getViewRect();                    
                    Rectangle b = w.getBounds();
                    
                    if(SwingUtilities.isRectangleContainingRectangle(a,b)) {
                    	w.paintNoteLine(g);                        
                    }
                }
            }            
        }
    }    
    
    private NoteWidget[] getNoticesForUnit(TransferUnit u) {
        List l = (List) unitMap.get(u);
        
        if(l != null) {
        	return (NoteWidget[]) l.toArray(new NoteWidget[] { });            
        } else {
        	return new NoteWidget[] { };
        }        
    }
    
    private void updateNoteWidgets() {        
    	SettingsTSC settings = getSettings();
    	
    	//Get unitlist from drawingarea
    	List units = drawingArea.getUnits();
    	if(units == null) return;        

        Iterator iter = units.iterator();
        int count = 0;
    	while(iter.hasNext()) {
            //System.out.println("----------");
    		TransferUnit u = (TransferUnit) iter.next();
            
            
            //See if this unit isn't in our unitMap yet
            if(!unitMap.containsKey(u)) {
            	//Create List as value
                unitMap.put(u,new LinkedList());
            }
            
            //Create a set of all the noticetriggers that trigger for this unit
        	HashSet triggerSet;
            triggerSet = new HashSet(settings.getTriggeredNotices(u));
            //System.out.println(triggerSet.size()+ " triggered");
           
            
            //Get current noticewidgets for this unit
            List widgets = (List) unitMap.get(u);
                            
            if(widgets == null) {
            	continue;
            } 
            
            //See which NoticeWidgets are not triggered anymore and should be removed
            Iterator widgetIterator = widgets.iterator();
            while(widgetIterator.hasNext()) {                
                NoteWidget w = (NoteWidget) widgetIterator.next();
                if(!triggerSet.contains(w.trigger) || !drawingArea.isUnitVisible(u)) {                
                    //This trigger should be removed from the unitlist since it's no longer valid
                   noteContainer.remove(w);
                   widgetIterator.remove();
                } else {
                	//System.out.println(w.trigger +" found in triggerSet " + triggerSet.size());
                }
            }           
                
            //See which of these triggers should be added to the list
            if(drawingArea.isUnitVisible(u)) { //Not added if unit isn't visible
                if(!removeTriggersFromCollection(triggerSet,widgets)) {
                    //No units were removed, should we be concerned????
                }
                
                Iterator triggerIterator = triggerSet.iterator();
                int counter=0;
                while(triggerIterator.hasNext()) {
                    NoticeTrigger trigger = (NoticeTrigger) triggerIterator.next();
                    
                    //Create new widget for this trigger
                    NoteWidget w = new NoteWidget(u,trigger);                    
                    widgets.add(w);
                    noteContainer.add(w,-1);              
                    counter++;
                }                
            }
    	}
    	
        validate();
    	
    }
    
    private boolean removeTriggersFromCollection(Collection a,Collection b) {
        int size = a.size();
        
        Iterator iter = b.iterator();
        while(iter.hasNext()) {
        	NoteWidget w = (NoteWidget) iter.next();
            a.remove(w.trigger);            
        }
        return a.size() < size;
    }
    
    private void printCollection(Collection  c) {
    	Iterator iter = c.iterator();
        System.out.println("BEGIN ==== '" + c + "' =====");
        while(iter.hasNext()) {
        	System.out.println(" '" + iter.next()+ "'");        	
        }
        System.out.println("END ==== '"  + c + "' =====");
        
    }
    
    private void addNoteWidgets(TransferUnit u,NoticeTrigger[] triggers,Map newMap) {
    	if(unitMap.containsKey(u)) return;
                
        List list = new LinkedList();
        list = new LinkedList();
        SettingsTSC tscSettings = getSettings();
        
        for(int i=0; i < triggers.length;i++) {
        	NoteWidget w = new NoteWidget(u,triggers[i]);
            list.add(w);
        }
        newMap.put(u,list);
    }
    
    public void paintComponent(Graphics g) {
        updateNoteWidgets();
    }
    
    
    private class NoteWidget extends JTextArea {
		private TransferUnit unit;

		private boolean selected = false;
        private NoticeTrigger trigger;

		public NoteWidget(TransferUnit u,NoticeTrigger trigger) {
			unit = u;			
            this.trigger = trigger;
			this.setPreferredSize(new Dimension(150, 50));
			this.setBorder(new LineBorder(Color.BLACK, 1));

			setText(trigger.getNotice());

			setToolTipText(getText());
			setEditable(false);
			//setEnabled(false);
			setWrapStyleWord(true);
			setLineWrap(true);                    
            
		}

        public NoticeTrigger getNoticeTrigger() {
        	return trigger;
        }
        
		public void setSelected(boolean sel) {
			selected = sel;

			if (sel) {
				setForeground(Color.BLACK);
			} else {
				setForeground(Color.GRAY);
			}
		} 
        
        public TransferUnit getUnit() {
        	return this.unit;
        }

		public boolean isSelected() {
			return this.unit == selectedUnit;
		}
        
        public void paintNoteLine(Graphics g) {
            if (isVisible() && drawingArea.isUnitVisible(unit) && mainPanel.isVisible()) {
                //Draw the line fr om the selected NoteWidget to the packet in
                // the DrawingArea.                
                Graphics pg = mainPanel.getGraphics(); 
                
                Point myPoint = getLocationOnScreen();                                
                Point unitPoint = drawingArea.getUnitCoordinates(unit);                
                Point mainPoint = mainPanel.getLocationOnScreen();
                
                myPoint.x+= getWidth()/2;
                myPoint.x-=mainPoint.x;
                myPoint.y-=mainPoint.y; 
                
                
                pg.setColor(Color.BLACK);                                
                g.drawLine(+myPoint.x,+myPoint.y,unitPoint.x,unitPoint.y);      
                g.drawLine(+myPoint.x+1,+myPoint.y,unitPoint.x+1,unitPoint.y);
            }
        }    
        public String toString() {
        	return trigger.getNotice();
        }
        
        public boolean equals(Object b) {
        	if(b == null) return false;
            
            if(b instanceof NoteWidget) {
            	NoteWidget w = (NoteWidget) b;
                if(w.trigger.equals(trigger) &&
                   w.unit.equals(unit)) {
                	return true;
                } else return false;                
            } else
            if(b instanceof NoticeTrigger) {                 
            	NoticeTrigger t = (NoticeTrigger) b;             
                if(t.equals(trigger)) return true;
                else return false;
            }
            return false;   	
        }
	}
    
}