/*
 * Created on 13.4.2005
 */
package fi.helsinki.dacopan.ui.tsc;

import java.awt.Color;
import java.util.HashMap;
import java.util.HashSet;
import java.util.LinkedList;
import java.util.List;
import java.util.Map;
import java.util.Set;
import java.util.Vector;

import fi.helsinki.dacopan.model.TransferUnit;

/**
 * @author mjvaanan
 */
public class SettingsTSC {
	public static final String COLORKEY_ACK = "color_ack";
	public static final String COLORKEY_ACTIVE = "color_active";
	public static final String COLORKEY_ADVERTIZED_WINDOW_LINE = "color_advertized_window_line";
	public static final String COLORKEY_CONNECTING_LINE = "color_connecting_line";
	public static final String COLORKEY_DELAY_LINE = "color_delay_line";
	public static final String COLORKEY_FIN = "color_fin";
	public static final String COLORKEY_GRID_MAJOR = "color_grid_major";
	public static final String COLORKEY_GRID_MINOR = "color_grid_minor";
	public static final String COLORKEY_SACK = "color_sack";
	public static final String COLORKEY_SEGMENT = "color_segment";
	public static final String COLORKEY_SYN = "color_syn";

	public static final String ELEMENTKEY_ACK = "element_ack";
	public static final String ELEMENTKEY_ADVERTIZED_WINDOW_LINE = "element_advertized_window";
	public static final String ELEMENTKEY_CONNECTING_LINE = "element_connecting_line";
	public static final String ELEMENTKEY_CROSSHAIR = "element_crosshair";
	public static final String ELEMENTKEY_DELAY_LINE = "element_delay_line";
	public static final String ELEMENTKEY_GRID = "element_grid";
	public static final String ELEMENTKEY_FIN = "element_fin";
	public static final String ELEMENTKEY_NOTICES = "element_notices";
	public static final String ELEMENTKEY_SACK = "element_sack";
	public static final String ELEMENTKEY_SEGMENT = "element_segment";
	public static final String ELEMENTKEY_SYN = "element_syn";
	
	private float speed = 1;
	private int pause_notice_sec = 1;
	private Map colors = new HashMap();		// String --> Color
	private Map elements = new HashMap();	// String --> Boolean
	private Set variables = new HashSet();	// String
	private boolean relative_seqno = true;
	private Vector triggers;
	
	// non-global settings
	private int host_id = -1;
	private long flow_id = -1;
	private double x_scale = -1;
	private double y_scale = -1;
	
	private boolean highlight_syn = true;
	private boolean highlight_fin = true;
	
	public SettingsTSC() {
		
		// defaults
		setColor(COLORKEY_ACK, Color.WHITE);
		setColor(COLORKEY_ACTIVE, Color.RED);
		setColor(COLORKEY_ADVERTIZED_WINDOW_LINE, Color.GREEN);
		setColor(COLORKEY_CONNECTING_LINE, Color.BLUE);
		setColor(COLORKEY_DELAY_LINE, Color.BLACK);
		setColor(COLORKEY_FIN, Color.GREEN);
		setColor(COLORKEY_GRID_MAJOR, Color.BLACK);
		setColor(COLORKEY_GRID_MINOR, new Color(150, 150, 150));
		setColor(COLORKEY_SACK, Color.ORANGE);
		setColor(COLORKEY_SEGMENT, new Color(100, 100, 255));
		setColor(COLORKEY_SYN, Color.YELLOW);
		
		// defaults
		String[] elems = new String[] {
			
			ELEMENTKEY_ACK,
			ELEMENTKEY_ADVERTIZED_WINDOW_LINE,
			ELEMENTKEY_CONNECTING_LINE,
			ELEMENTKEY_CROSSHAIR,
			ELEMENTKEY_DELAY_LINE,
			ELEMENTKEY_FIN,
			ELEMENTKEY_GRID,
			ELEMENTKEY_NOTICES,
			ELEMENTKEY_SACK,
			ELEMENTKEY_SEGMENT,
			ELEMENTKEY_SYN
		};
		for (int i = 0; i < elems.length; i++) {
			setElementDisplayed(elems[i], true);
		}
		triggers = new Vector();
	}

    public SettingsTSC(SettingsTSC newSettings) {
    	this.speed = newSettings.speed;
    	this.pause_notice_sec = newSettings.pause_notice_sec;
    	this.relative_seqno = newSettings.relative_seqno;
    	this.host_id = newSettings.host_id;
    	this.flow_id = newSettings.flow_id;
    	this.x_scale = newSettings.x_scale;
    	this.y_scale = newSettings.y_scale;
    	this.highlight_fin = newSettings.highlight_fin;
    	this.highlight_syn = newSettings.highlight_syn;
    	this.variables = new HashSet(newSettings.variables);

    	this.colors = new HashMap(newSettings.colors);
    	this.elements = new HashMap(newSettings.elements);
    	this.triggers = new Vector(newSettings.triggers);
    }	
	
	public float getAnimationSpeed() {
		return speed;
	}
	
	public void setAnimationSpeed(float speed) {
		this.speed = speed;
	}
	
	public int getNoticeDelaySeconds() {
		return pause_notice_sec;
	}
	
	public void setNoticeDelaySeconds(int sec) {
		pause_notice_sec = sec;
	}
	
	public boolean isElementDisplayed(String key) {
		Boolean value = (Boolean) elements.get(key);
		if (value != null) {
			return value.booleanValue();
		} else {
			return false;
		}
	}
	
	public void setElementDisplayed(String key, boolean flag) {
		elements.put(key, new Boolean(flag));
	}
	
	public Color getColor(String key) {
		if (key.equals(COLORKEY_FIN)) {
			if (highlight_fin == true) {
				return (Color) colors.get(COLORKEY_FIN);
			} else {
				return (Color) colors.get(COLORKEY_SEGMENT);				
			}
		} else if (key.equals(COLORKEY_SYN)) {
			if (highlight_syn == true) {
				return (Color) colors.get(COLORKEY_SYN);
			} else {
				return (Color) colors.get(COLORKEY_SEGMENT);				
			}
		} else {
			return (Color) colors.get(key);
		}
	}
	
	public Color getFINColor() {
		return (Color) colors.get(COLORKEY_FIN);
	}
	
	public Color getSYNColor() {
		return (Color) colors.get(COLORKEY_SYN);
	}
	
	public void setColor(String key, Color color) {
		colors.put(key, color);
	}
	
	public void highlightUnit(String key, boolean flag) {
		if (key.equals(ELEMENTKEY_FIN)) {
			highlight_fin = flag;
		}
		else if (key.equals(ELEMENTKEY_SYN)) {
			highlight_syn = flag;
		}
	}
	
	public boolean isUnitHighlighted(String key) {
		boolean highlighted = false;
		if (key.equals(ELEMENTKEY_FIN) && highlight_fin) {
			highlighted = highlight_fin;
		}
		else if (key.equals(ELEMENTKEY_SYN) && highlight_syn) {
			highlighted = highlight_syn;
		}
		return highlighted;
	}

	/**
	 * @return variable raw names
	 */
	public String[] getDisplayedData() {
		return (String[]) variables.toArray(new String[0]);
	}
	
	public void setDisplayedData(String[] data) {
		variables = new HashSet();
		for (int i = 0; i < data.length; i++) {
			variables.add(data[i]);
		}
	}
	
	public boolean isVariableDisplayed(String variable) {
		return variables.contains(variable);
	}
	
	public NoticeTrigger[] getNoticeTriggers() {
		return (NoticeTrigger [])triggers.toArray(new NoticeTrigger[0]);
	}

	public Vector getVectorNoticeTriggers() {
		return this.triggers;
	}
    
    
    public List getTriggeredNotices(TransferUnit u) {
    	List l = new LinkedList();
        NoticeTrigger[] triggers = getNoticeTriggers();
        
        for(int i=0; i < triggers.length;i++) {
            if(triggers[i].triggered(u)) {
                l.add(triggers[i]);                
            }
        }
        return l;
    }
	
	public void setNoticeTriggers(NoticeTrigger[] triggers) {
		this.triggers.removeAllElements();
		for (int i = 0; i<triggers.length; i++)
			this.addNoticeTrigger(triggers[i]);
	}
	
	public void addNoticeTrigger(NoticeTrigger trigger) {
		this.triggers.add((Object)trigger);
	}
	
	public boolean isRelativeSEQNO() {
		return relative_seqno;
	}
	
	public void setRelativeSEQNO(boolean flag) {
		relative_seqno = flag;
	}
	
	/* non-global settings */

	public void setFlowId(long flow_id) {
		this.flow_id = flow_id;
	}
	
	public long getFlowId() {
		return flow_id;
	}
	
	public void setHostId(int host_id) {
		this.host_id = host_id;
	}
	
	public int getHostId() {
		return host_id;
	}
	
	public void setXScale(double x_scale) {
		this.x_scale = x_scale;
	}
	
	public double getXScale() {
		return x_scale;
	}
	
	public void setYScale(double y_scale) {
		this.y_scale = y_scale;
	}
	
	public double getYScale() {
		return y_scale;
	}
	
    
	public void resetNonGlobalSettings() {
		flow_id = -1;
		host_id = -1;
		x_scale = -1;
		y_scale = -1;
	}
}
