/*
 * Created on 3.3.2005
 */
package fi.helsinki.dacopan.ui.tsc;

import java.awt.Color;
import java.awt.Dimension;
import java.awt.Graphics;
import java.text.DecimalFormat;
import java.text.NumberFormat;

import javax.swing.JPanel;

import fi.helsinki.dacopan.Localization;

/**
 * @author mjvaanan
 */
public class Xaxis extends JPanel {
	public static final int SCALE_SECONDS = 0;
	public static final int SCALE_MILLISECONDS = 1;
	
	private Label[] labels;
	private double time_window;
	private NumberFormat second_formatter = new DecimalFormat();
	private String title = Localization.getString("panel.tsc.xaxis.title");
	private String scale_s = Localization.getString("panel.tsc.xaxis.scale_s");
	private String scale_ms = Localization.getString("panel.tsc.xaxis.scale_ms");
	
	public Xaxis() {
		second_formatter.setMinimumFractionDigits(1);
		second_formatter.setMaximumFractionDigits(1);
	}

	public Dimension getPreferredSize()
	{
		return new Dimension(20, 20);
	}
	
	public Dimension getMinimumSize()
	{
		return getPreferredSize();
	}
	
	void setLabels(Label[] labels) {
		this.labels = labels;
	}
	
	void setExtent(double time_window) {
		this.time_window = time_window;
	}
	
	private float round(float time, int scale) {
		time *= scale;
		time = Math.round(time);
		time /= scale;
		return time;
	}
	
	private String getText(float time, int level) {
		if (time_window > 1.5) {
			return new Integer((int) Math.round(time)).toString();
		} else {
			int time_int = (int) time;
			float decimal_part = time - time_int;
			if (time_window > 0.01) {
				decimal_part = round(decimal_part, 100);
			} else {
				decimal_part = round(decimal_part, 1000);
			}
			decimal_part *= 1000;
			if ((int) decimal_part == 1000) {
				return "0";
			}
			return new Integer((int) decimal_part).toString();
		}
	}
	
	public void paintComponent(Graphics g) {
		if (labels != null) {
			String scale_str = null;
			int level = 0;
			if (time_window > 1.5) {
				level = 1;
				scale_str = scale_s;
			} else {
				level = 0;
				scale_str = scale_ms;
			}
			g.setColor(Color.BLACK);
			g.drawString(title + " (" + scale_str + ") ", 0, getSize().height - 5);
			for (int i = 0; i < labels.length; i++) {
				int x = labels[i].getPos();
				if (x > 70 && x < getSize().width - 50) {
					String text = getText(labels[i].getTime(), level);
					g.drawString(text, x, getSize().height - 5);
				}
			}
		}
	}
	
	public static class Label {
		private int pos;
		private float time;
		
		public Label(int pos, float time) {
			this.pos = pos;
			this.time = time;
		}

		public int getPos() {
			return pos;
		}
		
		public float getTime() {
			return time;
		}
	}

}
