;--------------------------------
; DaCoPAn Windows installer
; Uses NullSoft NSIS 2.0
;
; http://www.cs.helsinki.fi/group/dacopan
;
; Usage instructions:
; - Move all files in animator/istaller to C:\dacopan
; - Use NSIS compiler to compile installer 
;--------------------------------


;Include Modern UI

  !include "MUI.nsh"

;Variables
;--------------------------------

  Var STARTMENU_FOLDER
  Var MUI_TEMP

; General settings
;--------------------------------
  ; The name of the installer
  Name "DaCoPAn animator"

  ; The file to write
  OutFile "c:\dacopan\dacopan-animator-1.0.exe"

  ; The default installation directory
  InstallDir $PROGRAMFILES\DaCoPAn\animator

  ; Registry key to check for directory (so if you install again, it will
  ; overwrite the old one automatically)
  InstallDirRegKey HKLM "Software\DaCoPAn" "Install_Dir"


;Interface Configuration
;--------------------------------

  !define MUI_HEADERIMAGE
  !define MUI_ABORTWARNING

  !define MUI_HEADERIMAGE_BITMAP C:\dacopan\logo.bmp
  !define MUI_BGCOLOR B1B1B1
  !define MUI_COMPONENTSPAGE_NODESC


; Pages
;--------------------------------

  !insertmacro MUI_PAGE_LICENSE "C:\dacopan\license.txt"
  !insertmacro MUI_PAGE_COMPONENTS
  !insertmacro MUI_PAGE_DIRECTORY

  ;Start Menu Folder Page Configuration
  !define MUI_STARTMENUPAGE_REGISTRY_ROOT "HKCU"
  !define MUI_STARTMENUPAGE_REGISTRY_KEY "Software\DaCoPan"
  !define MUI_STARTMENUPAGE_REGISTRY_VALUENAME "Start Menu Folder"

  !insertmacro MUI_PAGE_STARTMENU Application $STARTMENU_FOLDER

  !insertmacro MUI_PAGE_INSTFILES

  !insertmacro MUI_UNPAGE_CONFIRM
  !insertmacro MUI_UNPAGE_INSTFILES


;Languages
;--------------------------------

  !insertmacro MUI_LANGUAGE "English"


;--------------------------------

; The stuff to install
Section "DaCoPAn animator (required)" dacopan

  SectionIn RO
  
  ; Set output path to the installation directory.
  SetOutPath $INSTDIR
  
  ; Put file there
  File "C:\dacopan\animator.jar"
  File "C:\dacopan\animator.bat"
  File "C:\dacopan\dacopan.ico"

  ; Write start menu information
  !insertmacro MUI_STARTMENU_WRITE_BEGIN Application

    ;Create shortcuts
    CreateDirectory "$SMPROGRAMS\$STARTMENU_FOLDER"
    CreateShortCut "$SMPROGRAMS\$STARTMENU_FOLDER\DaCoPAn animator.lnk" "$INSTDIR\animator.bat" parameters "$INSTDIR\dacopan.ico"
    CreateShortCut "$SMPROGRAMS\$STARTMENU_FOLDER\Uninstall.lnk" "$INSTDIR\Uninstall.exe"

  !insertmacro MUI_STARTMENU_WRITE_END


  ; Write the installation path into the registry
  WriteRegStr HKLM SOFTWARE\DaCoPAn "Install_Dir" "$INSTDIR"
  
  ; Write the uninstall keys for Windows
  WriteRegStr HKLM "Software\Microsoft\Windows\CurrentVersion\Uninstall\DaCoPAn" "DisplayName" "NSIS Example2"
  WriteRegStr HKLM "Software\Microsoft\Windows\CurrentVersion\Uninstall\DaCoPAn" "UninstallString" '"$INSTDIR\uninstall.exe"'
  WriteRegDWORD HKLM "Software\Microsoft\Windows\CurrentVersion\Uninstall\DaCoPAn" "NoModify" 1
  WriteRegDWORD HKLM "Software\Microsoft\Windows\CurrentVersion\Uninstall\DaCoPAn" "NoRepair" 1
  WriteUninstaller "uninstall.exe"
  
SectionEnd

; Optional section (can be disabled by the user)
Section "Example scenario files" scenarios

  ; Set output path to the installation directory.
  SetOutPath $INSTDIR\scenarios

  ; Put file there
  File "C:\dacopan\example.sce"

  SetOutPath $INSTDIR

SectionEnd


; Optional section (can be disabled by the user)
Section "Example protocol events files" pefs

  ; Set output path to the installation directory.
  SetOutPath $INSTDIR\pefs

  ; Put file there
  File "C:\dacopan\pefs\*.pef"

  SetOutPath $INSTDIR

SectionEnd

; Uninstaller
;--------------------------------

Section "Uninstall"
  
  ; Remove registry keys
  DeleteRegKey HKLM "Software\Microsoft\Windows\CurrentVersion\Uninstall\DaCoPAn"
  DeleteRegKey HKLM SOFTWARE\DaCoPAn

  ; Remove files and uninstaller
  Delete $INSTDIR\animator.bat
  Delete $INSTDIR\animator.jar
  Delete $INSTDIR\dacopan.ico
  Delete $INSTDIR\uninstall.exe

  ; Remove optional example pef files
  Delete $INSTDIR\pefs\*.*
  Delete $INSTDIR\scenarios\*.*

  ; Remove shortcuts, if any
  Delete "$SMPROGRAMS\DaCoPAn\*.*"

  ; Remove directories used
  RMDir "$SMPROGRAMS\DaCoPAn"
  RMDir "$INSTDIR\pefs"
  RMDir "$INSTDIR\scenarios"
  RMDir "$INSTDIR"
  
  !insertmacro MUI_STARTMENU_GETFOLDER Application $MUI_TEMP

  Delete "$SMPROGRAMS\$MUI_TEMP\Uninstall.lnk"
  Delete "$SMPROGRAMS\$MUI_TEMP\DaCoPAn animator.lnk"

  ;Delete empty start menu parent diretories
  StrCpy $MUI_TEMP "$SMPROGRAMS\$MUI_TEMP"

  startMenuDeleteLoop:
    RMDir $MUI_TEMP
    GetFullPathName $MUI_TEMP "$MUI_TEMP\.."

    IfErrors startMenuDeleteLoopDone

    StrCmp $MUI_TEMP $SMPROGRAMS startMenuDeleteLoopDone startMenuDeleteLoop
  startMenuDeleteLoopDone:

  DeleteRegKey /ifempty HKCU "Software\DaCoPAn"


SectionEnd
