/*
 * Created on 27.4.2005
 *
 */
package fi.helsinki.dacopan.contsig;

import fi.helsinki.dacopan.AbstractTestCase;
import fi.helsinki.dacopan.animseq.AnimationSequence;
import fi.helsinki.dacopan.model.DataView;
import fi.helsinki.dacopan.model.NoteManager;
import fi.helsinki.dacopan.model.ScenarioStepIterator;
import fi.helsinki.dacopan.model.StepIterator;
import fi.helsinki.dacopan.ui.TimePanel;

/**
 * @author pamietti
 *
 * JUnit test for AnimationTimeState.
 */
public class AnimationTimeStateTest extends AbstractTestCase {

	private DataView data;
	public static float DELTA = 0.0000001f;
	
	/*
	 * @see TestCase#setUp()
	 */
	protected void setUp() throws Exception {
		data = getRandomData(10, 10, 10, 0.1f, false);
	}

	/*
	 * Class under test for void AnimationTimeState(float, float, float, int, StepIterator)
	 */
	public void testAnimationTimeStatefloatfloatfloatintStepIterator() {
		AnimationTimeState timeState = new AnimationTimeState(0.0f, data.getEndTime(), 1, 1, 
				new ScenarioStepIterator(data.getLayerById(1), data, new NoteManager()));
		assertTrue("timeState is null", timeState != null);
	}

	/*
	 * Class under test for void AnimationTimeState(float, float, float, int, StepIterator, AnimationSequence, float)
	 */
	public void testAnimationTimeStatefloatfloatfloatintStepIteratorAnimationSequencefloat() {
		AnimationTimeState timeState = new AnimationTimeState(0.0f, data.getEndTime(), 1, 1, 
				new ScenarioStepIterator(data.getLayerById(1), data, new NoteManager()),
				new AnimationSequence(null), data.getEndTime(), data.getEndTime());
		assertTrue("timeState is null", timeState != null);
	}

	public void testSetAndGetTimeScale() {
		AnimationTimeState timeState = new AnimationTimeState(0.0f, data.getEndTime(), 1, 1, 
				new ScenarioStepIterator(data.getLayerById(1), data, new NoteManager()));
		timeState.setTimeScale(20.1f);
		assertEquals("timeScales are not same", 20.1f, timeState.getTimeScale(), DELTA);
	}

	public void testSetAndGetStepInterval() {
		AnimationTimeState timeState = new AnimationTimeState(0.0f, data.getEndTime(), 1, 1, 
				new ScenarioStepIterator(data.getLayerById(1), data, new NoteManager()));
		timeState.setStepInterval(100);
		assertEquals("StepIntervals are not same", 100, timeState.getStepInterval());
	}

	public void testSetAndGetNowTime() {
		AnimationTimeState timeState = new AnimationTimeState(0.0f, data.getEndTime(), 1, 1, 
				new ScenarioStepIterator(data.getLayerById(1), data, new NoteManager()));
		float nowTime = data.getEndTimeForLayer(data.getLayerById(1)) - 0.01f;
		timeState.setNowTime(nowTime);
		assertEquals("nowTimes are not same", nowTime, timeState.getNowTime(), DELTA);
	}

	public void testIsPaused() {
		AnimationTimeState timeState = new AnimationTimeState(0.0f, data.getEndTime(), 1, 1, 
				new ScenarioStepIterator(data.getLayerById(1), data, new NoteManager()));
		timeState.play();
		assertEquals("time state is pause", false, timeState.isPaused());
		timeState.pause();
		assertEquals("time state is not pause", true, timeState.isPaused());
	}

	public void testAddAndRemoveControlSignalsListener() {
		AnimationTimeState timeState = new AnimationTimeState(0.0f, data.getEndTime(), 1, 1, 
				new ScenarioStepIterator(data.getLayerById(1), data, new NoteManager()));
		TimePanel timePanel = TimePanel.getInstance(null, 0.0f);
		timeState.addControlSignalsListener(timePanel);
		timeState.removeControlSignalsListener(timePanel);
	}

	public void testPlay() {
		AnimationTimeState timeState = new AnimationTimeState(0.0f, data.getEndTime(), 1, 1, 
				new ScenarioStepIterator(data.getLayerById(1), data, new NoteManager()));
		timeState.play();
		assertTrue("time state is not in play", !timeState.isPaused());
	}

	public void testPause() {
		AnimationTimeState timeState = new AnimationTimeState(0.0f, data.getEndTime(), 1, 1, 
				new ScenarioStepIterator(data.getLayerById(1), data, new NoteManager()));
		timeState.pause();
		assertTrue("time state is not in pause", timeState.isPaused());
	}

	public void testToBeginning() {
		AnimationTimeState timeState = new AnimationTimeState(0.0f, data.getEndTime(), 1, 1, 
				new ScenarioStepIterator(data.getLayerById(1), data, new NoteManager()));
		timeState.setNowTime(data.getEndTimeForLayer(data.getLayerById(1)));
		timeState.toBeginning();
		assertEquals("time state is not at beginning", 0.0f, timeState.getNowTime(), DELTA);
	}

	public void testToEnd() {
		AnimationTimeState timeState = new AnimationTimeState(0.0f, data.getEndTime(), 1, 1, 
				new ScenarioStepIterator(data.getLayerById(1), data, new NoteManager()));
		timeState.toEnd();
		assertEquals("time state is not at end", data.getEndTime(), timeState.getNowTime(), DELTA);
	}

	public void testToEndOfScenarioItem() {
		float endOfScenarioItem = data.getEndTimeForLayer(data.getLayerById(1)) - 0.01f;
		AnimationTimeState timeState = new AnimationTimeState(0.0f, data.getEndTime(), 1, 1, 
				new ScenarioStepIterator(data.getLayerById(1), data, new NoteManager()),
				new AnimationSequence(null), endOfScenarioItem, endOfScenarioItem);
		timeState.toEndOfScenarioItem();
		assertEquals("time state is not at end of scenario item", endOfScenarioItem, timeState.getNowTime(), DELTA);
	}

	public void testStepForward() {
		AnimationTimeState timeState = new AnimationTimeState(0.0f, data.getEndTime(), 1, 1, 
				new ScenarioStepIterator(data.getLayerById(1), data, new NoteManager()));
		timeState.stepForward();
		assertTrue("time state has not advanced", timeState.getNowTime() > 0);
	}

	public void testStepBackward() {
		AnimationTimeState timeState = new AnimationTimeState(0.0f, data.getEndTime(), 1, 1, 
				new ScenarioStepIterator(data.getLayerById(1), data, new NoteManager()));
		timeState.toEnd();
		float endTime = timeState.getNowTime();
		timeState.stepBackward();
		assertTrue("time state has not stepped backwards", endTime > timeState.getNowTime());
		timeState.stepForward();
		assertEquals("time state has not returned back to the end", endTime, timeState.getNowTime(), DELTA);
	}

	public void testDiscard() {
		AnimationTimeState timeState = new AnimationTimeState(0.0f, data.getEndTime(), 1, 1, 
				new ScenarioStepIterator(data.getLayerById(1), data, new NoteManager()));
		timeState.discard();
	}

}
