package fi.helsinki.dacopan.scenario;

import java.io.StringReader;
import java.io.StringWriter;
import java.util.ArrayList;
import java.util.List;

import junit.framework.TestCase;

/**
 * Test case for ObjectSerializer implementations.
 * 
 * @version $Id: ObjectSerializerTestCase.java,v 1.4 2004/05/07 09:19:42 aarniala-dacopan Exp $
 */
public class ObjectSerializerTestCase extends TestCase {
    
    protected ObjectSerializer serializer = new XStreamObjectSerializer();
    
    public void testSaveAndLoad() {
        List stuff = new ArrayList();
        stuff.add("abc");
        stuff.add(new Integer(123));
        stuff.add("def");
        A a = new A(stuff, 42, new B("value of b"));
        StringWriter writer = new StringWriter();
        serializer.saveObject(a, writer);
        A other = (A)serializer.loadObject(new StringReader(writer.toString()));
        assertNotNull("A null", other);
        assertEquals("List incorrect", stuff, other.stuff);
        assertEquals("int field incorrect", 42, other.i);
        assertEquals("B's string incorrect", "value of b", other.b.s);
    }
    
    public void testNullParameters() {
        try {
            serializer.saveObject(null, new StringWriter());
            fail("Should've failed with null object");
        } catch(IllegalArgumentException e) { }
        try {
            serializer.saveObject("abc", null);
            fail("Should've failed with null writer");
        } catch(IllegalArgumentException e) { }
        try {
            serializer.loadObject(null);
            fail("Should've failed with null reader");
        } catch(IllegalArgumentException e) { }
    }
    
    /* how to handle associated objects (transferunits etc) */
    
    public static class A {
        private List stuff;
        private int i;
        private B b;
        public A(List stuff, int i, B b) {
            this.stuff = stuff;
            this.i = i;
            this.b = b;
        }
    }
    
    public static class B {
        private String s;
        public B(String s) {
            this.s = s;
        }
    }
    
}
