package fi.helsinki.dacopan.scenario;

import java.io.File;
import java.io.FileNotFoundException;
import java.io.IOException;
import java.io.InputStreamReader;
import java.io.Reader;
import java.util.Collections;

import fi.helsinki.dacopan.AbstractTestCase;
import fi.helsinki.dacopan.model.DataView;
import fi.helsinki.dacopan.model.DefaultDataView;
import fi.helsinki.dacopan.model.Layer;
import fi.helsinki.dacopan.model.Note;
import fi.helsinki.dacopan.model.NoteManager;
import fi.helsinki.dacopan.model.TransferUnit;
import fi.helsinki.dacopan.pef.XMLProtocolEventsReader;

/**
 * A local integration test for scenario saving.
 * 
 * @version $Id: ScenarioSavingTestCase.java,v 1.4 2004/05/11 14:49:19 aarniala-dacopan Exp $
 */
public class ScenarioSavingTestCase extends AbstractTestCase {

    private File file;
    
    private void createFile() throws IOException {
        File tempDir = new File(System.getProperty("java.io.tmpdir"));
        assertTrue("Temp dir doesn't exist", tempDir.isDirectory());
        file = new File(tempDir, "dacopan_scenario_saving_test" + System.currentTimeMillis());
        assertFalse("Temp file already exists", file.exists());
    }
    
    protected void tearDown() throws Exception {
        if(file != null) {
            file.delete();
        }
    }
    
    private Reader getData() {
        return new InputStreamReader(getClass().getResourceAsStream("/fi/helsinki/dacopan/pef/a.xml"));
    }
    
    public void testSaveScenarioWithNotes() throws FileNotFoundException, IOException, InvalidScenarioFileException {

        // first create the view and add MSC and ENC notes to notemanager
        
        DataView view = new DefaultDataView();
        new XMLProtocolEventsReader().read(getData(), view);
        
        Layer layer = view.getLayerById(1);
        TransferUnit unit = view.getUnitById(1);
        assertNotNull(layer);
        assertNotNull(unit);

        Note mscNote = new Note(layer, 12.34f, "Hello, MSC world");
        Note encNote = new Note(unit, "Hello, ENC world");

        NoteManager manager = new NoteManager();
        manager.addNoteTimeLayer(mscNote);
        manager.addNoteEnc(encNote);
        
        // save them
        createFile();
        ScenarioFile scenarioFile = new ScenarioFile(file, getData());
        scenarioFile.save(Collections.singletonList(manager));
        
        // load them again, populate the dataview as well
        System.out.println(manager.getNotesEnc());
        manager = new NoteManager();
        view = new DefaultDataView();
        scenarioFile = new ScenarioFile(file);
        scenarioFile.load(Collections.singletonList(manager), view, null);
        System.out.println(manager.getNotesEnc());

        layer = view.getLayerById(1);
        unit = view.getUnitById(1);
        assertNotNull(layer);
        assertNotNull(unit);
        
        assertEquals("Enc note incorrect", encNote, manager.getNoteEnc(unit));
        assertEquals("MSC note incorrect", mscNote, manager.getNoteTimeLayerPrev(layer, 12.34f, true));
        
    }
    
}
