/*
 * Created on 27.4.2005
 */
package fi.helsinki.dacopan.settings;

import fi.helsinki.dacopan.AbstractTestCase;
import fi.helsinki.dacopan.model.DataView;
import fi.helsinki.dacopan.model.Layer;
import fi.helsinki.dacopan.model.VariableDefinition;
import fi.helsinki.dacopan.ui.SettingsMSC;
import fi.helsinki.dacopan.ui.SettingsMSCTest;

/**
 * @author pamietti
 *
 * JUnit tests for GeneralSetings.
 */
public class GeneralSettingsTest extends AbstractTestCase {

	private DataView data;
	private static final float DELTA = 0.0000001f;
	/*
	 * @see TestCase#setUp()
	 */
	protected void setUp() throws Exception {
		data = getRandomData(10, 10, 10, 0.01f, false);
	}

	/*
	 * Class under test for void GeneralSettings()
	 */
	public void testGeneralSettings() {
		GeneralSettings settings = new GeneralSettings();
	}

	/*
	 * Class under test for void GeneralSettings(GeneralSettings)
	 */
	public void testGeneralSettingsGeneralSettings() {
		GeneralSettings settings = new GeneralSettings();

		settings.setAntiAliasing(false);
		settings.setRefreshDelay(99);
		settings.setDefaultLayer(data.getLayerById(1));

		GeneralSettings otherSettings = new GeneralSettings(settings);
		assertEquals("refres delays are not same", settings.getRefreshDelay(), otherSettings.getRefreshDelay());
		assertEquals("anti-aliasings are not same", settings.getAntiAliasing(), otherSettings.getAntiAliasing());
		assertEquals("default layers are not same", settings.getDefaultLayer(), otherSettings.getDefaultLayer());
		
	}

	public void testSetAndGetRefreshDelay() {
		GeneralSettings settings = new GeneralSettings();
		settings.setRefreshDelay(99);
		assertEquals("refresh delays are not same", 99, settings.getRefreshDelay());
	}

	/* Deprecated methods 
	public void testSetAndGetRefreshRate() {
		GeneralSettings settings = new GeneralSettings();
		try {
			settings.setRefreshRate(4);
			fail("shuld have thrown IllegalArgumentException");
		} catch (IllegalArgumentException e) {}
		settings.setRefreshRate(150);
		assertEquals("refresh rates are not same", 150, settings.getRefreshRate());
	}
*/

	public void testSetAndGetDefaultLayer() {
		GeneralSettings settings = new GeneralSettings();
		Layer layer = data.getLayerById(1);
		settings.setDefaultLayer(layer);
		assertEquals("layers are not same", layer, settings.getDefaultLayer());
	}

	public void testAddAndGetSettingsMSC() {
		GeneralSettings settings = new GeneralSettings();
		SettingsMSC settingsMSC = new SettingsMSC(data.getHostById(1), data.getHostById(2), data.getLayerById(1), 
				new VariableDefinition[0], new VariableDefinition[0]);
		settings.addSettingsMSC(settingsMSC);
		SettingsMSCTest.assertSettingsEquals(settingsMSC, settings.getSettingsMSC(data.getLayerById(1)), false);
	}

	public void testSetAndGetListSettingsMSC() {
		GeneralSettings settings = new GeneralSettings();
		settings.setSettingsMSC(null);
		assertEquals("settingsMSC lists are not same", null, settings.getListSettingsMSC());
	}

	public void testSetAndGetAntiAliasing() {
		GeneralSettings settings = new GeneralSettings();
		settings.setAntiAliasing(false);
		assertEquals("antiAliasings are not same", false, settings.getAntiAliasing());
	}

	/*
	 * Class under test for String toString()
	 */
	public void testToString() {
		GeneralSettings settings = new GeneralSettings();
		settings.setAntiAliasing(false);
		settings.setRefreshDelay(99);
		settings.setDefaultLayer(data.getLayerById(1));
		assertEquals("strings are not same", "GeneralSettings { refreshDelay: 99 / defaultLayer"+data.getLayerById(1).getName()
				+" / timeScale / antiAliasingfalse }", settings.toString());
	}

	public void testSetAndGetData() {
		GeneralSettings settings = new GeneralSettings();
		GeneralSettings otherSettings = new GeneralSettings();
		otherSettings.setData(settings.getData());
		
	}

	public void testSetAndGetSettingsTSC() {
		GeneralSettings settings = new GeneralSettings();
		fi.helsinki.dacopan.ui.tsc.SettingsTSC settingsTSC = new fi.helsinki.dacopan.ui.tsc.SettingsTSC();
		settings.setSettingsTSC(settingsTSC);
		settingsTSC = settings.getSettingsTSC();
	}

}
