/*
 * Created on 25.4.2005
 */
package fi.helsinki.dacopan.ui;

import fi.helsinki.dacopan.AbstractTestCase;
import fi.helsinki.dacopan.model.DataView;
import fi.helsinki.dacopan.model.Protocol;
import fi.helsinki.dacopan.model.VariableDefinition;

/**
 * JUnit test for SettingsMSC. Test all setters and getters and constructors.
 * @author pamietti
 *
 */
public class SettingsMSCTest extends AbstractTestCase {
	
	private static float DELTA = 0.0000001f;
	
	public void testConstructors() {
		try {
			SettingsMSC settings = new SettingsMSC(null, null, null, null, null);
			fail("SettingsMSC constructor should have failed");
		} catch (IllegalArgumentException e) {
			assertEquals("Exception text is not equal", "Null parameters not accepted!", e.getMessage());
		}
		try {
			SettingsMSC settings = new SettingsMSC(null);
			fail("SettigsMSC constructor should have failed");
		} catch (IllegalArgumentException e) {
			assertEquals("Exception text is not equal", "Null parameters not accepted!", e.getMessage());
		}
		
		DataView data = getRandomData(10, 10, 10, 0.01f, false);
		SettingsMSC settings = new SettingsMSC(data.getHostById(1), data.getHostById(2), data.getLayerById(1),
				new VariableDefinition[0], new VariableDefinition[0]);

		assertTrue("Left host is not correct", data.getHostById(1).equals(settings.getLeftHost()));
		assertTrue("Right host is not correct", data.getHostById(2).equals(settings.getRightHost()));
		assertTrue("Layer is not correct", data.getLayerById(1).equals(settings.getLayer()));
		assertEquals("variablesCenter nro is not correct", settings.getVariablesCenter().length, 0);
		assertEquals("variablesColumns is not correct", settings.getVariablesColumns().length, 0);
		
		SettingsMSC otherSettings = new SettingsMSC(settings);
		
		assertSettingsEquals(settings, otherSettings, false);
	}

	public void testSettersAndGetters() {
		DataView data = getRandomData(10, 10, 10, 0.01f, false);
		SettingsMSC settings = new SettingsMSC(data.getHostById(1), data.getHostById(2), data.getLayerById(1), 
				new VariableDefinition[0], new VariableDefinition[0]);
		
		int scaleMode = 1;
		settings.setScaleMode(scaleMode);
		assertEquals("Scale mode is not correct", scaleMode, settings.getScaleMode());
		
		float timeScale = 99.99f;
		settings.setTimeScale(timeScale);
		assertEquals("Time scale is not correct", timeScale, settings.getTimeScale(), DELTA);
		
		int visualScale = 99;
		settings.setVisualScale(visualScale);
		assertEquals("Visual scale is not correct", visualScale, settings.getVisualScale());
		
		Protocol proto = new Protocol(1, "PROTOCOL", data.getLayerById(1));
		
		VariableDefinition[] variablesCenter = new VariableDefinition[1];
		variablesCenter[0] = new VariableDefinition("variable", VariableDefinition.UNIT, proto);
		settings.setVariablesCenter(variablesCenter);
		assertEquals("VariablesCenter has wrong length", 1, settings.getVariablesCenter().length);
		assertTrue("VariablesCenter is not correct", variablesCenter[0].equals(settings.getVariablesCenter()[0]));
		
		VariableDefinition[] variablesColumns = new VariableDefinition[1];
		variablesColumns[0] = new VariableDefinition("variableColumn", VariableDefinition.UNIT_FIELD, proto);
		settings.setVariablesColumns(variablesColumns);
		assertEquals("VariablesColumns has wrong length", 1, settings.getVariablesColumns().length);
		assertTrue("VariablesColumns is not correct", variablesColumns[0].equals(settings.getVariablesColumns()[0]));
		/* Deprecated method
		assertTrue("Protocol is null", settings.getProtocol() != null);
		assertTrue("Protocols is not correct", proto.equals(settings.getProtocol()));
		*/
		float startTime = 99.99f;
		settings.setStartTime(startTime);
		assertEquals("Start time is not correct", startTime, settings.getStartTime(), DELTA);
		try {
			settings.setStartTime(-0.1f);
			fail("Negative start time allowed");
		} catch (IllegalArgumentException e) {
			assertEquals("Exception text is not correct", "Start time cannot be negative!", e.getMessage());
		}
		
		float endTime = 999.99f;
		settings.setEndTime(endTime);
		assertEquals("End time is not correct", endTime, settings.getEndTime(), DELTA);
		try {
			settings.setEndTime(0.01f);
			fail("End time before start time allowed");
		} catch (IllegalArgumentException e) {
			assertEquals("Exception text is not correct", "Cannot change end time to be before start time!", e.getMessage());
		}
		try {
			settings.setEndTime(-0.01f);
			fail("Negative end time allowed");
		} catch (IllegalArgumentException e) {
			assertEquals("Exception text is not correct", "End time cannot be zero or negative!", e.getMessage());
		}
		try {
			settings.setStartTime(9999.99f);
			fail("Start time after end time allowed");
		} catch (IllegalArgumentException e) {
			assertEquals("Exception text is not correct", "Cannot change start time to be after end time!", e.getMessage());
		}
		
		boolean scenarioAutoPlay = true;
		settings.setScenarioAutoPlay(scenarioAutoPlay);
		assertEquals("Scenario auto play is not correct", scenarioAutoPlay, settings.getScenarioAutoPlay());
		
		int minStepping = 9;
		settings.setMinStepping(minStepping);
		assertEquals("Minimum stepping is not correct", minStepping, settings.getMinStepping());
		
		int maxStepping = 99;
		settings.setMaxStepping(maxStepping);
		assertEquals("Maximum stepping is not correct", maxStepping, settings.getMaxStepping());
		
		int fontSize = 9;
		settings.setFontSize(fontSize);
		assertEquals("Font size is not correct", fontSize, settings.getFontSize());
		
		int progressSpeed = 99;
		settings.setProgressSpeed(progressSpeed);
		assertEquals("Progress speed is not correct", progressSpeed, settings.getProgressSpeed());
		
		boolean showTimeStamps = false;
		settings.switchShowTimeStamps();
		assertEquals("Time stamps is not correct", showTimeStamps, settings.showTimeStamps());
		
		boolean linearMode = true;
		settings.setLinearMode(linearMode);
		assertEquals("Linear mode is not correct", linearMode, settings.isLinearMode());
		
		SettingsMSC otherSettings = (SettingsMSC)settings.clone();
		
		assertSettingsEquals(settings, otherSettings, false);
		
		otherSettings = new SettingsMSC(data.getHostById(1), data.getHostById(2), data.getLayerById(1),
				new VariableDefinition[0], new VariableDefinition[0]);
		otherSettings.setAllFields(settings);
		assertSettingsEquals(settings, otherSettings, true);
	}

	public static void assertSettingsEquals(SettingsMSC settings, SettingsMSC otherSettings, boolean testStartAndEndTimes) {

		assertTrue("Left hosts are not equals", settings.getLeftHost().equals(otherSettings.getLeftHost()));
		assertTrue("Right hosts are not equals", settings.getRightHost().equals(otherSettings.getRightHost()));
		assertTrue("Layers are not equals", settings.getLayer().equals(otherSettings.getLayer()));
		
		/* Deprecated method
		if (settings.getProtocol() != null) {
			assertTrue("Protocols are not equals", settings.getProtocol().equals(otherSettings.getProtocol()));
		} else {
			assertTrue("Protocols are not nulls", otherSettings.getProtocol() == null);
		}
		*/
		assertEquals("Scale modes are not equals", settings.getScaleMode(), otherSettings.getScaleMode());
		assertEquals("Time scales are not equals", settings.getTimeScale(), otherSettings.getTimeScale(), DELTA);
		assertEquals("Visual scales are not equals", settings.getVisualScale(), otherSettings.getVisualScale());
		for (int i=0; i < settings.getVariablesCenter().length; i++) {
			assertTrue("VariablesCenter["+i+"] are not equals", settings.getVariablesCenter()[i].equals(otherSettings.getVariablesCenter()[i]));
		}
		for (int i=0; i < settings.getVariablesColumns().length; i++) {
			assertTrue("VariablesColumns["+i+"] are not equals", settings.getVariablesColumns()[i].equals(otherSettings.getVariablesColumns()[i]));			
		}
		if (testStartAndEndTimes) {
			assertEquals("Start times are not equals", settings.getStartTime(), otherSettings.getStartTime(), DELTA);
			assertEquals("End times are not equals", settings.getEndTime(), otherSettings.getEndTime(), DELTA);
		}
		assertEquals("Scenario autoplays are not equals", settings.getScenarioAutoPlay(), otherSettings.getScenarioAutoPlay());
		assertEquals("Scale multipliers are not equals", settings.getScaleMultiplier(), otherSettings.getScaleMultiplier());
		assertTrue("Scale multiplier units are not equals", settings.getScaleMultiplierUnit().equals(otherSettings.getScaleMultiplierUnit()));
		assertEquals("Min steppings are not equals", settings.getMinStepping(), otherSettings.getMinStepping());
		assertEquals("Max steppings are not equals", settings.getMaxStepping(), otherSettings.getMaxStepping());
		assertEquals("Font sizes are not equals", settings.getFontSize(), otherSettings.getFontSize());
		assertEquals("Progress bar speeds are not equals", settings.getProgressSpeed(), otherSettings.getProgressSpeed());
		assertEquals("Time stampt switch are not equals", settings.showTimeStamps(), otherSettings.showTimeStamps());
		assertEquals("Linear modes are not equals", settings.isLinearMode(), otherSettings.isLinearMode());
	}
}
