/*
 * Created on 28.4.2005
 *
 */
package fi.helsinki.dacopan.ui.tsc;

import fi.helsinki.dacopan.AbstractTestCase;
import fi.helsinki.dacopan.model.DataView;
import fi.helsinki.dacopan.model.TransferUnit;
import fi.helsinki.dacopan.model.VariableDefinition;

/**
 * @author pamietti
 *
 * JUnit test case for NoticeTrigger.
 */
public class NoticeTriggerTestCase extends AbstractTestCase {

	DataView data;
	VariableDefinition def;
	/*
	 * @see TestCase#setUp()
	 */
	protected void setUp() throws Exception {
		data = getRandomData(10, 10, 10, 0.01f, false);
		def = new VariableDefinition("my own variable", VariableDefinition.UNIT, AbstractTestCase.IPv4);
	}

	/*
	 * Class under test for void NoticeTrigger(VariableDefinition, String, int, String)
	 */
	public void testNoticeTriggerVariableDefinitionStringintString() {
		NoticeTrigger notice = new NoticeTrigger(def, "123456", NoticeTrigger.EQUALS, "this is notice");
		assertTrue("notice is null", notice != null);
	}

	/*
	 * Class under test for void NoticeTrigger(NoticeTrigger)
	 */
	public void testNoticeTriggerNoticeTrigger() {
		NoticeTrigger notice = new NoticeTrigger(def, "123456", NoticeTrigger.EQUALS, "this is notice");
		NoticeTrigger notice2 = new NoticeTrigger(notice);
		assertEquals("notices are not same", notice.toString(), notice2.toString());
	}

	public void testTriggered() {
		TransferUnit unit = createUnit(0.0f, 0.1f);
		NoticeTrigger notice = new NoticeTrigger(def, "123456", NoticeTrigger.EQUALS, "this is notice");
		unit.addVariableValue(def, data.getHostById(1), "123456");
		assertTrue("notice is not triggered", notice.triggered(unit));
	}

	public void testSetAndGetNotice() {
		NoticeTrigger notice = new NoticeTrigger(def, "123456", NoticeTrigger.EQUALS, "this is notice");
		assertEquals("notice texts are not same", "this is notice", notice.getNotice());
		notice.setNotice("t onkin toinen notice");
		assertEquals("set notice texts are not same", "t onkin toinen notice", notice.getNotice());
	}

	public void testSetAndIsEnabled() {
		NoticeTrigger notice = new NoticeTrigger(def, "123456", NoticeTrigger.EQUALS, "this is notice");
		notice.setEnabled(true);
		assertTrue("notice is not enabled", notice.isEnabled());
		notice.setEnabled(false);
		assertTrue("notice is enabled", !notice.isEnabled());
	}

	public void testSetAndGetType() {
		NoticeTrigger notice = new NoticeTrigger(def, "123456", NoticeTrigger.EQUALS, "this is notice");
		assertEquals("notice type is not correct", NoticeTrigger.EQUALS, notice.getType());
		notice.setType(NoticeTrigger.SMALLER);
		assertEquals("set notice types are not same", NoticeTrigger.SMALLER, notice.getType());
	}

	public void testSetAndGetValue() {
		NoticeTrigger notice = new NoticeTrigger(def, "123456", NoticeTrigger.EQUALS, "this is notice");
		assertEquals("notice value is not correct", "123456", notice.getValue());
		notice.setValue("654321");
		assertEquals("set notice value is not correct", "654321", notice.getValue());
	}

	public void testSetAndGetVariable() {
		NoticeTrigger notice = new NoticeTrigger(def, "123456", NoticeTrigger.EQUALS, "this is notice");
		assertEquals("notice variable is not correct", def, notice.getVariable());
		VariableDefinition def2 = new VariableDefinition("vardef", VariableDefinition.UNIT_FIELD, data.getProtocolById(1));
		notice.setVariable(def2);
		assertEquals("set notice variable is not correct", def2, notice.getVariable());
	}

	/*
	 * Class under test for String toString()
	 */
	public void testToString() {
		NoticeTrigger notice = new NoticeTrigger(def, "123456", NoticeTrigger.EQUALS, "this is notice");
		notice.setEnabled(true);
		assertEquals("notice toString is not correct", "this is notice", notice.toString());
		notice.setEnabled(false);
		assertEquals("notice toString is not correct", "this is notice [disabled]", notice.toString());
	}

}
